#!/usr/bin/env python
#
# GEemyle: Configuration module
# Copyright (C) 2003-2006 Easter-eggs <info@easter-eggs.com>
# Authors:
#      Cyril Lacoux (Yack) <clacoux@easter-eggs.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

__author__  = "Cyril Lacoux [Yack] <clacoux@easter-eggs.com>"
__cvsid__   = "$Id$"
__version__ = "$Revision$"[11:-2]
__all__     = ["read", "write", "get", "set"]

import user
import os
import string
import ConfigParser

######################
# Unmodifiables values
######################

# Pixmaps dirpath
PIXMAPS_DIRPATH = None
for dirpath in ('.', '/usr/share/geemyle'):
    if os.path.isdir(dirpath) and os.path.isdir(os.path.join(dirpath, 'pixmaps')):
        PIXMAPS_DIRPATH = os.path.join(dirpath, 'pixmaps')
        break
if PIXMAPS_DIRPATH is None:
    raise 'Failed to get pixmaps dir'

APP_ICON_FILEPATH = os.path.join(PIXMAPS_DIRPATH, 'geemyle.png')

# Config file
CONFIG_FILE = '/etc/geemyle.conf'

# Temp
VERSION  = '0.2'
PACKAGE  = 'GEemyle'

# Terminal color schemes
COLOR_SCHEMES = {}
COLOR_SCHEMES['Black on light yellow'] = (
    ( 0x0000, 0x0000, 0x0000 ),
    ( 0xFFFF, 0xFFFF, 0xDDDD ) )

COLOR_SCHEMES['Black on white'] = (
    ( 0x0000, 0x0000, 0x0000 ),
    ( 0xFFFF, 0xFFFF, 0xFFFF ) )

COLOR_SCHEMES['Gray on black'] = (
    ( 0xAAAA, 0xAAAA, 0xAAAA ),
    ( 0x0000, 0x0000, 0x0000 ) )

COLOR_SCHEMES['Green on black'] = (
    ( 0x0000, 0xFFFF, 0x0000 ),
    ( 0x0000, 0x0000, 0x0000 ) )

COLOR_SCHEMES['White on black'] = (
    ( 0xFFFF, 0xFFFF, 0xFFFF ),
    ( 0x0000, 0x0000, 0x0000 ) )

COLOR_SCHEMES['Default'] = COLOR_SCHEMES['White on black']

# Terminal palette colors
PALETTE_SCHEMES = {}
PALETTE_SCHEMES['Linux'] = (
    ( 0x0000, 0x0000, 0x0000 ),
    ( 0xaaaa, 0x0000, 0x0000 ),
    ( 0x0000, 0xaaaa, 0x0000 ),
    ( 0xaaaa, 0x5555, 0x0000 ),
    ( 0x0000, 0x0000, 0xaaaa ),
    ( 0xaaaa, 0x0000, 0xaaaa ),
    ( 0x0000, 0xaaaa, 0xaaaa ),
    ( 0xaaaa, 0xaaaa, 0xaaaa ),
    ( 0x5555, 0x5555, 0x5555 ),
    ( 0xffff, 0x5555, 0x5555 ),
    ( 0x5555, 0xffff, 0x5555 ),
    ( 0xffff, 0xffff, 0x5555 ),
    ( 0x5555, 0x5555, 0xffff ),
    ( 0xffff, 0x5555, 0xffff ),
    ( 0x5555, 0xffff, 0xffff ),
    ( 0xffff, 0xffff, 0xffff ) )

PALETTE_SCHEMES['XTerm'] = (
    ( 0x0000, 0x0000, 0x0000 ),
    ( 0xcdcb, 0x0000, 0x0000 ),
    ( 0x0000, 0xcdcb, 0x0000 ),
    ( 0xcdcb, 0xcdcb, 0x0000 ),
    ( 0x1e1a, 0x908f, 0xffff ),
    ( 0xcdcb, 0x0000, 0xcdcb ),
    ( 0x0000, 0xcdcb, 0xcdcb ),
    ( 0xe5e2, 0xe5e2, 0xe5e2 ),
    ( 0x4ccc, 0x4ccc, 0x4ccc ),
    ( 0xffff, 0x0000, 0x0000 ),
    ( 0x0000, 0xffff, 0x0000 ),
    ( 0xffff, 0xffff, 0x0000 ),
    ( 0x4645, 0x8281, 0xb4ae ),
    ( 0xffff, 0x0000, 0xffff ),
    ( 0x0000, 0xffff, 0xffff ),
    ( 0xffff, 0xffff, 0xffff ) )

PALETTE_SCHEMES['RXvt']  = (
    ( 0x0000, 0x0000, 0x0000 ),
    ( 0xcdcd, 0x0000, 0x0000 ),
    ( 0x0000, 0xcdcd, 0x0000 ),
    ( 0xcdcd, 0xcdcd, 0x0000 ),
    ( 0x0000, 0x0000, 0xcdcd ),
    ( 0xcdcd, 0x0000, 0xcdcd ),
    ( 0x0000, 0xcdcd, 0xcdcd ),
    ( 0xfafa, 0xebeb, 0xd7d7 ),
    ( 0x4040, 0x4040, 0x4040 ),
    ( 0xffff, 0x0000, 0x0000 ),
    ( 0x0000, 0xffff, 0x0000 ),
    ( 0xffff, 0xffff, 0x0000 ),
    ( 0x0000, 0x0000, 0xffff ),
    ( 0xffff, 0x0000, 0xffff ),
    ( 0x0000, 0xffff, 0xffff ),
    ( 0xffff, 0xffff, 0xffff ) )

PALETTE_SCHEMES['Default'] = PALETTE_SCHEMES['Linux']


BOOLEANS = {}
INTS     = {}
FLOATS   = {}
EVALS    = {}

# General
BOOLEANS['geemyle'] = (
    'full_screen',
    'show_menubar',
    'show_toolbar' )

INTS['geemyle'] = (
    'asterisk_port', )

# Eemyle
BOOLEANS['eemyle'] = (
    'keep_u_p',
    'ssl',
    'use_first_hop',
    'use_ssh_compression' )

INTS['eemyle'] = (
    'port', )

# Terminal
BOOLEANS['terminal'] = (
    'use_system_font',
    'audible_bell',
    'visible_bell',
    'allow_bold',
    'cursor_blinks',
    'mouse_autohide',
    'use_theme_colors',
    'background_transparent',
    'show_scrollbar',
    'scroll_on_output',
    'scroll_on_keystroke',
    'use_prompt_for_window_title',
    'use_prompt_for_tab_label' )

INTS['terminal'] = (
    'scrollback_lines',
    'scroll_step',
    'backspace_binding',
    'delete_binding' )

FLOATS['terminal'] = (
    'background_saturation', )

EVALS['terminal'] = (
    'custom_color_fg',
    'custom_color_bg',
    'custom_palette_0',
    'custom_palette_1',
    'custom_palette_2',
    'custom_palette_3',
    'custom_palette_4',
    'custom_palette_5',
    'custom_palette_6',
    'custom_palette_7',
    'custom_palette_8',
    'custom_palette_9',
    'custom_palette_10',
    'custom_palette_11',
    'custom_palette_12',
    'custom_palette_13',
    'custom_palette_14',
    'custom_palette_15',
    'commands' )




Config    = ConfigParser.ConfigParser()
user_file = os.path.join(user.home, '.%src' % string.lower(PACKAGE))

def read():
    """
    Reads user's config file
    """
    global Config, user_file

    # Read system config file
    Config.read(CONFIG_FILE)

    # Read user config file
    if os.path.isfile(user_file):
        Config.read(user_file)

    # Color Scheme
    COLOR_SCHEMES['Custom'] = (get('terminal', 'custom_color_fg'),
                               get('terminal', 'custom_color_bg'))

    # Palette Color
    PALETTE_SCHEMES['Custom'] = []
    for i in range(0, 16):
        PALETTE_SCHEMES['Custom'].append(get('terminal', 'custom_palette_%d' % i))

    

def write():
    """
    Writes users settings to config file
    Diff between system and user
    """
    global Config, user_file
    
    # Read system config file
    SystemConfig = ConfigParser.ConfigParser()
    SystemConfig.read(CONFIG_FILE)

    # New User config file
    UserConfig = ConfigParser.ConfigParser()

    for section in SystemConfig.sections():
        if not Config.has_section(section):
            continue

        for option in SystemConfig.options(section):
            if not Config.has_option(section, option):
                continue

            current_value = Config.get(section, option)
            if not SystemConfig.get(section, option) == current_value:
                # Create the section if needed
                if not UserConfig.has_section(section):
                    UserConfig.add_section(section)

                # Store the value for section, option
                UserConfig.set(section, option, current_value)

    # Something to save ?
    if UserConfig.sections():
        try:
            file = open(user_file, 'w')
            UserConfig.write(file)
            file.close()
            os.chmod(user_file, 0600)
        except:
            return False

    elif os.path.isfile(user_file):
        try:
            os.remove(user_file)
        except:
            return False

    return True
    

def get(section, option):
    global Config

    value = None

    if BOOLEANS.has_key(section) and option in BOOLEANS[section]:
        value = Config.getboolean(section, option)

    elif INTS.has_key(section) and option in INTS[section]:
        value = Config.getint(section, option)

    elif FLOATS.has_key(section) and option in FLOATS[section]:
        value = Config.getfloat(section, option)

    else:
        value = Config.get(section, option)

        if EVALS.has_key(section) and option in EVALS[section]:
            value = eval(value)

    if value == 'None':
        value = eval(value)

    return value

def set(section, option, value):
    global Config

    Config.set(section, option, str(value))
