#!/usr/bin/env python
#
# GEemyle: Eemyle View
# Copyright (C) 2003-2006 Easter-eggs <info@easter-eggs.com>
# Authors:
#      Cyril Lacoux (Yack) <clacoux@easter-eggs.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

__author__  = "Cyril Lacoux [Yack] <clacoux@easter-eggs.com>"
__cvsid__   = "$Id$"
__version__ = "$Revision$"[11:-2]
__all__     = ["ContactsView"]

import pygtk
pygtk.require('2.0')
import gtk
import gobject
import os

import geemyle.config    as config
import geemyle.gui.mygtk as mygtk
import geemyle.functions as functions
import geemyle.gettext   as gettext
import geemyle.helpers   as helpers

_ = gettext._

CONTACTS_TV_COLUMNS  = ( (_('Name'),             gtk.CellRendererText,   (),                    0,        False,        0),
                         (_('Surname'),          gtk.CellRendererText,   (),                    1,        False,        1),
                         (_('Formatted name'),   gtk.CellRendererText,   (),                    2,        True,         2),
                         (_('Mail'),             gtk.CellRendererText,   (),                    3,        False,        3),
                         (_('Phone'),            gtk.CellRendererText,   (),                    4,        False,        4),
                         (_('Mobile'),           gtk.CellRendererText,   (),                    5,        False,        5),
                         (_('Fax'),              gtk.CellRendererText,   (),                    6,        False,        6) )

def new_callback_button(image_filename, callback):
    """
    Generate a new gtk.Button with image. 
    Callback will be called on clicked event.
    """
    image = gtk.image_new_from_file(os.path.join(config.PIXMAPS_DIRPATH, image_filename))
    button = gtk.Button()
    button.set_image(image)
    button.set_relief(gtk.RELIEF_NONE)
    button.connect('clicked', callback)
    return button

class ContactView(gtk.Frame):

    def __init__(self):

        gtk.Frame.__init__(self, 'a')

        self.formatted_name = gtk.Label()
        self.mail_address   = gtk.Label()
        self.phone_number   = gtk.Label()
        self.mobile_number  = gtk.Label()
        self.fax_number     = gtk.Label()

        self.construct_frame()
        self.show_all()

    def construct_frame(self):

        self.get_label_widget().set_markup(('<b>%s</b>' % _('Details')))

        table = gtk.Table()
        table.set_border_width(5)
        table.set_row_spacings(5)
        table.set_col_spacings(5)

        i  = 0
        self.formatted_name.set_alignment(0, 0.5)
        self.formatted_name.set_selectable(True)
        label = gtk.Label(_('Formatted name:'))
        label.set_alignment(0, 0.5)
        table.attach(label,                1, 2, i, i+1)
        table.attach(self.formatted_name,  2, 3, i, i+1)

        i += 1
        label = gtk.Label(_('Mail address:'))
        label.set_alignment(0, 0.5)
        self.mail_address.set_selectable(True)
        button = new_callback_button('email.png', lambda button: helpers.mail_to(self.mail_address.get_text()))
        table.attach(label,                1, 2, i, i+1)
        table.attach(self.mail_address,    2, 3, i, i+1)
        table.attach(button,               3, 4, i, i+1)

        i += 1
        label = gtk.Label(_('Phone number:'))
        label.set_alignment(0, 0.5)
        self.phone_number.set_selectable(True)
        button = new_callback_button('phone.png', lambda button: helpers.phone_call(self.phone_number.get_text()))
        table.attach(label,                1, 2, i, i+1)
        table.attach(self.phone_number,    2, 3, i, i+1)
        table.attach(button,               3, 4, i, i+1)

        i += 1
        label = gtk.Label(_('Mobile number:'))
        label.set_alignment(0, 0.5)
        self.mobile_number.set_selectable(True)
        button = new_callback_button('phone.png', lambda button: helpers.phone_call(self.mobile_number.get_text()))
        table.attach(label,                1, 2, i, i+1)
        table.attach(self.mobile_number,   2, 3, i, i+1)
        table.attach(button,               3, 4, i, i+1)

        i += 1
        label = gtk.Label(_('Fax number:'))
        label.set_alignment(0, 0.5)
        self.fax_number.set_selectable(True)
        button = new_callback_button('fax.png', lambda button: helpers.send_fax(self.fax_number.get_text()))
        table.attach(label,                1, 2, i, i+1)
        table.attach(self.fax_number,      2, 3, i, i+1)
        table.attach(button,               3, 4, i, i+1)

        self.add(table)

    def set_contact(self, contact):
        self.formatted_name.set_text(contact[0])
        self.mail_address.set_text(contact[1])
        self.phone_number.set_text(contact[2])
        self.mobile_number.set_text(contact[3])
        self.fax_number.set_text(contact[4])

class ContactsView(gtk.Dialog):

    def __init__(self, contacts):
        
        gtk.Dialog.__init__(self, flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT, buttons=(gtk.STOCK_CLOSE, gtk.RESPONSE_CANCEL))

        self.set_title('%s - contacts list for %s' % (config.PACKAGE, contacts[0]['surname']))
        self.set_icon_from_file(config.APP_ICON_FILEPATH)

        self.contacts = contacts

        # Contact's list
        model = gtk.ListStore(gobject.TYPE_STRING,     # Name
                              gobject.TYPE_STRING,     # Surname
                              gobject.TYPE_STRING,     # Formatted name
                              gobject.TYPE_STRING,     # Mail
                              gobject.TYPE_STRING,     # Phone
                              gobject.TYPE_STRING,     # Mobile
                              gobject.TYPE_STRING)     # Fax

        modelfilter = model.filter_new()
        modelfilter.set_visible_func(self.contacts_visible_func)
        modelsort   = gtk.TreeModelSort(modelfilter)

        self.tv_contacts = gtk.TreeView(modelsort)

        # Contact's view
        self.contact_view = ContactView()

        self.construct_frame()
        self.feed_objects()

    def construct_frame(self):

        hbox = gtk.HBox()
        hbox.set_spacing(10)
        hbox.set_border_width(10)

        # Contact's list
        self.tv_contacts.set_rules_hint(True)
        self.tv_contacts.set_search_column(2)
        self.tv_contacts.set_headers_visible(False)
        self.tv_contacts.connect('cursor-changed',  self.on_tv_contacts_cursor_changed)

        mygtk.add_columns_to_treeview(self.tv_contacts, CONTACTS_TV_COLUMNS)

        sw = gtk.ScrolledWindow()
        sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        sw.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        sw.add(self.tv_contacts)

        hbox.pack_start(sw, expand=False)

        # Contact's view
        vbox = gtk.VBox()
        vbox.pack_start(self.contact_view, expand=False)
        hbox.pack_start(vbox)

        self.vbox.pack_start(hbox)

    def feed_objects(self):

        # Feeding the contacts list
        modelsort   = self.tv_contacts.get_model()
        modelfilter = modelsort.get_model()
        model       = modelfilter.get_model()

        for contact in self.contacts:
            # Stripping datas
            for key in contact.keys():
                contact[key] = contact[key].strip()

            if contact['email'].endswith('unknown.tld'):
                contact['email'] = ''

            formatted_name = '%(surname)s %(first_name)s' % contact

            iter = model.append((contact['first_name'],                             # Name
                                 contact['surname'],                                # Surname
                                 formatted_name,                                    # Formatted name
                                 contact['email'],                                  # Mail
                                 functions.convert_phone_number(contact['phone']),  # Phone number
                                 functions.convert_phone_number(contact['mobile']), # Mobile number
                                 functions.convert_phone_number(contact['fax'])))   # Fax number

    def contacts_visible_func(self, model, iter):
        
        return True
    
    def on_tv_contacts_cursor_changed(self, treeview):
        
        path  = treeview.get_cursor()[0]
        model = treeview.get_model()
        iter  = model.get_iter(path)

        self.contact_view.set_contact(model.get(iter, 2, 3, 4, 5, 6))
