#!/usr/bin/env python
#
# GEemyle: Eemyle View
# Copyright (C) 2003-2006 Easter-eggs <info@easter-eggs.com>
# Authors:
#      Cyril Lacoux (Yack) <clacoux@easter-eggs.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

__author__  = "Cyril Lacoux [Yack] <clacoux@easter-eggs.com>"
__cvsid__   = "$Id$"
__version__ = "$Revision$"[11:-2]
__all__     = ["AuthDialog", "TunnelDialog", "ConfirmDialog"]


import pygtk
pygtk.require('2.0')
import gtk

import geemyle.config as config
import geemyle.gettext as gettext

_ = gettext._


class AuthDialog(gtk.Dialog):

    def __init__(self, username, password, keep_u_p):

        gtk.Dialog.__init__(self, title=_('Authentication'), flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                                  buttons=(gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))

        self.set_icon_from_file(config.APP_ICON_FILEPATH)

        self.username = username
        self.password = password
        self.keep_u_p = keep_u_p

        self.construct_frame()

    def construct_frame(self):
    
        self.set_has_separator(True)

        table = gtk.Table()
        table.set_border_width(10)
        table.set_row_spacings(10)
        table.set_col_spacings(10)

        en_username = gtk.Entry()
        en_username.set_text(self.username)
        en_username.connect('changed', self.on_username_changed)
        en_username.set_activates_default(True)

        en_password = gtk.Entry()
        en_password.set_visibility(False)
        en_password.set_text(self.password)
        en_password.connect('changed', self.on_password_changed)
        en_password.set_activates_default(True)

        cb_keep_u_p = gtk.CheckButton(_('Keep username and password'))
        cb_keep_u_p.set_active(self.keep_u_p)
        cb_keep_u_p.connect('toggled', self.on_keep_u_p_toggled)

        i = 0
        label = gtk.Label(_('Authentication require on server'))
        label.set_alignment(0.0, 0.5)
        table.attach(label,       0, 2, i, i+1)

        i += 1
        label = gtk.Label(_('Username'))
        label.set_alignment(0.0, 0.5)
        table.attach(label,       0, 1, i, i+1)
        table.attach(en_username, 1, 2, i, i+1)

        i += 1
        label = gtk.Label(_('Password'))
        label.set_alignment(0.0, 0.5)
        table.attach(label,       0, 1, i, i+1)
        table.attach(en_password, 1, 2, i, i+1)

        i += 1
        table.attach(cb_keep_u_p, 0, 2, i, i+1)

        self.vbox.pack_start(table)

    def on_username_changed(self, entry):

        self.username = entry.get_text()

    def on_password_changed(self, entry):
        
        self.password = entry.get_text()

    def on_keep_u_p_toggled(self, check_button):
        
        self.keep_u_p = check_button.get_active()



class TunnelDialog(gtk.Dialog):
    
    def __init__(self):

        gtk.Dialog.__init__(self, title=_('Tunnel parameters'), flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                                  buttons=(gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))

        self.set_icon_from_file(config.APP_ICON_FILEPATH)

        self.tunnel_data  = { 'tunnel':       True,
                              'local_port':   8080,
                              'remote_host':  'localhost',
                              'remote_port':  80,
                              'allow_remote': False,
                              'compression':  False }

        self.construct_frame()


    def construct_frame(self):

        self.set_has_separator(True)

        table = gtk.Table()
        table.set_border_width(10)
        table.set_row_spacings(10)
        table.set_col_spacings(10)

        adj = gtk.Adjustment(0, 1024, 65534, 1, 1, 0)
        adj.set_value(self.tunnel_data['local_port'])
        adj.connect('value-changed', self.on_something_changed, 'local_port')

        sb_local_port = gtk.SpinButton(adj)
        sb_local_port.set_activates_default(True)

        en_remote_host = gtk.Entry()
        en_remote_host.set_text(self.tunnel_data['remote_host'])
        en_remote_host.connect('changed', self.on_something_changed, 'remote_host')
        en_remote_host.set_activates_default(True)

        adj = gtk.Adjustment(0, 1,   65534, 1, 1, 0)
        adj.set_value(self.tunnel_data['remote_port'])
        adj.connect('value-changed', self.on_something_changed, 'remote_port')

        sb_remote_port = gtk.SpinButton(adj)
        sb_remote_port.set_activates_default(True)

        cb_allow_remote = gtk.CheckButton(_('Allows remote hosts to connect to local forwarded ports'))
        cb_allow_remote.set_active(self.tunnel_data['allow_remote'])
        cb_allow_remote.connect('toggled', self.on_something_changed, 'allow_remote')

        cb_compression = gtk.CheckButton(_('Enable compression'))
        cb_compression.set_active(self.tunnel_data['compression'])
        cb_compression.connect('toggled', self.on_something_changed, 'compression')

        i = 0
        label = gtk.Label(_('Local port'))
        label.set_alignment(0.0, 0.5)
        table.attach(label,           0, 1, i, i+1)
        table.attach(sb_local_port,   1, 2, i, i+1)

        i += 1
        label = gtk.Label(_('Remote host'))
        label.set_alignment(0.0, 0.5)
        table.attach(label,           0, 1, i, i+1)
        table.attach(en_remote_host,  1, 2, i, i+1)

        i += 1
        label = gtk.Label(_('Remote port'))
        label.set_alignment(0.0, 0.5)
        table.attach(label,           0, 1, i, i+1)
        table.attach(sb_remote_port,  1, 2, i, i+1)

        i += 1
        table.attach(cb_allow_remote, 0, 2, i, i+1)

        i += 1
        table.attach(cb_compression,  0, 2, i, i+1)
        
        i += 1
        label = gtk.Label()
        label.set_markup('<b><i>%s</i></b>' % _('You must use a "local port" that is free on each hops used to connect to this server!'))
        label.set_alignment(0.0, 0.5)
        table.attach(label,           0, 2, i, i+1)

        self.vbox.pack_start(table)

    def on_something_changed(self, widget, key):
        
        if isinstance(widget, gtk.Adjustment):
            value = int(widget.get_value())

        elif isinstance(widget, gtk.Entry):
            value = widget.get_text()

        elif isinstance(widget, gtk.CheckButton):
            value = widget.get_active()

        else:
            value = None

        self.tunnel_data[key] = value

class ConfirmDialog(gtk.MessageDialog):
    
    def __init__(self, message_format):
        
        gtk.MessageDialog.__init__(self, flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                                   type=gtk.MESSAGE_QUESTION,
                                   buttons=gtk.BUTTONS_OK_CANCEL,
                                   message_format=message_format)

        self.set_title(config.PACKAGE)
        self.realize()
        self.set_icon_from_file(config.APP_ICON_FILEPATH)

