#!/usr/bin/env python
#
# GEemyle: Eemyle View
# Copyright (C) 2003-2006 Easter-eggs <info@easter-eggs.com>
# Authors:
#      Cyril Lacoux (Yack) <clacoux@easter-eggs.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

__author__  = "Cyril Lacoux [Yack] <clacoux@easter-eggs.com>"
__cvsid__   = "$Id$"
__version__ = "$Revision$"[11:-2]
__all__     = ["load_config", "get_connect_data", "EemyleView"]


import pygtk
pygtk.require('2.0')
import gtk
import gobject
import re

import geemyle.eemyle_connector as connector
import geemyle.helpers          as helpers
import geemyle.config           as config
import geemyle.functions        as functions
import geemyle.gettext          as gettext
import geemyle.gui.mygtk        as mygtk

import contacts
import server
import dialogs
import statistics
from server import common

_ = gettext._

# For load configuration
_eemyle_view = None

                          # Name                 # Renderer              # Renderer attributes  # col_id  # visibility # sort_id
CUSTOMERS_TV_COLUMNS = ( (_('Eeid'),             gtk.CellRendererText,   (),                    0,        True,        0),
                         (_('Has server'),       gtk.CellRendererToggle, (),                    1,        False,       1),
                         (_('Name'),             gtk.CellRendererText,   (),                    2,        True,        2))
                         #(_('Address'),          gtk.CellRendererText,   (),                    3,        True,        3),
                         #(_('City code'),        gtk.CellRendererText,   (('xalign', 1.0), ),   4,        True,        4),
                         #(_('City'),             gtk.CellRendererText,   (),                    5,        True,        5),
                         #(_('Phone'),            gtk.CellRendererText,   (),                    6,        False,       6),
                         #(_('Fax'),              gtk.CellRendererText,   (),                    7,        False,       7) )

CUSTOMERS_HAS_SERVER_COLUMN_ID = 1

SERVERS_TV_COLUMNS   = ( (_('Server ID'),        gtk.CellRendererText,   (),                    0,        False,       0),
                         (_('Eeid'),             gtk.CellRendererText,   (),                    1,        True,        1),
                         (_('Name'),             gtk.CellRendererText,   (),                    2,        True,        2),
                         (_('Operating system'), gtk.CellRendererText,   (),                    3,        True,        3),
                         (_('Usage'),             gtk.CellRendererText, (),                    4,        True,        4),
                         (_('Security'),         gtk.CellRendererToggle, (),                    5,        True,        5),
                         ('last_eepdate',        gtk.CellRendererText,   (),                    6,        False,       6),
                         (_('Last eepdate'),     gtk.CellRendererText,   (('xalign', 1.0), ),   7,        True,        6),
                         ('updates',             gtk.CellRendererText,   (),                    8,        False,       8),
                         (_('Updates'),          gtk.CellRendererText,   (('xalign', 1.0), ),   9,        True,        9),
                         (_('Dead'),            gtk.CellRendererToggle,   (),                   10,        True,       10) )

SERVERS_TV_DEAD_COLUMN_ID = 10

def load_config():
    """
    Load config for all Eemyle views
    """
    global _eemyle_view

    _eemyle_view.load_config()

def get_connect_data(server_id):
    """
    """
    global _eemyle_view

    return _eemyle_view.connect_data[server_id]



class EemyleView(gtk.VBox):

    def __init__(self, geemyle_ui):

        gtk.VBox.__init__(self)

        # The ui
        self.geemyle_ui = geemyle_ui

        # Connector
        connector.eemyle_connector.connect('xmlrpc-data-received', self.on_xmlrpc_data_received)

        # To get extra connection data
        self.connect_data = {}

        # Customers
        model = gtk.ListStore(gobject.TYPE_STRING,     # Eeid
                              gobject.TYPE_BOOLEAN,    # Has server
                              gobject.TYPE_STRING,     # Customer name
                              gobject.TYPE_STRING,     # Street
                              gobject.TYPE_STRING,     # City code
                              gobject.TYPE_STRING,     # City
                              gobject.TYPE_STRING,     # Phone number
                              gobject.TYPE_STRING)     # Fax number

        modelfilter = model.filter_new()
        modelfilter.set_visible_func(self.customers_visible_func)
        modelsort   = gtk.TreeModelSort(modelfilter)

        self.tv_customers = gtk.TreeView(modelsort)

        # Search
        self.en_customers = gtk.Entry()
        self.en_customers.set_width_chars(10)
        self.en_customers.connect('changed', self.on_customers_refilter_activate)

        self.cb_customers = gtk.combo_box_new_text()
        self.cb_customers.connect('changed', self.on_customers_refilter_activate)

        self.cb_no_server = gtk.CheckButton(_('View all customers'))
        self.cb_no_server.connect('toggled', self.on_customers_refilter_activate)

        # Servers
        model = gtk.ListStore(gobject.TYPE_INT,        # Server ID
                              gobject.TYPE_STRING,     # Eeid
                              gobject.TYPE_STRING,     # Server name
                              gobject.TYPE_STRING,     # OS
                              gobject.TYPE_STRING,    # Dead
                              gobject.TYPE_BOOLEAN,    # Security maintenance
                              gobject.TYPE_INT,        # last_eepdate
                              gobject.TYPE_STRING,     # Last eepdate
                              gobject.TYPE_INT,        # updates
                              gobject.TYPE_STRING,     # Updates
                              gobject.TYPE_BOOLEAN)     # Usage

        modelfilter = model.filter_new()
        modelfilter.set_visible_func(self.servers_visible_func)
        modelsort   = gtk.TreeModelSort(modelfilter)

        self.tv_servers = gtk.TreeView(modelsort)

        # Search
        self.en_servers = gtk.Entry()
        self.en_servers.connect('changed', self.on_servers_refilter_activate)
        self.en_servers.set_width_chars(10)

        self.cb_servers = gtk.combo_box_new_text()
        self.cb_servers.connect('changed', self.on_servers_refilter_activate)

        self.cb_dead    = gtk.CheckButton(_('View dead servers'))
        self.cb_dead.connect('toggled', self.on_servers_refilter_activate)

        # Buttons
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/ServerRemove').set_sensitive(False)
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/ServerAdd').set_sensitive(False)
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/ServerDuplicate').set_sensitive(False)
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/TerminalIPMI').set_sensitive(False)
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/TerminalX11').set_sensitive(False)
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/TerminalTunnel').set_sensitive(False)
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/TerminalConnect').set_sensitive(False)
        self.geemyle_ui.ui.get_widget('/Menubar/ConnectionMenu').set_sensitive(False)

        # Server View
        self.server_view = server.ServerView()
        self.server_view.tickets.set_url_callback(self.ticket_url_callback)

        self.load_config()

        self.feed_objects()
        self.construct_frame()

        # Initializing communication with Eemyle
        self.on_bt_display_list_refresh_clicked()

    def feed_objects(self):
        
        self.cb_customers.append_text(_('All'))
        for criteria, renderer, renderer_properties, column_id, visible, sort_id in CUSTOMERS_TV_COLUMNS:
            if visible:
                self.cb_customers.append_text(criteria)

        self.cb_customers.set_active(0)

        self.cb_servers.append_text(_('All'))
        for criteria, renderer, renderer_properties, column_id, visible, sort_id in SERVERS_TV_COLUMNS:
            if visible:
                self.cb_servers.append_text(criteria)

        self.cb_servers.set_active(0)

    def construct_frame(self):
        global _eemyle_view

        # Adding the Eemyle view to all Eemyle views
        _eemyle_view = self

        self.set_spacing(5)
        self.set_border_width(5)

        hpaned = gtk.HPaned()
        vpaned = gtk.VPaned()

        #######################################################################
        # Customers view
        self.tv_customers.set_rules_hint(True)
        self.tv_customers.set_search_column(1)
        self.tv_customers.set_headers_visible(True)
        self.tv_customers.set_headers_clickable(True)
        self.tv_customers.connect('cursor-changed',  self.on_tv_customers_cursor_changed)

        mygtk.add_columns_to_treeview(self.tv_customers, CUSTOMERS_TV_COLUMNS)

        sw = gtk.ScrolledWindow()
        sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        sw.add(self.tv_customers)

        # Force widgets to have the same size
        sg = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
        sg.add_widget(self.cb_customers)
        sg.add_widget(self.cb_servers)

        # Customers search
        hbox = gtk.HBox()
        hbox.set_spacing(10)

        label = gtk.Label(_('Search:'))
        label.set_alignment(0.0, 0.5)
        hbox.pack_start(label,             expand=False)
        hbox.pack_start(self.en_customers, expand=False)

        label = gtk.Label(_('on'))
        label.set_alignment(0.0, 0.5)
        hbox.pack_start(label,             expand=False)
        alg = gtk.Alignment(0.5, 0.5)
        alg.add(self.cb_customers)
        hbox.pack_start(alg,               expand=False)
        hbox.pack_start(self.cb_no_server, expand=False)

        vbox = gtk.VBox()
        vbox.set_border_width(5)
        vbox.set_spacing(5)
        vbox.pack_start(sw)
        vbox.pack_start(hbox, expand=False)

        frame = gtk.Frame('a')
        frame.set_border_width(5)
        frame.get_label_widget().set_markup(('<b>%s</b>' % _('Customers')))
        frame.add(vbox)

        hpaned.pack1(frame, resize=True)

        #######################################################################
        # Servers view
        selection = self.tv_servers.get_selection()
        selection.set_mode(gtk.SELECTION_MULTIPLE)
        selection.connect('changed', self.on_tv_servers_selection_changed)

        self.tv_servers.set_rules_hint(True)
        self.tv_servers.set_search_column(1)
        self.tv_servers.set_headers_visible(True)
        self.tv_servers.set_headers_clickable(True)
        self.tv_servers.connect('cursor-changed', self.on_tv_servers_cursor_changed)
        self.tv_servers.connect('row-activated',  self.on_tv_servers_row_activated)

        mygtk.add_columns_to_treeview(self.tv_servers, SERVERS_TV_COLUMNS)

        sw = gtk.ScrolledWindow()
        sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        sw.add(self.tv_servers)

        # Servers search
        hbox = gtk.HBox()
        hbox.set_spacing(10)

        label = gtk.Label(_('Search:'))
        label.set_alignment(0.0, 0.5)
        hbox.pack_start(label,           expand=False)
        hbox.pack_start(self.en_servers, expand=False)

        label = gtk.Label(_('on'))
        label.set_alignment(0.0, 0.5)
        hbox.pack_start(label,           expand=False)
        alg = gtk.Alignment(0.5, 0.5)
        alg.add(self.cb_servers)
        hbox.pack_start(alg,             expand=False, fill=False)
        hbox.pack_start(self.cb_dead,    expand=False)
        
        vbox = gtk.VBox()
        vbox.set_border_width(5)
        vbox.set_spacing(5)
        vbox.pack_start(sw)
        vbox.pack_start(hbox, expand=False)

        frame = gtk.Frame('a')
        frame.set_border_width(5)
        frame.get_label_widget().set_markup(('<b>%s</b>' % _('Servers')))
        frame.add(vbox)
        hpaned.pack2(frame, resize=True)

        #######################################################################
        # Current server view
        vpaned.pack1(hpaned,           resize=True)
        vpaned.pack2(self.server_view, resize=True)

        self.pack_start(vpaned)

        self.show_all()
        
    def load_config(self):
        """
        Load config for Eemyle
        """

        hostname = config.get('eemyle', 'hostname')
        keep_u_p = config.get('eemyle', 'keep_u_p')

        if keep_u_p:
            username = config.get('eemyle', 'username')
            password = config.get('eemyle', 'password')
        else:
            username = ''
            password = ''

        wservice = config.get('eemyle', 'wservice')
        port     = config.get('eemyle', 'port')
        ssl      = config.get('eemyle', 'ssl')

        if ssl:
            scheme = 'https'
        else:
            scheme = 'http'
        
        connector.eemyle_connector.set_hostname(hostname)
        connector.eemyle_connector.set_wservice(wservice)
        connector.eemyle_connector.set_username(username)
        connector.eemyle_connector.set_password(password)
        connector.eemyle_connector.set_scheme(scheme)
        connector.eemyle_connector.set_port(str(port))

        del(hostname)
        del(username)
        del(password)
        del(wservice)
        del(port)
        del(ssl)

    def on_bt_display_list_refresh_clicked(self, button=None):

        # Buttons
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/ServerAdd').set_sensitive(False)
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/CustomerRefresh').set_sensitive(False)
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/CustomerViewContacts').set_sensitive(False)

        # Set the cursor working
        self.geemyle_ui.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
        self.geemyle_ui.sb_geemyle.pop(0)
        self.geemyle_ui.sb_geemyle.push(0, _('Getting data from Eemyle...'))

        # Getting all display lists from Eemyle
        connector.eemyle_connector.send_request('eemyle.get_display_lists')

    def on_bt_customers_refresh_clicked(self, button=None):

        # Buttons
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/ServerAdd').set_sensitive(False)
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/CustomerRefresh').set_sensitive(False)
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/CustomerViewContacts').set_sensitive(False)

        # Set the cursor working
        self.geemyle_ui.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
        self.geemyle_ui.sb_geemyle.pop(0)
        self.geemyle_ui.sb_geemyle.push(0, _('Getting customers\' list from Eemyle... '))

        # Getting Cutomers' list from Eemyle
        connector.eemyle_connector.send_request('eemyle.get_customers_having_server')

    def on_bt_view_contacts_clicked(self, button=None):
        
        # Set the cursor working
        self.geemyle_ui.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
        self.geemyle_ui.sb_geemyle.pop(0)
        self.geemyle_ui.sb_geemyle.push(0, _('Getting contacts list from Eemyle...'))

        customer_id, path = self.get_current_customer_id()
        connector.eemyle_connector.send_request('eemyle.get_contacts_by_customer_id', str(customer_id))

    def stop_connector(self):
        
        connector.stop()

    def on_xmlrpc_data_received(self, eemyle_connector, function_name, result, data):

        gtk.gdk.threads_enter()

        # Set the cursor standard
        self.geemyle_ui.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.LEFT_PTR))
        self.geemyle_ui.sb_geemyle.pop(0)

        # Buttons
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/CustomerRefresh').set_sensitive(True)

        # Error
        if not result:
            self.geemyle_ui.sb_geemyle.push(0, _('Failed to get data from server, function: %s, reason was: %s') % (function_name, data.errmsg.lower()))

            if data.errcode == 401:
                keep_u_p = config.get('eemyle', 'keep_u_p')
                if keep_u_p:
                    username = config.get('eemyle', 'username')
                    password = config.get('eemyle', 'password')
                else:
                    username = ''
                    password = ''

                dialog = dialogs.AuthDialog(username, password, keep_u_p)
                dialog.set_default_response(gtk.RESPONSE_OK)
                dialog.realize()
                dialog.set_transient_for(self.geemyle_ui)
                dialog.show_all()
                response = dialog.run()
                dialog.destroy()

                if response == gtk.RESPONSE_OK:
                    username = dialog.username
                    password = dialog.password
                    keep_u_p = dialog.keep_u_p

                    if keep_u_p:
                        config.set('eemyle', 'username', username)
                        config.set('eemyle', 'password', password)
                    else:
                        config.set('eemyle', 'username', '')
                        config.set('eemyle', 'password', '')
                    config.set('eemyle', 'keep_u_p', keep_u_p)
                    config.write()

                    eemyle_connector.set_username(username)
                    eemyle_connector.set_password(password)

                    # Recall the function
                    if function_name == 'eemyle.get_display_lists':
                        self.on_bt_display_list_refresh_clicked()

                    elif function_name == 'eemyle.get_customers_having_server':
                        self.on_bt_customers_refresh_clicked()

                    elif function_name == 'eemyle.get_contacts_by_customer_id':
                        self.on_bt_view_contacts_clicked()

                    elif function_name == 'eemyle.get_servers_by_customer_id':
                        self.on_tv_customers_cursor_changed()

                    elif function_name == 'eemyle.get_server_by_server_id':
                        self.on_tv_servers_cursor_changed()

                    elif function_name == 'eemyle.get_server_traceroute_by_server_id':
                        server_ids = self.get_selected_server_ids()
                        for server_id in server_ids:
                            eemyle_connector.send_request('eemyle.get_server_traceroute_by_server_id', str(server_id))

                    del(username)
                    del(password)

                del(dialog)

        # Connector errors
        elif type(data) == int and not data in [0, 1]:
            if data == connector.ERROR_UNKNOWN:
                self.geemyle_ui.sb_geemyle.push(0, _('Unknown error while performing action'))

            elif data == connector.ERROR_ACCES_DENIED:
                self.geemyle_ui.sb_geemyle.push(0, _('You don\'t have sufficient access to perform this action'))

            elif data == connector.ERROR_TRACEROUTE_NO_SERVER:
                self.geemyle_ui.sb_geemyle.push(0, _('Missing server while getting route to server'))

            elif data == connector.ERROR_TRACEROUTE_NO_ROUTE:
                self.geemyle_ui.sb_geemyle.push(0, _('Failed to get route to server'))

        # Internal display lists
        elif function_name == 'eemyle.get_display_lists':
            
            if data:
                self.server_view.set_display_lists(data)
                self.on_bt_customers_refresh_clicked()

        # Customers list
        elif function_name == 'eemyle.get_customers_having_server':

            # Flushing the liststore
            modelsort   = self.tv_customers.get_model()
            modelfilter = modelsort.get_model()
            model       = modelfilter.get_model()
            model.clear()

            if data:
                # Adding the special customer for getting all the servers
                iter = model.append(('', True,  _('All customers'), '', '', '', '', ''))

                # Feeding the liststore
                for customer in data:
                    iter = model.append((customer['eeid'],                                          # Eeid
                                         customer['has_server'],                                    # Has server
                                         customer['name'].strip(),                                  # Name
                                         customer['street'].strip(),                                # Street
                                         customer['city_code'].strip(),                             # City code
                                         customer['city'].strip(),                                  # City
                                         functions.convert_phone_number(customer['phone'].strip()), # Phone number
                                         functions.convert_phone_number(customer['fax'].strip())))  # Fax number

                self.tv_customers.columns_autosize()

            else:
                self.geemyle_ui.sb_geemyle.push(0, _('An error occured while receiving customers list'))


        # Contacts list
        elif function_name == 'eemyle.get_contacts_by_customer_id':
            if data:
                dialog = contacts.ContactsView(data)
                #FIXME: doesn't work: dialog.set_default_response(gtk.RESPONSE_OK)
                dialog.realize()
                dialog.set_transient_for(self.geemyle_ui)
                dialog.show_all()
                dialog.run()
                dialog.destroy()

            else:
                self.geemyle_ui.sb_geemyle.push(0, _('An error occured while receiving contacts list'))

        # Servers list
        elif function_name == 'eemyle.get_servers_by_customer_id':
            # Flushing the liststore
            modelsort   = self.tv_servers.get_model()
            modelfilter = modelsort.get_model()
            model       = modelfilter.get_model()
            model.clear()

            if data:
                # Feeding the liststore
                for server in data:
                    if server['dead'] == 'f':
                        dead = 0
                    else:
                        dead = 1

                    if server['security_check'] == 'f':
                        security_check = 0
                    else:
                        security_check = 1

                    # Last time we had the list of available updates
                    last_eepdate = server['last_eepdate']
                    if last_eepdate:
                        int_last_eepdate = int(last_eepdate)
                        if int_last_eepdate == 0:
                            str_last_eepdate = _('today')
                        elif int_last_eepdate == 1:
                            str_last_eepdate = _('yesterday')
                        else:
                            str_last_eepdate = _('%d days ago') % int_last_eepdate
                    else:
                        int_last_eepdate = 99999999
                        str_last_eepdate = _('unknown')

                    # Number of package to update
                    int_updates = int(server['updates'])
                    if int_updates == 0:
                        if int_last_eepdate == 99999999:
                            int_updates = 99999999
                            str_updates = _('unknown')
                        else:
                            str_updates = _('up to date')
                    else:
                        str_updates = _('%s package(s)') % int_updates

                    # Add (v) to indicate this is a virtual server if needed
                    str_server_name = server['server_name']
                    if not server['id_server_physical_host'] == "0":
                        str_server_name += _(' (v)')

                    iter = model.append((int(server['id_server']),              # Server ID
                                         server['id_customer'],                 # Eeid
                                         str_server_name,                       # Server name
                                         server['os_name'],                     # OS
                                         server['usage'].replace('\r\n', ','),  # Usage
                                         security_check,                        # Security maintenance
                                         int_last_eepdate,                      # last_eepdate
                                         str_last_eepdate,                      # Last eepdate
                                         int_updates,                           # updates
                                         str_updates,                           # Updates
                                         dead ))                                # Dead

                self.tv_servers.columns_autosize()

            else:
                self.geemyle_ui.sb_geemyle.push(0, _('An error occured while receiving servers list'))


        # Server details
        elif function_name == 'eemyle.get_server_by_server_id':
            if data:
                self.server_view.set_server(data)
            else:
                self.geemyle_ui.sb_geemyle.push(0, _('An error occured while receiving server details'))

        # Tickets
        elif function_name == 'eemyle.get_tickets_by_server_id':
            self.server_view.set_tickets(data)

        # Server Traceroute
        elif function_name == 'eemyle.get_server_traceroute_by_server_id':
            if data:
                self.geemyle_ui.terminal_connect(data)
            else:
                self.geemyle_ui.sb_geemyle.push(0, _('An error occured while receiving server traceroute'))

        # Server IPMI infos
        elif function_name == 'eemyle.get_server_ipmi_infos_by_server_id':
            if data:
                self.on_server_ipmi_infos_received(data)
            else:
                self.geemyle_ui.sb_geemyle.push(0, _('An error occured while receiving server IPMI infos'))

        # Tickets URL
        elif function_name == 'eemyle.get_link_to_ticket_by_ticket_id':
            if data:
                helpers.open_uri(data)
            else:
                self.geemyle_ui.sb_geemyle.push(0, _('An error occured while receiving ticket URL'))

        # Database statistics
        elif function_name == 'eemyle.get_database_statistics':
            if data:
                dialog = statistics.StatisticsView(data)
                #FIXME: doesn't work: dialog.set_default_response(gtk.RESPONSE_OK)
                dialog.realize()
                dialog.set_transient_for(self.geemyle_ui)
                dialog.show_all()
                dialog.run()
                dialog.destroy()

            else:
                self.geemyle_ui.sb_geemyle.push(0, _('An error occured while receiving database statistics'))

        # Add server
        elif function_name == 'eemyle.add_server':
            if data:
                self.geemyle_ui.sb_geemyle.push(0, _('Successfully added server'))
                # Refresh current server view
                self.on_tv_servers_cursor_changed()
            else:
                self.geemyle_ui.sb_geemyle.push(0, _('Failed to add server'))

        # Remove server
        elif function_name == 'eemyle.remove_server':
            if data:
                self.geemyle_ui.sb_geemyle.push(0, _('Successfully removed server'))
                # Refresh current server's list
                self.on_tv_customers_cursor_changed()
            else:
                self.geemyle_ui.sb_geemyle.push(0, _('Failed to remove server'))

        # Update server
        elif function_name == 'eemyle.update_server':
            if data:
                self.geemyle_ui.sb_geemyle.push(0, _('Successfully updated server'))
                # Refresh current server view
                self.on_tv_servers_cursor_changed()
            else:
                self.geemyle_ui.sb_geemyle.push(0, _('Failed to update server'))

        # Duplicate server
        elif function_name == 'eemyle.duplicate_server':
            if data:
                self.geemyle_ui.sb_geemyle.push(0, _('Successfully duplicated server'))
                # Refresh current servers' list
                self.on_tv_customers_cursor_changed()
            else:
                self.geemyle_ui.sb_geemyle.push(0, _('Failed to duplicate server'))

        # Remove interface
        elif function_name == 'eemyle.remove_interface':
            if data:
                self.geemyle_ui.sb_geemyle.push(0, _('Successfully removed interface'))
                # Refresh current server view
                self.on_tv_servers_cursor_changed()
            else:
                self.geemyle_ui.sb_geemyle.push(0, _('Failed to remove interface'))

        # Update interface
        elif function_name == 'eemyle.update_interface':
            if data:
                self.geemyle_ui.sb_geemyle.push(0, _('Successfully updated interface'))
                # Refresh current server view
                self.on_tv_servers_cursor_changed()
            else:
                self.geemyle_ui.sb_geemyle.push(0, _('Failed to update interface'))

        gtk.gdk.threads_leave()

    def customers_visible_func(self, model, iter):
        
        criteria  = self.cb_customers.get_active_text()
        search    = self.en_customers.get_text()
        no_server = self.cb_no_server.get_active()

        if not no_server:
            value = model.get_value(iter, CUSTOMERS_HAS_SERVER_COLUMN_ID)
            if not value:
                return False

        if search:
            if criteria == _('All'):
                for name, renderer, renderer_properties, column_id, visible, sort_id in CUSTOMERS_TV_COLUMNS:
                    value = str(model.get_value(iter, column_id))
                    if re.search(search, value, re.IGNORECASE):
                        return True
            else:
                for name, renderer, renderer_properties, column_id, visible, sort_id in CUSTOMERS_TV_COLUMNS:
                    if name == criteria:
                        value = str(model.get_value(iter, column_id))
                        if re.search(search, value, re.IGNORECASE):
                            return True
        else:
            return True

        return False

    def on_customers_refilter_activate(self, widget):

        modelsort   = self.tv_customers.get_model()
        modelfilter = modelsort.get_model()
        modelfilter.refilter()

        # Select the customer if there is only one match
        n_children = modelfilter.iter_n_children(None)
        if n_children == 1:
            #iter = modelfilter.get_iter_root()
            #path = modelfilter.get_path(iter)
            self.tv_customers.set_cursor((0,))

        self.tv_customers.columns_autosize()

    def get_current_customer_id(self):
        
        path  = self.tv_customers.get_cursor()[0]
        model = self.tv_customers.get_model()
        iter  = model.get_iter(path)

        customer_id = model.get_value(iter, 0)

        return (customer_id, path)

    def on_tv_customers_cursor_changed(self, treeview=None):
        
        # Buttons
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/CustomerRefresh').set_sensitive(True)
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/CustomerViewContacts').set_sensitive(True)
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/ServerRemove').set_sensitive(False)
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/ServerAdd').set_sensitive(True)
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/ServerDuplicate').set_sensitive(False)
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/TerminalIPMI').set_sensitive(False)
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/TerminalX11').set_sensitive(False)
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/TerminalTunnel').set_sensitive(False)
        self.geemyle_ui.ui.get_widget('/EemyleToolBar/TerminalConnect').set_sensitive(False)
        self.geemyle_ui.ui.get_widget('/Menubar/ConnectionMenu').set_sensitive(False)

        # Resetting the view
        self.server_view.set_server()

        customer_id, path = self.get_current_customer_id()

        # Set the cursor working
        self.geemyle_ui.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
        self.geemyle_ui.sb_geemyle.pop(0)
        self.geemyle_ui.sb_geemyle.push(0, _('Getting servers list from Eemyle...'))

        connector.eemyle_connector.send_request('eemyle.get_servers_by_customer_id', customer_id)

    def servers_visible_func(self, model, iter):
        
        criteria = self.cb_servers.get_active_text()
        search   = self.en_servers.get_text()
        dead     = self.cb_dead.get_active()

        if not dead:
            value = model.get_value(iter, SERVERS_TV_DEAD_COLUMN_ID)
            if value:
                return False

        if search:
            if criteria == _('All'):
                for name, renderer, renderer_properties, column_id, visible, sort_id in SERVERS_TV_COLUMNS:
                    value = str(model.get_value(iter, column_id))
                    if re.search(search, value, re.IGNORECASE):
                        return True
            else:
                for name, renderer, renderer_properties, column_id, visible, sort_id in SERVERS_TV_COLUMNS:
                    if name == criteria:
                        value = str(model.get_value(iter, column_id))
                        if re.search(search, value, re.IGNORECASE):
                            return True
        else:
            return True

        return False

    def on_servers_refilter_activate(self, widget):

        modelsort   = self.tv_servers.get_model()
        modelfilter = modelsort.get_model()
        modelfilter.refilter()

        # Select the server if there is only one match
        n_children = modelfilter.iter_n_children(None)
        if n_children == 1:
            #iter = modelfilter.get_iter_root()
            #path = modelfilter.get_path(iter)
            self.tv_servers.set_cursor((0,))

        self.tv_servers.columns_autosize()

    def get_current_server_id(self):
        
        path  = self.tv_servers.get_cursor()[0]
        model = self.tv_servers.get_model()
        iter  = model.get_iter(path)

        server_id = model.get_value(iter, 0)

        return (server_id, path)

    def get_selected_server_ids(self):

        selection     = self.tv_servers.get_selection()
        model, pathes = selection.get_selected_rows()
        server_ids    = []

        for path in pathes:
            iter = model.get_iter(path)
            server_ids.append(model.get_value(iter, 0))
        
        return server_ids

    def get_selected_servers(self):

        selection     = self.tv_servers.get_selection()
        model, pathes = selection.get_selected_rows()
        servers    = {}

        for path in pathes:
            iter = model.get_iter(path)
            server_id=model.get_value(iter, 0)
            server_name=model.get_value(iter, 2)
            servers[server_id]=server_name

        return servers


    def on_tv_servers_selection_changed(self, selection):
        """
        Managing buttons function of the selection
        """
        
        model, pathes = selection.get_selected_rows()
        if pathes:
            self.geemyle_ui.ui.get_widget('/Menubar/ConnectionMenu').set_sensitive(True)
            # X11 and Tunnel are only available with one server
            if len(pathes) == 1:
                self.geemyle_ui.ui.get_widget('/EemyleToolBar/ServerRemove').set_sensitive(True)
                self.geemyle_ui.ui.get_widget('/EemyleToolBar/ServerDuplicate').set_sensitive(True)
                self.geemyle_ui.ui.get_widget('/EemyleToolBar/TerminalX11').set_sensitive(True)
                self.geemyle_ui.ui.get_widget('/EemyleToolBar/TerminalTunnel').set_sensitive(True)
                self.geemyle_ui.ui.get_widget('/Menubar/ConnectionMenu/TerminalX11').set_sensitive(True)
                self.geemyle_ui.ui.get_widget('/Menubar/ConnectionMenu/KillCurrentIPMITerminal').set_sensitive(True)
                self.geemyle_ui.ui.get_widget('/Menubar/ConnectionMenu/RebootByIPMI').set_sensitive(True)
                self.geemyle_ui.ui.get_widget('/Menubar/ConnectionMenu/TerminalTunnel').set_sensitive(True)
            else:
                self.geemyle_ui.ui.get_widget('/EemyleToolBar/ServerRemove').set_sensitive(False)
                self.geemyle_ui.ui.get_widget('/EemyleToolBar/ServerDuplicate').set_sensitive(False)
                self.geemyle_ui.ui.get_widget('/EemyleToolBar/TerminalX11').set_sensitive(False)
                self.geemyle_ui.ui.get_widget('/EemyleToolBar/TerminalTunnel').set_sensitive(False)
                self.geemyle_ui.ui.get_widget('/Menubar/ConnectionMenu/TerminalX11').set_sensitive(False)
                self.geemyle_ui.ui.get_widget('/Menubar/ConnectionMenu/KillCurrentIPMITerminal').set_sensitive(False)
                self.geemyle_ui.ui.get_widget('/Menubar/ConnectionMenu/RebootByIPMI').set_sensitive(False)
                self.geemyle_ui.ui.get_widget('/Menubar/ConnectionMenu/TerminalTunnel').set_sensitive(False)

            self.geemyle_ui.ui.get_widget('/EemyleToolBar/TerminalConnect').set_sensitive(True)
            self.geemyle_ui.ui.get_widget('/Menubar/ConnectionMenu/TerminalConnect').set_sensitive(True)
            self.geemyle_ui.ui.get_widget('/EemyleToolBar/TerminalIPMI').set_sensitive(True)
            self.geemyle_ui.ui.get_widget('/Menubar/ConnectionMenu/TerminalIPMI').set_sensitive(True)

        else:
            self.geemyle_ui.ui.get_widget('/EemyleToolBar/ServerRemove').set_sensitive(False)
            self.geemyle_ui.ui.get_widget('/EemyleToolBar/ServerDuplicate').set_sensitive(False)
            self.geemyle_ui.ui.get_widget('/EemyleToolBar/TerminalX11').set_sensitive(False)
            self.geemyle_ui.ui.get_widget('/EemyleToolBar/TerminalIPMI').set_sensitive(False)
            self.geemyle_ui.ui.get_widget('/EemyleToolBar/TerminalTunnel').set_sensitive(False)
            self.geemyle_ui.ui.get_widget('/EemyleToolBar/TerminalConnect').set_sensitive(False)
            self.geemyle_ui.ui.get_widget('/Menubar/ConnectionMenu').set_sensitive(False)

    def on_tv_servers_cursor_changed(self, treeview=None):

        server_id, path = self.get_current_server_id()

        # Resetting the view
        self.server_view.set_server()

        # Set the cursor working
        self.geemyle_ui.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
        self.geemyle_ui.sb_geemyle.pop(0)
        self.geemyle_ui.sb_geemyle.push(0, _('Getting server details from Eemyle...'))

        connector.eemyle_connector.send_request('eemyle.get_server_by_server_id',  str(server_id))
        connector.eemyle_connector.send_request('eemyle.get_tickets_by_server_id', str(server_id))
    
    def on_tv_servers_row_activated(self, teeview, path, view_column):
        """
        Connect to the current selected server when the user double click on it
        """

        self.on_bt_connect_clicked()

    def on_bt_connect_clicked(self, button=None, keep_connect_data=False):
        """
        Connect to the current selected server when the user click on the "Connect" button
        """
        
        server_ids = self.get_selected_server_ids()

        # Nothing selected
        if not server_ids:
            return False

        # Set the cursor working
        self.geemyle_ui.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
        self.geemyle_ui.sb_geemyle.pop(0)
        self.geemyle_ui.sb_geemyle.push(0, _('Getting server traceroute from Eemyle...'))

        for server_id in server_ids:
            if not keep_connect_data:
                self.connect_data[server_id] = {}
            connector.eemyle_connector.send_request('eemyle.get_server_traceroute_by_server_id', str(server_id))

    def on_bt_ipmi_clicked(self, button=None):
        """
        Connect to the current selected server's IPMI console when the user click on the "Connect IPMI" button
        """

        server_ids = self.get_selected_server_ids()

        # Nothing selected
        if not server_ids:
            return False

        # Set the cursor working
        self.geemyle_ui.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
        self.geemyle_ui.sb_geemyle.pop(0)
        self.geemyle_ui.sb_geemyle.push(0, _('Getting server IPMI infos from Eemyle...'))

        for server_id in server_ids:
	    self.connect_data[server_id]={'ipmi_command': 'sol activate'}
            connector.eemyle_connector.send_request('eemyle.get_server_ipmi_infos_by_server_id', str(server_id))

    def on_bt_kill_ipmi_clicked(self, button=None):
        """
        Kill current IPMI console session opened on the current selected server when the user click on the "Kill current IPMI console session" button
        """

        server_ids = self.get_selected_server_ids()

        # Nothing selected
        if not server_ids:
            return False

        server_id = server_ids[0]
	self.connect_data[server_id]={'ipmi_command': 'sol deactivate'}

        # Set the cursor working
        self.geemyle_ui.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
        self.geemyle_ui.sb_geemyle.pop(0)
        self.geemyle_ui.sb_geemyle.push(0, _('Getting server IPMI infos from Eemyle...'))

        connector.eemyle_connector.send_request('eemyle.get_server_ipmi_infos_by_server_id', str(server_id))

    def on_reboot_by_ipmi_clicked(self, button=None):
        """
        Reboot current selected server by IPMI when the user click on the "Reboot server by IPMI" button
        """
	servers = self.get_selected_servers()

        # Nothing selected
        if not servers:
            return False

	server_id = servers.keys()[0]
	server_name = servers[server_id]

	self.connect_data[server_id]={'ipmi_command': 'chassis power cycle'}

        dialog = gtk.MessageDialog(flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,type=gtk.MESSAGE_WARNING, buttons=gtk.BUTTONS_OK_CANCEL, message_format=_('Do you realy want to reboot this server %s by IPMI ?' % server_name))

        dialog.set_title(_('Reboot server by IPMI'))
        dialog.set_default_response(gtk.RESPONSE_CANCEL)

        dialog.realize()
        dialog.set_resizable(False)

        hbox = gtk.HBox()
        hbox.set_spacing(5)
        hbox.set_border_width(10)

        label = gtk.Label(_('Server name:'))
        label.set_alignment(0, 0.5)
        hbox.pack_start(label, expand=False)

        entry  = gtk.Entry()
        hbox.pack_start(entry)

        dialog.vbox.pack_start(hbox)

        dialog.show_all()

        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            name = entry.get_text()
            if name == server_name:
                print ""
                self.geemyle_ui.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
                self.geemyle_ui.sb_geemyle.pop(0)
                self.geemyle_ui.sb_geemyle.push(0, _('Getting server IPMI infos from Eemyle...'))

                connector.eemyle_connector.send_request('eemyle.get_server_ipmi_infos_by_server_id', str(server_id))
            else:
                msg = common.ErrorDialog(message_format=_('Incorrect server name. Check the selected server !'))
                msg.run()
                msg.destroy()

        dialog.destroy()
        return True


    def on_server_ipmi_infos_received(self, ipmi_infos):
        """
        Connect to a server's IPMI console
        """

        if not ipmi_infos['host'] or not ipmi_infos['user']:
            self.geemyle_ui.sb_geemyle.pop(0)
            self.geemyle_ui.sb_geemyle.push(0, _("This server doesn't have IPMI console"))
            return True

	server_id=int(ipmi_infos['id_server'])
	if server_id in self.connect_data and 'ipmi_command' in self.connect_data[server_id]:
            cmd = 'ipmitool -I lanplus -H %s -U %s -P %s %s' % (ipmi_infos['host'],ipmi_infos['user'],ipmi_infos['password'],self.connect_data[server_id]['ipmi_command'])
            self.connect_data[int(ipmi_infos['id_server'])] = { 'remote_command': cmd, 'run_on_previous_hop': True, 'session_name_prefix': 'IPMI', 'IPMI': True }
            self.on_bt_connect_clicked(keep_connect_data=True)

    def on_bt_add_server_clicked(self, button=None):
        """
        Create a new server attached to the current customer
        """

        customer_id, path = self.get_current_customer_id()

        self.server_view.on_add_server_activate(customer_id)
        
    def on_bt_remove_server_clicked(self, button=None):
        """
        Remove the selected server
        """

        server_ids = self.get_selected_server_ids()
        # Only one server must be selected
        if not len(server_ids) == 1:
            return False

        server_id = server_ids[0]

        dialog   = dialogs.ConfirmDialog(_('Do you really want to remove the selected server ?'))
        response = dialog.run()
        dialog.destroy()

        if response == gtk.RESPONSE_OK:
            self.geemyle_ui.sb_geemyle.pop(0)
            self.geemyle_ui.sb_geemyle.push(0, _('Removing the server...'))
            connector.eemyle_connector.send_request('eemyle.remove_server', str(server_id))

    def on_bt_duplicate_server_clicked(self, button=None):
        """
        Duplicate the server using Eemyle
        """

        server_ids = self.get_selected_server_ids()
        # Only one server must be selected
        if not len(server_ids) == 1:
            return False

        server_id = server_ids[0]

        dialog   = dialogs.ConfirmDialog(_('Do you really want to duplicate the selected server ?'))
        response = dialog.run()
        dialog.destroy()

        if response == gtk.RESPONSE_OK:
            self.geemyle_ui.sb_geemyle.pop(0)
            self.geemyle_ui.sb_geemyle.push(0, _('Duplicating the server...'))
            connector.eemyle_connector.send_request('eemyle.duplicate_server', str(server_id))

    def on_database_statistics_clicked(self, button=None):
        """
        View the database statistics
        """
    
        self.geemyle_ui.sb_geemyle.pop(0)
        self.geemyle_ui.sb_geemyle.push(0, _('Getting statistics data from server...'))
        connector.eemyle_connector.send_request('eemyle.get_database_statistics')

    def ticket_url_callback(self, ticket_id):
        """
        Edit the ticket using RT
        """

        self.geemyle_ui.sb_geemyle.pop(0)
        self.geemyle_ui.sb_geemyle.push(0, _('Starting webbrowser...'))
        connector.eemyle_connector.send_request('eemyle.get_link_to_ticket_by_ticket_id', str(ticket_id))

    def on_bt_x11_clicked(self, button=None):
        """
        Open an X11 connection with the selected server when the user click on the "X11" button
        """

        server_ids = self.get_selected_server_ids()
        # Only one server must be selected
        if not len(server_ids) == 1:
            return False

        server_id = server_ids[0]

        # Storing extra parameters
        self.connect_data[server_id] = { 'compression':   True,
                                         'x11forwarding': True }

        # Set the cursor working
        self.geemyle_ui.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
        self.geemyle_ui.sb_geemyle.pop(0)
        self.geemyle_ui.sb_geemyle.push(0, _('Getting server traceroute from Eemyle...'))

        connector.eemyle_connector.send_request('eemyle.get_server_traceroute_by_server_id', str(server_id))

    def on_bt_tunnel_clicked(self, button=None):
        """
        Open a tunnel with the selected server when the user click on the "Tunnel" button
        """
        
        server_ids = self.get_selected_server_ids()
        # Only one server must be selected
        if not len(server_ids) == 1:
            return False
        
        server_id = server_ids[0]

        # Getting the tunnel parameters
        dialog = dialogs.TunnelDialog()
        dialog.set_default_response(gtk.RESPONSE_OK)
        dialog.realize()
        dialog.set_transient_for(self.geemyle_ui)
        dialog.show_all()
        response = dialog.run()
        dialog.destroy()

        if response == gtk.RESPONSE_OK:
            # Storing extra parameters
            self.connect_data[server_id] = dialog.tunnel_data.copy()

            # Set the cursor working
            self.geemyle_ui.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
            self.geemyle_ui.sb_geemyle.pop(0)
            self.geemyle_ui.sb_geemyle.push(0, _('Getting server traceroute from Eemyle...'))

            connector.eemyle_connector.send_request('eemyle.get_server_traceroute_by_server_id', str(server_id))

    def on_close_activate(self):
        """
        Called by close button
        """
        
        global _eemyle_view

        # Removing the Eemyle view from all Eemyle views
        _eemyle_view = None

        self.stop_connector()
        self.emit('close-clicked')

