#!/usr/bin/env python
#
# GEemyle: Eemyle preferences
# Copyright (C) 2003-2006 Easter-eggs <info@easter-eggs.com>
# Authors:
#      Cyril Lacoux (Yack) <clacoux@easter-eggs.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

__author__  = "Cyril Lacoux [Yack] <clacoux@easter-eggs.com>"
__cvsid__   = "$Id$"
__version__ = "$Revision$"[11:-2]
__all__     = ["EemylePreferences"]


import pygtk
pygtk.require('2.0')
import gtk
import gobject
import re

import geemyle.config          as config
import geemyle.gettext as gettext
from   geemyle.gui.preferences import Preferences

_ = gettext._


class GeneralPreferences(Preferences):

    def __init__(self):

        Preferences.__init__(self, _('General'))

        self.full_screen     = gtk.CheckButton(_('Full Screen mode'))
        self.show_menubar    = gtk.CheckButton(_('Show Menubar'))
        self.show_toolbar    = gtk.CheckButton(_('Show Toolbar'))
        self.tab_position    = gtk.ComboBox(gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING))

        self.construct_frame()
        self.get_config()

    def construct_frame(self):

        self.full_screen.connect('toggled', self.emit_changed)
        self.show_menubar.connect('toggled', self.emit_changed)
        self.show_toolbar.connect('toggled', self.emit_changed)
        self.tab_position.connect('changed',  self.emit_changed)

        cell = gtk.CellRendererText()
        self.tab_position.pack_start(cell, True)
        self.tab_position.add_attribute(cell, 'text', 0)

        i = 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('General:'))
        self.attach(label,                1, 3, i, i+1)

        i += 1
        self.attach(self.full_screen,     1, 4, i, i+1)

        i += 1
        self.attach(self.show_menubar,    1, 4, i, i+1)

        i += 1
        self.attach(self.show_toolbar,    1, 4, i, i+1)
        
        i += 1
        label = gtk.Label(_('Tab position:'))
        label.set_alignment(0, 0.5)
        self.attach(label,                1, 2, i, i+1)
        self.attach(self.tab_position,    2, 4, i, i+1)

        self.show_all()

    def get_config(self):

        self.full_screen.set_active(config.get('geemyle',  'full_screen'))
        self.show_menubar.set_active(config.get('geemyle', 'show_menubar'))
        self.show_toolbar.set_active(config.get('geemyle', 'show_toolbar'))

        model = self.tab_position.get_model()
        model.clear()
        for display, position in ((_('Top'), 'top'), (_('Bottom'), 'bottom'), (_('Right'), 'right'), (_('Left'), 'left')):
            iter = model.append((display, position))
            if config.get('geemyle', 'tab_position') == position:
                self.tab_position.set_active_iter(iter)

    def set_config(self):
        
        config.set('geemyle', 'full_screen',  self.full_screen.get_active())
        config.set('geemyle', 'show_menubar', self.show_menubar.get_active())
        config.set('geemyle', 'show_toolbar', self.show_toolbar.get_active())

        model = self.tab_position.get_model()
        iter  = self.tab_position.get_active_iter()
        config.set('geemyle', 'tab_position', model.get_value(iter, 1))


class ServerPreferences(Preferences):

    def __init__(self):

        Preferences.__init__(self, _('Server'))

        self.hostname   = gtk.Entry()
        self.port       = gtk.SpinButton()
        self.ssl        = gtk.CheckButton(_('Enable SSL'))
        self.wservice   = gtk.Entry()
        self.username   = gtk.Entry()
        self.password   = gtk.Entry()
        self.keep_u_p   = gtk.CheckButton(_('Keep username and password'))

        self.construct_frame()
        self.get_config()

    def construct_frame(self):

        self.hostname.connect('changed',   self.emit_changed)
        self.ssl.connect('toggled',        self.on_ssl_toggled)
        self.wservice.connect('changed',   self.emit_changed)
        self.username.connect('changed',   self.emit_changed)
        self.password.connect('changed',   self.emit_changed)
        self.password.set_visibility(False)
        self.keep_u_p.connect('toggled',   self.on_keep_u_p_toggled)
        
        adjustment = gtk.Adjustment(0, 1, 65535, 1, 1, 0)
        adjustment.connect('value-changed', self.emit_changed)

        self.port.set_adjustment(adjustment)
        self.port.set_max_length(5)
        self.port.set_width_chars(5)

        i = 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Server:'))
        self.attach(label,                1, 4, i, i+1)

        i += 1
        label = gtk.Label(_('Hostname and port'))
        label.set_alignment(0, 0.5)
        self.attach(label,                1, 2, i, i+1)
        self.attach(self.hostname,        2, 3, i, i+1)
        self.attach(self.port,            3, 4, i, i+1)

        i += 1
        self.attach(self.ssl,             1, 4, i, i+1)

        i += 1
        label = gtk.Label(_('Service'))
        label.set_alignment(0, 0.5)
        self.attach(label,                1, 2, i, i+1)
        self.attach(self.wservice,        2, 3, i, i+1)

        i += 1
        label = gtk.Label(_('Username'))
        label.set_alignment(0, 0.5)
        self.attach(label,                1, 2, i, i+1)
        self.attach(self.username,        2, 3, i, i+1)

        i += 1
        label = gtk.Label(_('Password'))
        label.set_alignment(0, 0.5)
        self.attach(label,                1, 2, i, i+1)
        self.attach(self.password,        2, 3, i, i+1)

        i += 1
        self.attach(self.keep_u_p,        1, 4, i, i+1)

        self.show_all()

    def on_ssl_toggled(self, check_button):
        
        port = self.port.get_value_as_int()
        if port == 80:
            if check_button.get_active():
                self.port.set_value(443)

        elif port == 443:
            if not check_button.get_active():
                self.port.set_value(80)

        self.emit_changed()

    def on_keep_u_p_toggled(self, check_button):

        if check_button.get_active():
            self.username.set_sensitive(True)
            self.password.set_sensitive(True)
        else:
            self.username.set_sensitive(False)
            self.password.set_sensitive(False)
    def get_config(self):

        self.hostname.set_text(config.get('eemyle',   'hostname'))
        self.port.set_value(config.get('eemyle',      'port'))
        self.ssl.set_active(config.get('eemyle',      'ssl'))
        self.wservice.set_text(config.get('eemyle',   'wservice'))

        keep_u_p = config.get('eemyle', 'keep_u_p')
        self.keep_u_p.set_active(keep_u_p)
        if keep_u_p:
            self.username.set_text(config.get('eemyle',   'username'))
            self.password.set_text(config.get('eemyle',   'password'))

    def set_config(self):
        
        config.set('eemyle', 'hostname', self.hostname.get_text())
        config.set('eemyle', 'port',     self.port.get_value_as_int())
        config.set('eemyle', 'ssl',      self.ssl.get_active())
        config.set('eemyle', 'wservice', self.wservice.get_text())

        keep_u_p = self.keep_u_p.get_active()
        if keep_u_p:
            config.set('eemyle', 'username', self.username.get_text())
            config.set('eemyle', 'password', self.password.get_text())
        else:
            config.set('eemyle', 'username', '')
            config.set('eemyle', 'password', '')
        config.set('eemyle', 'keep_u_p', keep_u_p)


class AsteriskPreferences(Preferences):

    def __init__(self):

        Preferences.__init__(self, _('Asterisk'))

        self.asterisk_host = gtk.Entry()
        self.asterisk_port = gtk.SpinButton()
        self.asterisk_user = gtk.Entry()
        self.asterisk_pass = gtk.Entry()
        self.asterisk_ext  = gtk.Entry()
        self.asterisk_ctx  = gtk.Entry()

        self.construct_frame()
        self.get_config()

    def construct_frame(self):

        self.asterisk_host.connect('changed', self.emit_changed)
        self.asterisk_user.connect('changed', self.emit_changed)
        self.asterisk_pass.connect('changed', self.emit_changed)
        self.asterisk_ctx.connect('changed',  self.emit_changed)
        self.asterisk_ext.connect('changed',  self.emit_changed)

        adjustment = gtk.Adjustment(0, 1, 65535, 1, 1, 0)
        adjustment.connect('value-changed',   self.emit_changed)

        self.asterisk_port.set_adjustment(adjustment)
        self.asterisk_port.set_max_length(5)
        self.asterisk_port.set_width_chars(5)

        self.asterisk_pass.set_visibility(False)

        i = 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Asterisk:'))
        self.attach(label,                1, 3, i, i+1)

        i += 1
        label = gtk.Label(_('Hostname and port'))
        label.set_alignment(0, 0.5)
        self.attach(label,                1, 2, i, i+1)
        self.attach(self.asterisk_host,   2, 3, i, i+1)
        self.attach(self.asterisk_port,   3, 4, i, i+1)

        i += 1
        label = gtk.Label(_('Username'))
        label.set_alignment(0, 0.5)
        self.attach(label,                1, 2, i, i+1)
        self.attach(self.asterisk_user,   2, 3, i, i+1)

        i += 1
        label = gtk.Label(_('Password'))
        label.set_alignment(0, 0.5)
        self.attach(label,                1, 2, i, i+1)
        self.attach(self.asterisk_pass,   2, 3, i, i+1)

        i += 1
        label = gtk.Label(_('Context'))
        label.set_alignment(0, 0.5)
        self.attach(label,                1, 2, i, i+1)
        self.attach(self.asterisk_ctx,    2, 3, i, i+1)

        i += 1
        label = gtk.Label(_('Extension'))
        label.set_alignment(0, 0.5)
        self.attach(label,                1, 2, i, i+1)
        self.attach(self.asterisk_ext,    2, 3, i, i+1)

        self.show_all()

    def get_config(self):

        self.asterisk_host.set_text(config.get('geemyle',   'asterisk_host'))
        self.asterisk_port.set_value(config.get('geemyle',  'asterisk_port'))
        self.asterisk_user.set_text(config.get('geemyle',   'asterisk_user'))
        self.asterisk_pass.set_text(config.get('geemyle',   'asterisk_pass'))
        self.asterisk_ctx.set_text(config.get('geemyle',    'asterisk_ctx'))
        self.asterisk_ext.set_text(config.get('geemyle',    'asterisk_ext'))

    def set_config(self):

        config.set('geemyle',  'asterisk_host', self.asterisk_host.get_text())
        config.set('geemyle',  'asterisk_port', self.asterisk_port.get_value_as_int())
        config.set('geemyle',  'asterisk_user', self.asterisk_user.get_text())
        config.set('geemyle',  'asterisk_pass', self.asterisk_pass.get_text())
        config.set('geemyle',  'asterisk_ctx',  self.asterisk_ctx.get_text())
        config.set('geemyle',  'asterisk_ext',  self.asterisk_ext.get_text())


class HelpersPreferences(Preferences):

    def __init__(self):

        Preferences.__init__(self, _('Helpers'))

        self.browser_command = gtk.Entry()
        self.mail_command    = gtk.Entry()
        self.fax_command     = gtk.Entry()

        self.construct_frame()
        self.get_config()

    def construct_frame(self):

        self.browser_command.connect('changed',  self.emit_changed)
        self.mail_command.connect('changed',     self.emit_changed)
        self.fax_command.connect('changed',      self.emit_changed)

        i = 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Commands:'))
        self.attach(label,                1, 3, i, i+1)

        i += 1
        label = gtk.Label(_('Browser command:'))
        label.set_alignment(0, 0.5)
        self.attach(label,                1, 2, i, i+1)
        self.attach(self.browser_command, 2, 4, i, i+1)

        i += 1
        label = gtk.Label(_('Mail command:'))
        label.set_alignment(0, 0.5)
        self.attach(label,                1, 2, i, i+1)
        self.attach(self.mail_command,    2, 4, i, i+1)

        i += 1
        label = gtk.Label(_('Fax command:'))
        label.set_alignment(0, 0.5)
        self.attach(label,                1, 2, i, i+1)
        self.attach(self.fax_command,     2, 4, i, i+1)

        self.show_all()

    def get_config(self):

        self.browser_command.set_text(config.get('geemyle', 'browser_command'))
        self.mail_command.set_text(config.get('geemyle',    'mail_command'))
        self.fax_command.set_text(config.get('geemyle',     'fax_command'))

    def set_config(self):

        config.set('geemyle', 'browser_command', self.browser_command.get_text())
        config.set('geemyle', 'mail_command',    self.mail_command.get_text())
        config.set('geemyle', 'fax_command',     self.fax_command.get_text())


class ConnectPreferences(Preferences):

    def __init__(self):

        Preferences.__init__(self, _('Connection'))

        self.use_first_hop = gtk.CheckButton(_('Use a first hop'))
        self.fh_ip_address = gtk.Entry()
        self.fh_hostname   = gtk.Entry()
        self.fh_username   = gtk.Entry()
        self.fh_password   = gtk.Entry()
        self.use_ssh_compression  = gtk.CheckButton(_('Use ssh compression'))

        self.construct_frame()
        self.get_config()

    def construct_frame(self):

        self.use_first_hop.connect('toggled',  self.on_use_first_hop_toggled)
        self.fh_ip_address.connect('changed',  self.emit_changed)
        self.fh_hostname.connect('changed',    self.emit_changed)
        self.fh_username.connect('changed',    self.emit_changed)
        self.fh_password.connect('changed',    self.emit_changed)
        self.fh_password.set_visibility(False)
        self.use_ssh_compression.connect('toggled',  self.emit_changed)
        
        i = 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('First hop:'))
        self.attach(label,                1, 4, i, i+1)

        i += 1
        self.attach(self.use_first_hop,   1, 4, i, i+1)

        i += 1
        label = gtk.Label(_('IP address'))
        label.set_alignment(0, 0.5)
        self.attach(label,                1, 2, i, i+1)
        self.attach(self.fh_ip_address,   2, 4, i, i+1)

        i += 1
        label = gtk.Label(_('Hostname'))
        label.set_alignment(0, 0.5)
        self.attach(label,                1, 2, i, i+1)
        self.attach(self.fh_hostname,     2, 4, i, i+1)

        i += 1
        label = gtk.Label(_('Username'))
        label.set_alignment(0, 0.5)
        self.attach(label,                1, 2, i, i+1)
        self.attach(self.fh_username,     2, 4, i, i+1)

        i += 1
        label = gtk.Label(_('Password'))
        label.set_alignment(0, 0.5)
        self.attach(label,                1, 2, i, i+1)
        self.attach(self.fh_password,     2, 4, i, i+1)

        i += 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        self.attach(label,                1, 2, i, i+1)
        self.attach(self.use_ssh_compression,     2, 4, i, i+1)


        self.show_all()

    def on_use_first_hop_toggled(self, checkbutton):
        
        if checkbutton.get_active():
            self.fh_ip_address.set_sensitive(True)
            self.fh_hostname.set_sensitive(True)
            self.fh_username.set_sensitive(True)
            self.fh_password.set_sensitive(True)
        else:
            self.fh_ip_address.set_sensitive(False)
            self.fh_hostname.set_sensitive(False)
            self.fh_username.set_sensitive(False)
            self.fh_password.set_sensitive(False)
        
        self.emit_changed()

    def get_config(self):

        self.use_first_hop.set_active(config.get('eemyle', 'use_first_hop'))
        if not self.use_first_hop.get_active():
            self.fh_ip_address.set_sensitive(False)
            self.fh_hostname.set_sensitive(False)
            self.fh_username.set_sensitive(False)
            self.fh_password.set_sensitive(False)
            
        self.fh_ip_address.set_text(config.get('eemyle',   'fh_ip_address'))
        self.fh_hostname.set_text(config.get('eemyle',     'fh_hostname'))
        self.fh_username.set_text(config.get('eemyle',     'fh_username'))
        self.fh_password.set_text(config.get('eemyle',     'fh_password'))
        self.use_ssh_compression.set_active(config.get('eemyle', 'use_ssh_compression'))

    def set_config(self):
        
        config.set('eemyle', 'use_first_hop', self.use_first_hop.get_active())
        config.set('eemyle', 'fh_ip_address', self.fh_ip_address.get_text())
        config.set('eemyle', 'fh_hostname',   self.fh_hostname.get_text())
        config.set('eemyle', 'fh_username',   self.fh_username.get_text())
        config.set('eemyle', 'fh_password',   self.fh_password.get_text())
        config.set('eemyle', 'use_ssh_compression', self.use_ssh_compression.get_active())

    
class EemylePreferences(gtk.Dialog):

    def __init__(self, load_config_callback):

        gtk.Dialog.__init__(self, flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT, buttons=(gtk.STOCK_CLOSE, gtk.RESPONSE_CANCEL))

        self.set_title('%s - %s' % (config.PACKAGE, _('Eemyle preferences')))
        self.set_resizable(False)

        self.load_config_callback = load_config_callback
        self.notebook = gtk.Notebook()

        self.construct_frame()
        self.connect('destroy', self.on_destroy)
    
    def construct_frame(self):

        for pref in GeneralPreferences, ServerPreferences, HelpersPreferences, AsteriskPreferences, ConnectPreferences:

            inst_pref = pref()
            inst_pref.connect('something-changed', self.something_changed)

            table = gtk.Table(3,3)
            table.attach(inst_pref, 1,2,1,2, gtk.SHRINK, gtk.SHRINK)

            self.notebook.append_page(table, inst_pref.label)

        self.vbox.pack_start(self.notebook)

    def something_changed(self, pref):
        
        # Retrieving the config from all tabs
        page_num = self.notebook.get_n_pages()
        for i in range(0, page_num):
            child_pref = self.notebook.get_nth_page(i).get_children()[0]
            if child_pref:
                child_pref.set_config()

        # Applying the changes
        self.load_config_callback()

    def on_destroy(self, dialog):

        # Write the config
        if not config.write():
            pass
            # FIXME: onpopup

