#!/usr/bin/env python
#
# GEemyle: Server View
# Copyright (C) 2003-2006 Easter-eggs <info@easter-eggs.com>
# Authors:
#      Cyril Lacoux (Yack) <clacoux@easter-eggs.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

__author__  = "Cyril Lacoux [Yack] <clacoux@easter-eggs.com>"
__cvsid__   = "$Id: server.py 102 2006-06-01 19:34:20Z yack $"
__version__ = "$Revision: 102 $"[11:-2]
__all__     = ["ServerComments", "ServerHistory"]


import pygtk
pygtk.require('2.0')
import gtk

import geemyle.eemyle_connector as connector
import geemyle.gettext as gettext
import common

_ = gettext._

class ServerComments(common.ServerTab):
    
    def __init__(self):

        common.ServerTab.__init__(self, _('Comments'), has_sw=False)

        self.id_server = -1

    def set_server(self, server=None):
        
        self.remove_children()
        self.set_sensitive(False)

        if not server:
            self.id_server = -1
            return True

        self.id_server = server['id_server']
        self.set_sensitive(True)

        tb = gtk.TextBuffer()
        tb.set_text(server['comments'])

        tv = gtk.TextView(tb)
        sw = gtk.ScrolledWindow()
        sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        sw.add(tv)

        frame = common.HardwareFrame(_('Comments'), has_remove=False)
        frame.bt_save.connect('clicked', self.on_update_server_clicked, tb)
        frame.set_sensitive(True)
        frame.pack_start(sw)

        self.add_child(frame)

        self.show_all()

    def get_server(self, text_buffer):

        start_iter = text_buffer.get_start_iter()
        end_iter   = text_buffer.get_end_iter()
        text       = text_buffer.get_text(start_iter, end_iter)

        server = { 'comments': text }

        return server

    def on_update_server_clicked(self, button, text_buffer):
        
        server = self.get_server(text_buffer)
        connector.eemyle_connector.send_request('eemyle.update_server', self.id_server, server)


class ServerHistory(common.ServerTab):
    
    def __init__(self):

        common.ServerTab.__init__(self, _('History'), has_sw=False)

        self.id_server = -1

    def set_server(self, server=None):

        self.remove_children()
        self.set_sensitive(False)

        if not server:
            self.id_server = -1
            return True

        self.id_server = server['id_server']
        self.set_sensitive(True)
        
        tb = gtk.TextBuffer()
        tb.set_text(server['history'])

        tv = gtk.TextView(tb)
        sw = gtk.ScrolledWindow()
        sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        sw.add(tv)

        frame = common.HardwareFrame(_('History'), has_remove=False)
        frame.bt_save.connect('clicked', self.on_update_server_clicked, tb)
        frame.set_sensitive(True)
        frame.pack_start(sw)

        self.add_child(frame)

        self.show_all()

    def get_server(self, text_buffer):

        start_iter = text_buffer.get_start_iter()
        end_iter   = text_buffer.get_end_iter()
        text       = text_buffer.get_text(start_iter, end_iter)

        server = { 'history': text }

        return server

    def on_update_server_clicked(self, button, text_buffer):
        
        server = self.get_server(text_buffer)
        connector.eemyle_connector.send_request('eemyle.update_server', self.id_server, server)


