#!/usr/bin/env python
#
# GEemyle: Server View
# Copyright (C) 2003-2006 Easter-eggs <info@easter-eggs.com>
# Authors:
#      Cyril Lacoux (Yack) <clacoux@easter-eggs.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

__author__  = "Cyril Lacoux [Yack] <clacoux@easter-eggs.com>"
__cvsid__   = "$Id: server.py 102 2006-06-01 19:34:20Z yack $"
__version__ = "$Revision: 102 $"[11:-2]
__all__     = ["ServerTab", "HardwareFrame", "ServerTable"]


import pygtk
pygtk.require('2.0')
import gtk

import geemyle.gui.mygtk as mygtk
import geemyle.config    as config

class ServerTab(gtk.VBox):

    def __init__(self, label='', has_add=False, add_label='', has_sw=True, has_vp=True, widget_class=gtk.VBox):

        gtk.VBox.__init__(self)

        self.label  = gtk.Label(label)
        self.widget = widget_class()

        # Buttons
        if has_add:
            self.bt_add = mygtk.Button(stock=gtk.STOCK_ADD, label=add_label)
        else:
            self.bt_add = None

        if has_sw:
            self.sw = gtk.ScrolledWindow()
            if has_vp:
                self.vp = gtk.Viewport()
            else:
                self.vp = None
        else:
            self.sw = None
            self.vp = None

        self.display_lists = None

        self.construct_frame()
        self.set_sensitive(False)

    def set_display_lists(self, display_lists):

        self.display_lists = display_lists

    def construct_frame(self):

        self.set_border_width(5)
        self.set_spacing(5)

        if isinstance(self.widget, gtk.VBox):
            self.widget.set_spacing(5)

        # Buttons
        hbbox = gtk.HButtonBox()
        hbbox.set_layout(gtk.BUTTONBOX_END)
        hbbox.set_spacing(10)
        if self.bt_add:
            hbbox.pack_start(self.bt_add)

        self.pack_start(hbbox, expand=False)

        if self.sw:
            self.sw.set_shadow_type(gtk.SHADOW_NONE)
            self.sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

            if self.vp:
                self.vp.set_shadow_type(gtk.SHADOW_NONE)
                self.vp.add(self.widget)
                self.sw.add(self.vp)
            else:
                self.sw.add(self.widget)

            self.pack_start(self.sw)

        else:
            self.pack_start(self.widget)

    def add_child(self, child, *args, **kargs):
        
        if not isinstance(self.widget, gtk.VBox):
            return False

        self.widget.pack_start(child, *args, **kargs)

    def remove_children(self):

        if not isinstance(self.widget, gtk.VBox):
            return False

        for child in self.widget.get_children():
            child.destroy()

    def set_sensitive(self, sensitive):
        
        if self.bt_add:
            self.bt_add.set_sensitive(sensitive)

    def set_server(self, server=None):

        pass

class HardwareFrame(gtk.Frame):

    def __init__(self, label=None, has_remove=True):
        
        gtk.Frame.__init__(self, label)
        self.vbox = gtk.VBox()

        # Buttons
        if has_remove:
            self.bt_remove = gtk.Button(stock=gtk.STOCK_REMOVE)
        else:
            self.bt_remove = None
        self.bt_save = gtk.Button(stock=gtk.STOCK_SAVE)
        
        self.construct_frame()
        self.set_sensitive(False)

    def construct_frame(self):

        self.vbox.set_border_width(5)
        self.vbox.set_spacing(10)

        # Buttons
        hbbox = gtk.HButtonBox()
        hbbox.set_layout(gtk.BUTTONBOX_END)
        hbbox.set_spacing(10)
        if self.bt_remove:
            hbbox.pack_start(self.bt_remove)
        hbbox.pack_start(self.bt_save)

        self.vbox.pack_end(hbbox, expand=False)
        
        self.add(self.vbox)

    def pack_start(self, *args, **kargs):

        self.vbox.pack_start(*args, **kargs)

    def set_sensitive(self, sensitive):
        
        if self.bt_remove:
            self.bt_remove.set_sensitive(sensitive)
        self.bt_save.set_sensitive(sensitive)

class ServerTable(gtk.Table):

    def __init__(self):

        gtk.Table.__init__(self)

        self.set_border_width(5)
        self.set_row_spacings(5)
        self.set_col_spacings(20)

class AddDialog(gtk.Dialog):
    
    def __init__(self, add_widget, label):

        gtk.Dialog.__init__(self, flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                            buttons=(gtk.STOCK_OK,     gtk.RESPONSE_OK,
                                     gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))

        self.set_title('%s: %s' % (config.PACKAGE, label))
        self.realize()
        self.set_icon_from_file(config.APP_ICON_FILEPATH)

        self.vbox.pack_start(add_widget)
        add_widget.show_all()
        

class RemoveDialog(gtk.MessageDialog):
    
    def __init__(self, message_format):
        
        gtk.MessageDialog.__init__(self, flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                                   type=gtk.MESSAGE_QUESTION,
                                   buttons=gtk.BUTTONS_OK_CANCEL,
                                   message_format=message_format)

        self.set_title(config.PACKAGE)
        self.realize()
        self.set_icon_from_file(config.APP_ICON_FILEPATH)

class ErrorDialog(gtk.MessageDialog):

    def __init__(self, message_format):

        gtk.MessageDialog.__init__(self, flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                                   type=gtk.MESSAGE_ERROR,
                                   buttons=gtk.BUTTONS_OK,
                                   message_format=message_format)

        self.set_title(config.PACKAGE)
        self.realize()
        self.set_icon_from_file(config.APP_ICON_FILEPATH)
