#!/usr/bin/env python
#
# GEemyle: Server View
# Copyright (C) 2003-2006 Easter-eggs <info@easter-eggs.com>
# Authors:
#      Cyril Lacoux (Yack) <clacoux@easter-eggs.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

__author__  = "Cyril Lacoux [Yack] <clacoux@easter-eggs.com>"
__cvsid__   = "$Id: server.py 102 2006-06-01 19:34:20Z yack $"
__version__ = "$Revision: 102 $"[11:-2]
__all__     = ["ServerGeneral"]

import pygtk
pygtk.require('2.0')
import gtk
import gobject

import geemyle.gettext          as gettext
import geemyle.gui.mygtk        as mygtk
import geemyle.eemyle_connector as connector
import common

_ = gettext._
USERS_TV_COLUMN_ID   = 0
USERS_TV_COLUMN_CONN = 1
USERS_TV_COLUMN_NAME = 2
USERS_TV_COLUMN_PASS = 3
USERS_TV_COLUMN_HIDE = 4

USERS_TV_COLUMNS = (
   # Name           Renderer                Renderer attributes                            col_id                visibility   sort_id
    ('',            gtk.CellRendererText,   (),                                            USERS_TV_COLUMN_ID,   False,       USERS_TV_COLUMN_ID),    # User ID
    (_('C'),        gtk.CellRendererToggle, (('radio',    True),),                         USERS_TV_COLUMN_CONN, True,        USERS_TV_COLUMN_CONN),  # Connect user
    (_('Username'), gtk.CellRendererText,   (('editable', True), ('family', 'Monospace')), USERS_TV_COLUMN_NAME, True,        USERS_TV_COLUMN_NAME),  # Username
    (_('Password'), gtk.CellRendererText,   (('editable', True), ('family', 'Monospace')), USERS_TV_COLUMN_PASS, False,       USERS_TV_COLUMN_PASS),  # Password
    (_('Shadow'),   gtk.CellRendererText,   (),                                            USERS_TV_COLUMN_HIDE, True,        USERS_TV_COLUMN_HIDE),  # Password (hidden)
    )
                    
class ServerInformations(common.ServerTable):
    
    def __init__(self, id_customer, server=None, display_lists=None):

        common.ServerTable.__init__(self)

        self.id_customer = id_customer
        self.id_server   = -1

        self.en_server_name  = gtk.Entry()
        self.bf_address      = gtk.TextBuffer()
        self.cb_dead         = gtk.CheckButton(_('Dead'))
        self.cb_sudo_root    = gtk.CheckButton(_('Use sudo for root'))

        # Users
        model = gtk.ListStore(gobject.TYPE_INT,     # User ID
                              gobject.TYPE_BOOLEAN, # Connect user
                              gobject.TYPE_STRING,  # Username
                              gobject.TYPE_STRING,  # Password
                              gobject.TYPE_STRING ) # Password (hided)

        self.tv_users          = gtk.TreeView(model)
        self.cb_connectable    = gtk.CheckButton(_('Connection is available on'))
        self.cb_interface      = mygtk.SimpleComboBox(server['connection_interface_data'])
        self.cb_ipmi_connectable = gtk.CheckButton(_('IPMI connection is available'))
        self.cb_ipmi_interface   = mygtk.SimpleComboBox(server['connection_interface_data'])
        self.en_ipmi_user_name   = gtk.Entry()
        self.en_ipmi_user_pwd    = gtk.Entry()
        self.en_ipmi_user_pwd.set_property('visibility',False)
        self.cb_previous_hop   = mygtk.SimpleComboBox(server['previous_hops_data'])
        self.cb_physical_host  = mygtk.SimpleComboBox(server['physical_host_data'])
        self.sb_ssh_port       = gtk.SpinButton()

        self.construct_frame()
        if server:
            self.set_server(server)

    def construct_frame(self):

        # Users
        self.tv_users.set_rules_hint(True)
        self.tv_users.set_search_column(1)
        self.tv_users.set_headers_visible(True)
        self.tv_users.set_headers_clickable(True)
    
        mygtk.add_columns_to_treeview(self.tv_users, USERS_TV_COLUMNS)

        # Connection User
        model    = self.tv_users.get_model()
        column   = self.tv_users.get_column(USERS_TV_COLUMN_CONN)
        renderer = column.get_cell_renderers()[0]
        renderer.connect('toggled', self.on_cell_toggled, model)

        # Username Edition
        column   = self.tv_users.get_column(USERS_TV_COLUMN_NAME)
        renderer = column.get_cell_renderers()[0]
        renderer.connect('edited',  self.on_cell_edited,  model)

        # Password Edition
        column   = self.tv_users.get_column(USERS_TV_COLUMN_PASS)
        renderer = column.get_cell_renderers()[0]
        renderer.connect('edited',  self.on_cell_edited,  model)
        
        # Connection
        adjustment = gtk.Adjustment(0, 1, 65535, 1, 10, 0)
        adjustment.set_value(22)
        self.sb_ssh_port.set_adjustment(adjustment)

        self.cb_connectable.connect('toggled', self.on_cb_connectable_toggled)

        self.cb_interface.set_sensitive(False)
        self.sb_ssh_port.set_sensitive(False)
        self.cb_previous_hop.set_sensitive(False)

        i = 0
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Server name'))
        self.attach(label,                 0, 1, i, i+1)
        hbox = gtk.HBox()
        hbox.set_spacing(10)
        hbox.pack_start(self.en_server_name)
        hbox.pack_start(self.cb_dead, expand=False)
        self.attach(hbox,                  1, 2, i, i+1)

        sw = gtk.ScrolledWindow()
        sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        sw.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        sw.add(self.tv_users)

        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Users'))
        self.attach(label,                 2, 3, i, i+1)

        hbox = gtk.HBox()
        hbox.set_spacing(5)
        hbox.pack_start(sw)

        vbbox = gtk.VButtonBox()
        vbbox.set_layout(gtk.BUTTONBOX_EDGE)
        vbbox.set_spacing(10)
        
        button = gtk.Button(stock=gtk.STOCK_ADD)
        button.connect('clicked', self.on_bt_add_user_clicked)
        vbbox.pack_start(button)

        button = gtk.Button(stock=gtk.STOCK_REMOVE)
        button.connect('clicked', self.on_bt_remove_user_clicked)
        vbbox.pack_start(button)

        button = mygtk.Button(stock=gtk.STOCK_DIALOG_AUTHENTICATION, label=_('S_how'))
        button.connect('clicked', self.on_bt_show_passwords_clicked)
        vbbox.pack_start(button)

        hbox.pack_start(vbbox, expand=False)

        self.attach(hbox,                 3, 4, i,  i+6)

        i += 1
        textview = gtk.TextView(self.bf_address)
        sw = gtk.ScrolledWindow()
        sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        sw.add(textview)
        
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Address'))
        self.attach(label,                 0, 1, i, i+1)
        self.attach(sw,                    1, 2, i, i+5)

        i += 5
        self.attach(self.cb_connectable,   0, 1, i, i+1)
        hbox = gtk.HBox()
        hbox.set_spacing(10)
        hbox.pack_start(self.cb_interface)
        hbox.pack_start(gtk.Label(':'),   expand=False)
        hbox.pack_start(self.sb_ssh_port, expand=False)
        self.attach(hbox,                  1, 2, i, i+1)

        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Previous hop'))
        self.attach(label,                 2, 3, i, i+1)
        self.attach(self.cb_previous_hop,  3, 4, i, i+1)

        i += 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Physical host'))
        self.attach(label,                 0, 1, i, i+1)
        self.attach(self.cb_physical_host,  1, 2, i, i+1)

        self.cb_ipmi_connectable.connect('toggled', self.on_cb_ipmi_connectable_toggled)
        self.attach(self.cb_ipmi_connectable, 3, 4, i, i+1)

        i += 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Use sudo for root'))
        self.attach(label,                 2, 3, i, i+1)
        self.attach(self.cb_sudo_root,  3, 4, i, i+1)

        i += 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('IPMI Interface'))
        self.attach(label,                 2, 3, i, i+1)
        self.attach(self.cb_ipmi_interface,  3, 4, i, i+1)
        self.cb_ipmi_interface.set_sensitive(False)

        i += 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('IPMI User name'))
        self.attach(label,                 2, 3, i, i+1)
        self.attach(self.en_ipmi_user_name, 3, 4, i, i+1)
        self.en_ipmi_user_name.set_sensitive(False)

        i += 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('IPMI User password'))
        self.attach(label,                 2, 3, i, i+1)

        hbox = gtk.HBox()
        hbox.set_spacing(10)
        hbox.pack_start(self.en_ipmi_user_pwd)
        self.en_ipmi_user_pwd.set_sensitive(False)

        button = mygtk.Button(stock=gtk.STOCK_DIALOG_AUTHENTICATION, label=_('S_how'))
        button.connect('clicked', self.on_bt_show_ipmi_password_clicked)
        hbox.pack_start(button, expand=False)


        self.attach(hbox,  3, 4, i, i+1)


    def on_cell_toggled(self, cell, path_string, model):
        
        column = cell.get_data("column")
        iter   = model.get_iter_from_string(path_string)
        value  = model.get_value(iter, column)
        if value == True:
            return True

        iter = model.get_iter_first()
        while iter:
            if str(model.get_path(iter)[0]) == path_string:
                model.set_value(iter, column, True)
            else:
                model.set_value(iter, column, False)

            iter = model.iter_next(iter)

    def on_cell_edited(self, cell, path_string, new_text, model):
        
        iter   = model.get_iter_from_string(path_string)
        path   = model.get_path(iter)[0]
        column = cell.get_data('column')

        model.set_value(iter, column, new_text)
        if column == USERS_TV_COLUMN_PASS:
            model.set_value(iter, USERS_TV_COLUMN_HIDE, '*' * len(new_text))

    def on_cb_connectable_toggled(self, checkbutton):
        
        if checkbutton.get_active():
            self.cb_interface.set_sensitive(True)
            self.sb_ssh_port.set_sensitive(True)
            self.cb_previous_hop.set_sensitive(True)

        else:
            self.cb_interface.set_sensitive(False)
            self.sb_ssh_port.set_sensitive(False)
            self.cb_previous_hop.set_sensitive(False)

    def on_cb_ipmi_connectable_toggled(self, checkbutton):

        if checkbutton.get_active():
            self.cb_ipmi_interface.set_sensitive(True)
            self.en_ipmi_user_name.set_sensitive(True)
            self.en_ipmi_user_pwd.set_sensitive(True)

        else:
            self.cb_ipmi_interface.set_sensitive(False)
            self.en_ipmi_user_name.set_sensitive(False)
            self.en_ipmi_user_pwd.set_sensitive(False)
 
    def on_bt_show_passwords_clicked(self, button):
        
        pass_column = self.tv_users.get_column(USERS_TV_COLUMN_PASS)
        hide_column = self.tv_users.get_column(USERS_TV_COLUMN_HIDE)

        if pass_column.get_visible():
            pass_column.set_visible(False)
            hide_column.set_visible(True)
            button.set_label(_('S_how'))
        else:
            pass_column.set_visible(True)
            hide_column.set_visible(False)
            button.set_label(_('_Hide'))

    def on_bt_show_ipmi_password_clicked(self, button):
        if self.en_ipmi_user_pwd.get_property('visibility'):
            self.en_ipmi_user_pwd.set_property('visibility',False)
            button.set_label(_('S_how'))
        else:
            self.en_ipmi_user_pwd.set_property('visibility',True)
            button.set_label(_('_Hide'))

    def on_bt_add_user_clicked(self, button):
        
        model = self.tv_users.get_model()
        model.append((-1,               # User ID
                      False,            # Connect user
                      _('New user'),    # Username
                      '',               # Password
                      ''))              # Password (hided)

    def on_bt_remove_user_clicked(self, button):
        
        selection  = self.tv_users.get_selection()
        model,iter = selection.get_selected()
        
        if iter:
            path = model.get_path(iter)[0]
            model.remove(iter)

    def get_active_user(self):

        model = self.tv_users.get_model()
        iter  = model.get_iter_first()
        id_user = -1
        while iter:
            selected = model.get_value(iter, USERS_TV_COLUMN_CONN)
            if selected:
                id_user = model.get_value(iter, USERS_TV_COLUMN_ID)
                iter = None
            else:
                iter = model.iter_next(iter)

        return id_user

    def set_server(self, server):

        self.id_server = server['id_server']

        self.en_server_name.set_text(server['server_name'])
        self.bf_address.set_text(server['address'])
        if server['dead'] == 't':
            self.cb_dead.set_active(True)

        # Users
        model = self.tv_users.get_model()
        model.clear()
        if server['users']:
            for user in server['users']:
                if server['id_connection_user'] == user['id_user']:
                    selected = True
                else:
                    selected = False

                model.append((int(user['id_user']),         # User ID
                              selected,                     # Connect user
                              user['name'],                 # Username
                              user['pass'],                 # Password
                              '*' * len(user['pass'])))     # Password (hided)
                              
            
        if server['connectable'] == 't':
            self.cb_connectable.set_active(True)

        if server['sudo_root'] == 't':
            self.cb_sudo_root.set_active(True)

        if server['ipmi_connectable'] == 't':
            self.cb_ipmi_connectable.set_active(True)

        self.cb_interface.set_active(server['id_connection_interface'])
        self.cb_ipmi_interface.set_active(server['id_ipmi_interface'])
        self.en_ipmi_user_name.set_text(server['ipmi_user_name'])
        self.en_ipmi_user_pwd.set_text(server['ipmi_user_password'])
        self.sb_ssh_port.set_value(int(server['ssh_port']))
        self.cb_previous_hop.set_active(server['id_server_previous_hop'])
        self.cb_physical_host.set_active(server['id_server_physical_host'])

    def get_server(self):

        if self.cb_dead.get_active():
            dead = 't'
        else:
            dead = 'f'

        if self.cb_sudo_root.get_active():
            sudo_root = 't'
        else:
            sudo_root = 'f'

        start_iter = self.bf_address.get_start_iter()
        end_iter   = self.bf_address.get_end_iter()
        address    = self.bf_address.get_text(start_iter, end_iter)
        
        # Users
        model = self.tv_users.get_model()
        users = []
        iter  = model.get_iter_first()
        while iter:
            users.append({ 'id_user': model.get_value(iter, USERS_TV_COLUMN_ID),
                           'name':    model.get_value(iter, USERS_TV_COLUMN_NAME),
                           'pass':    model.get_value(iter, USERS_TV_COLUMN_PASS) })
        
            iter = model.iter_next(iter)

        if self.cb_connectable.get_active():
            connectable = 't'
        else:
            connectable = 'f'

        if self.cb_ipmi_connectable.get_active():
            ipmi_connectable = 't'
        else:
            ipmi_connectable = 'f'

        server = { 'name':                    self.en_server_name.get_text(),
                   'dead':                    dead,
                   'address':                 address,
                   'connectable':             connectable,
                   'id_server_previous_hop':  self.cb_previous_hop.get_active(),
                   'id_server_physical_host': self.cb_physical_host.get_active(),
                   'id_connection_interface': self.cb_interface.get_active(),
                   'id_ipmi_interface':       self.cb_ipmi_interface.get_active(),
                   'sudo_root':               sudo_root,
                   'ipmi_user_name':          self.en_ipmi_user_name.get_text(),
                   'ipmi_user_password':      self.en_ipmi_user_pwd.get_text(),
                   'ipmi_connectable':        ipmi_connectable,
                   'id_connection_user':      self.get_active_user(),
                   'ssh_port':                self.sb_ssh_port.get_value(),
                   'users':                   users }
        return server

    def on_update_server_clicked(self, button):

        server = self.get_server()
        connector.eemyle_connector.send_request('eemyle.update_server', self.id_server, server)



USAGES_TV_COLUMNS = (
   # Name            Renderer                Renderer attributes    col_id    visibility   sort_id
    (_(''),          gtk.CellRendererToggle, (),                    2,        False,       2),        # Checked
    (_('Usage'),     gtk.CellRendererText,   (),                    3,        True,        3),        # Usage name
    )

class ServerUsage(common.ServerTable):

    def __init__(self, id_customer, server=None, display_lists=None):
        
        common.ServerTable.__init__(self)

        self.id_customer = id_customer
        self.id_server   = -1

        self.cb_os             = mygtk.SimpleComboBox(display_lists['oses'])
        self.cb_security_check = gtk.CheckButton(_('Security'))
        self.en_usage          = gtk.Entry()

        # Usages
        model = gtk.ListStore(
            gobject.TYPE_BOOLEAN, # Visibility flag (internal)
            gobject.TYPE_STRING,  # Usage ID
            gobject.TYPE_BOOLEAN, # Checked
            gobject.TYPE_STRING,  # Usage name
            )


        self.__show_all_usages = False
        modelfilter = model.filter_new()
        self.tv_usages = gtk.TreeView(modelfilter)

        self.feed_objects(display_lists)
        self.construct_frame()
        if server:
            self.set_server(server)

    def feed_objects(self, display_lists):
        # Usages
        modelfilter = self.tv_usages.get_model()
        model = modelfilter.get_model()
        for id_usage, usage in display_lists['usages']:
            iter = model.append((False, id_usage, False, usage))
        modelfilter.set_visible_column(0) # Setting visible column after data is faster
        modelfilter.refilter()


    def construct_frame(self):
        # Usages
        self.tv_usages.set_rules_hint(True)
        self.tv_usages.set_search_column(3)
        self.tv_usages.set_headers_visible(True)
        self.tv_usages.set_headers_clickable(True)

        mygtk.add_columns_to_treeview(self.tv_usages, USAGES_TV_COLUMNS)

        # Buttons
        column = self.tv_usages.get_column(0)
        renderer = column.get_cell_renderers()[0]
        renderer.connect('toggled', self.on_renderer_toggled, self.tv_usages.get_model(),    2)

        i = 0
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Operating System'))
        self.attach(label,                 0, 1, i, i+1)
        hbox = gtk.HBox()
        hbox.set_spacing(10)
        hbox.pack_start(self.cb_os)
        hbox.pack_start(self.cb_security_check, expand=False)
        self.attach(hbox,                  1, 2, i, i+1)

        i += 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Usage description'))
        self.attach(label,                 0, 1, i, i+1)
        self.attach(self.en_usage,         1, 2, i, i+1)

        i += 1
        button = mygtk.Button(stock=gtk.STOCK_DIALOG_AUTHENTICATION, label=_('Show all'))
        button.connect('clicked', self.on_bt_visibility_clicked, 'usages')

        alg    = gtk.Alignment(1.0, 0.5)
        alg.add(button)
        self.attach(alg,                   0, 1, i, i+5)

        sw = gtk.ScrolledWindow()
        sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        sw.add(self.tv_usages)
        self.attach(sw,                    1, 2, i, i+5)

        # FIXME: don't work
        # Force widgets to have the same size
        self.tv_usages.set_size_request(-1, 150)

        sg = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
        sg.add_widget(self.tv_usages)

    def on_bt_visibility_clicked(self, button, thing):

        if thing == 'usages':
            self.__show_all_usages    = not self.__show_all_usages
            show_all = self.__show_all_usages
            treeview = self.tv_usages

        check_column = treeview.get_column(0)
        modelfilter = treeview.get_model()
        model = modelfilter.get_model()
        iter = model.get_iter_root()
        if show_all:
            check_column.set_visible(True)
            while iter:
                model.set_value(iter, 0, True)
                iter = model.iter_next(iter)
            button.set_label(_('Show selected'))
        else:
            check_column.set_visible(False)
            while iter:
                model.set_value(iter, 0, model.get_value(iter, 2))
                iter = model.iter_next(iter)
            button.set_label(_('Show all'))

        modelfilter.refilter()

    def on_renderer_toggled(self, cell, path, modelfilter, column):
        
        iter  = modelfilter.get_iter((int(path),))
        iter  = modelfilter.convert_iter_to_child_iter(iter)
        model = modelfilter.get_model()

        value = model.get_value(iter, column)
        value = not value

        model.set_value(iter, column, value)

    def set_server(self, server):

        self.id_server = server['id_server']

        self.cb_os.set_active(server['id_os'])
        if server['security_check'] == 't':
            self.cb_security_check.set_active(True)
        self.en_usage.set_text(server['usage'].replace('\r\n', ' - '))
        
        # Usages
        if server['usages']:
            usage_ids = server['usages']

            modelfilter = self.tv_usages.get_model()
            model       = modelfilter.get_model()
            iter        = model.get_iter_root()
            while iter:
                usage_id = model.get_value(iter, 1)
                # Checked
                if usage_id in usage_ids:
                    model.set(iter, 0, True, 2, True)
                
                iter = model.iter_next(iter)


    def get_server(self):
        
        # Security
        if self.cb_security_check.get_active():
            security_check = 't'
        else:
            security_check = 'f'

        # Usages:
        usages      = []
        modelfilter = self.tv_usages.get_model()
        model       = modelfilter.get_model()
        iter = model.get_iter_root()
        while iter:
            if model.get_value(iter, 2):
                usages.append(model.get_value(iter, 1))

            iter = model.iter_next(iter)

        server = { 'id_os':          self.cb_os.get_active(),
                   'security_check': security_check,
                   'usage':          self.en_usage.get_text(),
                   'usages':         usages }

        return server

    def on_update_server_clicked(self, button):

        server = self.get_server()
        connector.eemyle_connector.send_request('eemyle.update_server', self.id_server, server)


class ServerGeneral(common.ServerTab):
    
    def __init__(self):

        common.ServerTab.__init__(self, _('General'))

        self.id_customer = ''

    def set_server(self, server=None):

        self.remove_children()
        self.set_sensitive(False)

        if not server:
            self.id_customer = ''
            return True

        self.id_customer = server['id_customer']
        self.set_sensitive(True)

        # Informations ;-)
        users = ServerInformations(self.id_customer, server=server, display_lists=self.display_lists)

        frame = common.HardwareFrame(_('Informations'), has_remove=False)
        frame.bt_save.connect('clicked', users.on_update_server_clicked)
        frame.set_sensitive(True)
        frame.pack_start(users, expand=False)

        self.add_child(frame, expand=False)

        # Usage
        other = ServerUsage(self.id_customer, server=server, display_lists=self.display_lists)

        frame = common.HardwareFrame(_('Usage'), has_remove=False)
        frame.bt_save.connect('clicked', other.on_update_server_clicked)
        frame.set_sensitive(True)
        frame.pack_start(other, expand=False)

        self.add_child(frame, expand=False)

        self.show_all()

