#!/usr/bin/env python
#
# GEemyle: Server View
# Copyright (C) 2003-2006 Easter-eggs <info@easter-eggs.com>
# Authors:
#      Cyril Lacoux (Yack) <clacoux@easter-eggs.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

__author__  = "Cyril Lacoux [Yack] <clacoux@easter-eggs.com>"
__cvsid__   = "$Id: server.py 102 2006-06-01 19:34:20Z yack $"
__version__ = "$Revision: 102 $"[11:-2]
__all__     = ["ServerMainboard"]


import pygtk
pygtk.require('2.0')
import gtk

import geemyle.eemyle_connector as connector
import geemyle.gettext as gettext
import geemyle.gui.mygtk        as mygtk
import common

_ = gettext._

PROCESSOR_QTIES = [ '1', '2', '4', '8', '16', '32', '64' ]

class ServerMainboardComponents(common.ServerTable):
    
    def __init__(self, id_customer, server=None, display_lists=None):

        common.ServerTable.__init__(self)

        self.id_customer = id_customer
        self.id_server   = -1

        self.cb_manufacturer   = mygtk.SimpleComboBox(display_lists['manufacturers'])
        self.en_model          = gtk.Entry()
        self.cb_processor      = mygtk.SimpleComboBox(display_lists['processors'])
        self.cb_processor_qty  = gtk.combo_box_new_text()
        self.sb_processor_freq = gtk.SpinButton()
        self.sb_memory_qty     = gtk.SpinButton()

        self.feed_objects()
        self.construct_frame()
        if server:
            self.set_server(server)

    def feed_objects(self):
        
        for processor_qty in PROCESSOR_QTIES:
            self.cb_processor_qty.append_text(processor_qty)

    def construct_frame(self):


        adjustment = gtk.Adjustment(0, 0, 10000, 100, 1000, 0)
        self.sb_processor_freq.set_adjustment(adjustment)

        adjustment = gtk.Adjustment(0, 0, 100000, 128, 512, 0)
        self.sb_memory_qty.set_adjustment(adjustment)

        i  = 0
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Manufacturer'))
        self.attach(label,                  0, 1, i, i+1)
        self.attach(self.cb_manufacturer,   1, 2, i, i+1)

        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Model'))
        self.attach(label,                  2, 3, i, i+1)
        self.attach(self.en_model,          3, 4, i, i+1)

        i += 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Memory/qty (MB)'))
        self.attach(label,                  0, 1, i, i+1)
        hbox = gtk.HBox()
        hbox.set_spacing(10)
        hbox.pack_start(self.sb_memory_qty)
        self.attach(hbox,                   1, 2, i, i+1)

        i += 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Processor'))
        self.attach(label,                  0, 1, i, i+1)
        self.attach(self.cb_processor,      1, 2, i, i+1)

        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Freq. (MHz)/qty'))
        self.attach(label,                  2, 3, i, i+1)
        hbox = gtk.HBox()
        hbox.set_spacing(10)
        hbox.pack_start(self.sb_processor_freq)
        hbox.pack_start(self.cb_processor_qty)
        self.attach(hbox,                   3, 4, i, i+1)

        # To force the size of theses widgets
        size_group = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
        size_group.add_widget(self.cb_processor_qty)

    def set_server(self, server):
        
        self.id_server = server['id_server']

        self.cb_manufacturer.set_active(server['id_manufacturer'])
        self.en_model.set_text(server['model'])
        self.cb_processor.set_active(server['id_processor'])
        self.sb_processor_freq.set_value(int(server['processor_freq']))
        processor_qty = server['processor_qty']
        if processor_qty in PROCESSOR_QTIES:
            self.cb_processor_qty.set_active(PROCESSOR_QTIES.index(processor_qty))
        self.sb_memory_qty.set_value(int(server['memory_qty']))
        
    def get_server(self):
        
        server = { 'id_manufacturer': self.cb_manufacturer.get_active(),
                   'model':           self.en_model.get_text(),
                   'id_processor':    self.cb_processor.get_active(),
                   'processor_freq':  self.sb_processor_freq.get_value(),
                   'processor_qty':   self.cb_processor_qty.get_active_text(),
                   'memory_qty':      self.sb_memory_qty.get_value() }

        return server

    def on_add_server_activate(self):

        dialog = common.AddDialog(self, _('add a server'))

        refresh = False
        loop = True
        while loop:
            response = dialog.run()
            if response == gtk.RESPONSE_OK:
                server = self.get_server()
                
                result, data = connector.eemyle_connector.send_direct_request('eemyle.add_server', self.id_customer, server)
                if result and data:
                    loop = False
                    refresh = True
            else:
                loop = False

        dialog.destroy()

        if refresh:
            connector.eemyle_connector.send_request('eemyle.get_servers_by_customer_id', self.id_customer)

    def on_update_server_clicked(self, button):

        server = self.get_server()
        connector.eemyle_connector.send_request('eemyle.update_server', self.id_server, server)


        
class ServerMainboard(common.ServerTab):

    def __init__(self):

        common.ServerTab.__init__(self, _('Mainboard'))

        self.id_customer = ''

    def set_server(self, server=None):
        
        self.remove_children()
        self.set_sensitive(False)

        if not server:
            self.id_customer = ''
            return True

        self.id_customer = server['id_customer']
        self.set_sensitive(True)

        mb = ServerMainboardComponents(self.id_customer, server=server, display_lists=self.display_lists)

        frame = common.HardwareFrame(_('Mainboard'), has_remove=False)
        frame.bt_save.connect('clicked', mb.on_update_server_clicked)
        frame.set_sensitive(True)
        frame.pack_start(mb, expand=False)

        self.add_child(frame, expand=False)

        self.show_all()

    def on_add_server_activate(self, id_customer):
        
        mb = ServerMainboardComponents(id_customer, display_lists=self.display_lists)
        mb.on_add_server_activate()


