#!/usr/bin/env python
#
# GEemyle: Server View
# Copyright (C) 2003-2006 Easter-eggs <info@easter-eggs.com>
# Authors:
#      Cyril Lacoux (Yack) <clacoux@easter-eggs.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

__author__  = "Cyril Lacoux [Yack] <clacoux@easter-eggs.com>"
__cvsid__   = "$Id: server.py 102 2006-06-01 19:34:20Z yack $"
__version__ = "$Revision: 102 $"[11:-2]
__all__     = ["ServerTickets"]


import pygtk
pygtk.require('2.0')
import gtk
import gobject

import geemyle.gettext   as gettext
import geemyle.gui.mygtk as mygtk
import common

_ = gettext._

TICKETS_TV_COLUMNS   = ( (_('Number'),           gtk.CellRendererText,   (('xalign', 1.0), ),   0,        True,        0),
                         (_('Subject'),          gtk.CellRendererText,   (),                    1,        True,        1),
                         (_('Date'),             gtk.CellRendererText,   (),                    2,        True,        2),
                         (_('Last updated'),     gtk.CellRendererText,   (),                    3,        True,        3),
                         (_('Status'),           gtk.CellRendererText,   (),                    4,        True,        4),
                         (_('Owner'),            gtk.CellRendererText,   (),                    5,        True,        5) )


class ServerTickets(common.ServerTab):

    def __init__(self):

        common.ServerTab.__init__(self, _('Tickets'), has_vp=False, widget_class=gtk.TreeView)

        # To open a webbrowser
        self.url_callback = None

        # Tickets
        model = gtk.ListStore(gobject.TYPE_INT,    # #ticket
                              gobject.TYPE_STRING, # Subject
                              gobject.TYPE_STRING, # Date
                              gobject.TYPE_STRING, # Last updated
                              gobject.TYPE_STRING, # Status
                              gobject.TYPE_STRING) # Owner

        self.tv_tickets = self.widget

        self.tv_tickets.set_model(model)
        self.tv_tickets.set_rules_hint(True)
        self.tv_tickets.set_search_column(1)
        self.tv_tickets.set_headers_visible(True)
        self.tv_tickets.set_headers_clickable(True)
        self.tv_tickets.connect('row-activated', self.on_tv_tickets_row_activated)

        mygtk.add_columns_to_treeview(self.tv_tickets, TICKETS_TV_COLUMNS)

        self.add_child(self.tv_tickets)

    def on_tv_tickets_row_activated(self, treeview, path, view_column):
        """
        Connect to the current selected server when the user double click on it
        """

        model  = treeview.get_model()
        iter   = model.get_iter(path)
        number = model.get_value(iter, 0)

        if self.url_callback:
            self.url_callback(number)

    def set_url_callback(self, callback):

        self.url_callback = callback

    def set_tickets(self, tickets=None):
        
        model = self.tv_tickets.get_model()
        model.clear()

        if not tickets:
            return True
    
        for ticket in tickets:
            iter = model.append((int(ticket['number']),
                                 ticket['subject'],
                                 ticket['date'],
                                 ticket['last_updated'],
                                 ticket['status'],
                                 ticket['owner']))

        self.tv_tickets.columns_autosize()


