#!/usr/bin/env python
#
# GEemyle: Eemyle View
# Copyright (C) 2003-2006 Easter-eggs <info@easter-eggs.com>
# Authors:
#      Cyril Lacoux (Yack) <clacoux@easter-eggs.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

__author__  = "Cyril Lacoux [Yack] <clacoux@easter-eggs.com>"
__cvsid__   = "$Id$"
__version__ = "$Revision$"[11:-2]
__all__     = ["StatisticsView"]

import pygtk
pygtk.require('2.0')
import gtk
import gobject

import geemyle.config    as config
import geemyle.gettext   as gettext
import geemyle.gui.mygtk as mygtk

_ = gettext._


                             # Name               # Renderer              # Renderer attributes  # col_id  # visibility # sort_id
OSES_COUNT_TV_COLUMNS   = ( (_('OS name'),        gtk.CellRendererText,   (),                    0,        True,       0),
                            (_('Servers count'),  gtk.CellRendererText,   (('xalign', 1.0), ),   1,        True,       1))

                             # Name               # Renderer              # Renderer attributes  # col_id  # visibility # sort_id
USAGES_COUNT_TV_COLUMNS = ( (_('Usage name'),     gtk.CellRendererText,   (),                    0,        True,       0),
                            (_('Servers count'),  gtk.CellRendererText,   (('xalign', 1.0), ),   1,        True,       1))

class StatisticsView(gtk.Dialog):

    def __init__(self, statistics):
        
        gtk.Dialog.__init__(self, flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT, buttons=(gtk.STOCK_CLOSE, gtk.RESPONSE_CANCEL))

        self.set_title('%s - Database statistics' % config.PACKAGE)
        self.set_icon_from_file(config.APP_ICON_FILEPATH)

        self.lb_servers_count             = gtk.Label()
        self.lb_alive_servers_count       = gtk.Label()
        self.lb_dead_servers_count        = gtk.Label()
        self.lb_avg_servers_per_customer  = gtk.Label()
        self.lb_max_servers_per_customer  = gtk.Label()
        self.lb_connectable_servers_count = gtk.Label()
        self.lb_security_servers_count    = gtk.Label()

        model = gtk.ListStore(gobject.TYPE_STRING,
                              gobject.TYPE_INT)

        self.tv_oses_count                = gtk.TreeView(model)

        model = gtk.ListStore(gobject.TYPE_STRING,
                              gobject.TYPE_INT)

        self.tv_usages_count              = gtk.TreeView(model)

        self.construct_frame()
        if statistics:
            self.set_statistics(statistics)

    def construct_frame(self):

        table = gtk.Table()
        table.set_border_width(5)
        table.set_row_spacings(5)
        table.set_col_spacings(20)

        self.lb_servers_count.set_alignment(1.0, 0.5)
        self.lb_alive_servers_count.set_alignment(1.0, 0.5)
        self.lb_dead_servers_count.set_alignment(1.0, 0.5)
        self.lb_avg_servers_per_customer.set_alignment(1.0, 0.5)
        self.lb_max_servers_per_customer.set_alignment(1.0, 0.5)
        self.lb_connectable_servers_count.set_alignment(1.0, 0.5)
        self.lb_security_servers_count.set_alignment(1.0, 0.5)

        self.lb_servers_count.set_selectable(True)
        self.lb_alive_servers_count.set_selectable(True)
        self.lb_dead_servers_count.set_selectable(True)
        self.lb_avg_servers_per_customer.set_selectable(True)
        self.lb_max_servers_per_customer.set_selectable(True)
        self.lb_connectable_servers_count.set_selectable(True)
        self.lb_security_servers_count.set_selectable(True)

        i = 0
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Total servers count'))
        label.set_selectable(True)
        table.attach(label,                             0, 1, i, i+1)
        table.attach(self.lb_servers_count,             1, 2, i, i+1)

        # Oses count
        self.tv_oses_count.set_rules_hint(True)
        self.tv_oses_count.set_search_column(1)
        self.tv_oses_count.set_headers_visible(False)

        mygtk.add_columns_to_treeview(self.tv_oses_count, OSES_COUNT_TV_COLUMNS)

        sw = gtk.ScrolledWindow()
        sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        sw.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        sw.set_border_width(5)
        sw.add(self.tv_oses_count)

        frame = gtk.Frame(label=_('OSes count'))
        frame.add(sw)
        table.attach(frame,                             2, 3, i, i+7)
        
        # Usages count
        self.tv_usages_count.set_rules_hint(True)
        self.tv_usages_count.set_search_column(1)
        self.tv_usages_count.set_headers_visible(False)

        mygtk.add_columns_to_treeview(self.tv_usages_count, USAGES_COUNT_TV_COLUMNS)

        sw = gtk.ScrolledWindow()
        sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        sw.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        sw.set_border_width(5)
        sw.add(self.tv_usages_count)

        frame = gtk.Frame(label=_('Usages count'))
        frame.add(sw)
        table.attach(frame,                             3, 4, i, i+7)

        i += 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Alive servers count'))
        label.set_selectable(True)
        table.attach(label,                             0, 1, i, i+1)
        table.attach(self.lb_alive_servers_count,       1, 2, i, i+1)
        
        i += 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Dead servers count'))
        label.set_selectable(True)
        table.attach(label,                             0, 1, i, i+1)
        table.attach(self.lb_dead_servers_count,        1, 2, i, i+1)
        
        i += 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Average servers count per customer'))
        label.set_selectable(True)
        table.attach(label,                             0, 1, i, i+1)
        table.attach(self.lb_avg_servers_per_customer,  1, 2, i, i+1)
        
        i += 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Maximum servers count per customer'))
        label.set_selectable(True)
        table.attach(label,                             0, 1, i, i+1)
        table.attach(self.lb_max_servers_per_customer,  1, 2, i, i+1)
        
        i += 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Connectable server count'))
        label.set_selectable(True)
        table.attach(label,                             0, 1, i, i+1)
        table.attach(self.lb_connectable_servers_count, 1, 2, i, i+1)
        
        i += 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Security servers count'))
        label.set_selectable(True)
        table.attach(label,                             0, 1, i, i+1)
        table.attach(self.lb_security_servers_count,    1, 2, i, i+1)
        
        self.vbox.pack_start(table)

        
    def set_statistics(self, statistics):

        self.lb_servers_count.set_text(str(statistics['servers_count']))
        self.lb_alive_servers_count.set_text(str(statistics['alive_servers_count']))
        self.lb_dead_servers_count.set_text(str(statistics['dead_servers_count']))
        self.lb_avg_servers_per_customer.set_text(str(statistics['avg_servers_per_customer']))
        self.lb_max_servers_per_customer.set_text(str(statistics['max_servers_per_customer']))
        self.lb_connectable_servers_count.set_text(str(statistics['connectable_servers_count']))
        self.lb_security_servers_count.set_text(str(statistics['security_servers_count']))

        # Oses count
        model = self.tv_oses_count.get_model()
        model.clear()
        oses = statistics['oses_count'].keys()
        oses.sort()
        for os_name in oses:
            model.append((os_name,
                          int(statistics['oses_count'][os_name])))

        # Usages count
        model = self.tv_usages_count.get_model()
        model.clear()
        usages = statistics['usages_count'].keys()
        usages.sort()
        for usage_name in usages:
            model.append((usage_name,
                          int(statistics['usages_count'][usage_name])))

