#!/usr/bin/env python
#
# GEemyle: Main Window
# Copyright (C) 2003-2006 Easter-eggs <info@easter-eggs.com>
# Authors:
#      Cyril Lacoux (Yack) <clacoux@easter-eggs.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

__author__  = "Cyril Lacoux [Yack] <clacoux@easter-eggs.com>"
__cvsid__   = "$Id$"
__version__ = "$Revision$"[11:-2]
__all__     = ["GEemyleUI"]

license = """This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details."""

import pygtk
pygtk.require('2.0')
import gtk
import gobject
import re

import geemyle.config          as config
import geemyle.gettext         as gettext
import geemyle.gui.mygtk       as mygtk
import geemyle.gui.terminal    as terminal
import geemyle.gui.eemyle      as eemyle

_ = gettext._

# For load configuration
_all_windows = []

window_actions = [
  ('FileMenu',     None, '_File'),
  ('Quit',                  gtk.STOCK_QUIT,         '_Quit',                        '<control><shift>Q',     'Quit application',              'on_quit_activate'),

  ('ConnectionMenu', None, '_Connection'),
  ('TerminalConnect',       gtk.STOCK_CONNECT,      'Co_nnect',                     '<control><shift>C',     'Connect to the server',         'on_terminal_connect_activate'),
  ('TerminalTunnel',        gtk.STOCK_NETWORK,      'Open _tunnel',                 '<control><shift>T',     'Open a tunnel with the server', 'on_terminal_tunnel_activate'),
  ('TerminalX11',           gtk.STOCK_SELECT_COLOR, '_X11 connection',              '<control><shift>X',     'Open an X11 connection',        'on_terminal_x11_activate'),
  ('TerminalIPMI',          gtk.STOCK_JUMP_TO,      'Co_nnect IPMI console',        None,                    'Connect to IPMI console',       'on_terminal_ipmi_activate'),
  ('KillCurrentIPMITerminal', gtk.STOCK_DISCONNECT, 'Kill current IPMI connection', None,                    'Kill current IPMI connection',  'on_kill_ipmi_activate'),
  ('RebootByIPMI',          gtk.STOCK_EXECUTE,      'Reboot server by IPMI',        None,                    'Reboot server by IPMI',         'on_reboot_by_ipmi_activate'),

  ('TerminalMenu', None, '_Terminal'),
  ('CustomerRefresh',       gtk.STOCK_REFRESH,      '_Refresh',                     None,                    'Refresh the cutomers\' list',   'on_customer_refresh_activate'),
  ('CustomerViewContacts',  gtk.STOCK_FIND,         '_View contacts',               None,                    'View the cutomers\' contacts',  'on_customer_view_contacts_activate'),
  ('ServerRemove',          gtk.STOCK_REMOVE,       '_Remove',                      None,                    'Remove the server',             'on_server_remove_activate'),
  ('ServerAdd',             gtk.STOCK_ADD,          '_Add',                         None,                    'Add a server',                  'on_server_add_activate'),
  ('ServerDuplicate',       gtk.STOCK_COPY,         '_Duplicate',                   None,                    'Duplicate the server',          'on_server_duplicate_activate'),
  ('DatabaseStatistics',    gtk.STOCK_PASTE,        'Database _Statistics',         None,                    'Database statistics',           'on_database_statistics_activate'),
  ('TerminalReset',         gtk.STOCK_CLEAR,        'Reset the _terminal',          None,                    'Reset the terminal',            'on_terminal_reset_activate'),
  ('TerminalNew',           gtk.STOCK_NEW,          '_New terminal',                '<control><shift>N',     'New terminal',                  'on_terminal_new_activate'),
  ('TerminalClose',         gtk.STOCK_CLOSE,        '_Close the terminal',          '<control><shift>W',     'Close the terminal',            'on_terminal_close_activate'),
  ('TerminalSendFiles',     gtk.STOCK_GOTO_TOP,     '_Send file(s)...',             None,                    'Send file(s) to the terminal',  'on_terminal_send_files_activate'),
  ('TerminalStopFeedChild', gtk.STOCK_CANCEL,       'Stop _feeding the terminal',   None,                    'Stop feeding the terminal',     'on_terminal_stop_feed_child_activate'),
  ('TerminalDuplicate',     gtk.STOCK_COPY,         '_Duplicate the terminal',      '<control><shift>D',     'Duplicate the terminal',        'on_terminal_duplicate_activate'),
  ('TerminalDetach',        gtk.STOCK_ZOOM_FIT,     'D_etach the terminal',         '<control><shift>E',     'Detach the terminal',           'on_terminal_detach_activate'),
  ('TerminalRename',        gtk.STOCK_EDIT,         '_Rename the terminal',         '<control><shift>R',     'Rename the terminal',           'on_terminal_rename_activate'),
  ('TerminalBecomeRoot',    gtk.STOCK_PROPERTIES,   'Become _root',                 '<control><shift>U',     'Become root',                   'on_terminal_become_root_activate'),

  ('TerminalEncodingsMenu', None, '_Encodings'),
  ('TerminalWidthsMenu',    None, 'Terminal _width'),

  ('TerminalCommandsMenu', None, 'Co_mmands'),

  ('ViewMenu',     None, '_View'),
  ('ChangeCurrentTabLeft',  gtk.STOCK_GO_BACK,      'Change current tab _left',     '<shift>Left',           'Change current tab left',       'on_change_current_tab_left_activate'),
  ('ChangeCurrentTabRight', gtk.STOCK_GO_FORWARD,   'Change current tab _right',    '<shift>Right',          'Change current tab left',       'on_change_current_tab_right_activate'),
  ('MoveTabLeft',           gtk.STOCK_GO_BACK,      'Move tab _left',               '<control><shift>Left',  'Move tab left',                 'on_move_tab_left_activate'),
  ('MoveTabRight',          gtk.STOCK_GO_FORWARD,   'Move tab _right',              '<control><shift>Right', 'Move tab left',                 'on_move_tab_right_activate'),

  ('OptionMenu',   None, '_Option'),
  ('GPreferences',          gtk.STOCK_PREFERENCES,  '_GEemyle preferences',         '<control><shift>G',     'GEemyle preferences',           'on_geemyle_preferences_activate'),
  ('TPreferences',          gtk.STOCK_PREFERENCES,  '_Terminal preferences',        '<control><shift>T',     'Terminal preferences',          'on_terminal_preferences_activate'),

  ('HelpMenu',     None, '_Help'),
  ('About',                 gtk.STOCK_ABOUT,        '_About',                       None,                    'About application',             'on_about_activate'),
]

toggle_actions = [
  ('FullScreen',            None,                   '_Full screen mode',            '<control><shift>F',     'Full screen mode',              'on_fullscreen_toggled',              config.get('geemyle',  'full_screen')),
  ('ShowMenubar',           None,                   'Show _menubar',                '<control><shift>M',     'Show menubar',                  'on_show_menubar_toggled',            config.get('geemyle',  'show_menubar')),
  ('ShowToolbar',           None,                   'Show tool_bar',                '<control><shift>B',     'Show toolbar',                  'on_show_toolbar_toggled',            config.get('geemyle',  'show_toolbar')),
  ('TerminalShowScrollbar', None,                   'Show terminal _scrollbar',     '<control><shift>S',     'Show terminal scrollbar',       'on_terminal_show_scrollbar_toggled', config.get('terminal', 'show_scrollbar')),
  ('TerminalBroadcast',     gtk.STOCK_COLOR_PICKER, 'Send _input to all terminals', '<control><shift>I',     'Send input to all terminals',   'on_terminal_broadcast_toggled',      False),
]

radios_actions = {}

ui_string = ""
def init():
    global window_actions, radios_actions, ui_string

    # Constructing the defined commands
    terminal_menus = {}
    terminal_menus['terminal_commands_menu']  = ''
    terminal_menus['terminal_commands_tool']  = ''
    terminal_menus['terminal_encodings_menu'] = ''
    terminal_menus['terminal_widths_menu']    = ''
    
    # Commands
    commands = config.get('terminal', 'commands')
    i = 0
    if commands:
        for stock_id, label, accelerator, tooltip, command in commands:
            name = 'TerminalCommand_%d' % i
            
            window_actions.append((name, stock_id, label, accelerator, tooltip, 'on_terminal_send_command_activate'))
            terminal_menus['terminal_commands_menu'] += "\n<menuitem action='%s'/>" % name
            terminal_menus['terminal_commands_tool'] += "\n<toolitem action='%s'/>" % name
            i += 1
        
    # Encodings
    radio_actions = []
    i = 0
    for label, encoding in terminal.ENCODINGS:
        name = 'TerminalEncoding_%d' % i
        radio_actions.append((name, None, _(label), None, _('Set encoding to %s') % label))

        terminal_menus['terminal_encodings_menu'] += "\n<menuitem action='%s'/>" % name
        i += 1

    radios_actions['on_terminal_set_encoding_changed'] = radio_actions

    # Widths
    radio_actions = []
    i = 0
    for label, width in terminal.WIDTHS:
        name = 'TerminalWidth_%d' % i
        radio_actions.append((name, None, _(label), None, _('Set terminal size to %s') % label))

        terminal_menus['terminal_widths_menu'] += "\n<menuitem action='%s'/>" % name
        i += 1
        
    radios_actions['on_terminal_set_width_changed'] = radio_actions

    ui_string = """<ui>
      <menubar name='Menubar'>
        <menu action='FileMenu'>
          <menuitem action='TerminalNew'/>
          <separator/>
          <menuitem action='Quit'/>
        </menu>
        <menu action='ConnectionMenu'>
          <menuitem action='TerminalConnect'/>
          <menuitem action='TerminalTunnel'/>
          <menuitem action='TerminalX11'/>
          <menuitem action='TerminalIPMI'/>
          <menuitem action='KillCurrentIPMITerminal'/>
          <menuitem action='RebootByIPMI'/>
        </menu>
        <menu action='TerminalMenu'>
          <menuitem action='TerminalNew'/>
          <menuitem action='TerminalClose'/>
          <menuitem action='TerminalSendFiles'/>
          <separator/>
          <menuitem action='TerminalReset'/>
          <menu action='TerminalEncodingsMenu'>
            %(terminal_encodings_menu)s
          </menu>
          <menu action='TerminalWidthsMenu'>
            %(terminal_widths_menu)s
          </menu>
          <separator/>
          <menuitem action='TerminalStopFeedChild'/>
          <menuitem action='TerminalDuplicate'/>
          <menuitem action='TerminalDetach'/>
          <menuitem action='TerminalRename'/>
          <menuitem action='TerminalBroadcast'/>
          <menu action='TerminalCommandsMenu'>
            <menuitem action='TerminalBecomeRoot'/>
            %(terminal_commands_menu)s
          </menu>
        </menu>
        <menu action='ViewMenu'>
          <menuitem action='MoveTabLeft'/>
          <menuitem action='MoveTabRight'/>
        </menu>
        <menu action='OptionMenu'>
          <menuitem action='ShowMenubar'/>
          <menuitem action='ShowToolbar'/>
          <menuitem action='TerminalShowScrollbar'/>
          <separator/>
          <menuitem action='FullScreen'/>
          <separator/>
          <menuitem action='GPreferences'/>
          <menuitem action='TPreferences'/>
        </menu>
        <menu action='HelpMenu'>
          <menuitem action='About'/>
        </menu>
      </menubar>
    
      <toolbar name='EemyleToolBar'>
        <toolitem action='TerminalNew'/>
        <separator/>
        <toolitem action='CustomerRefresh'/>
        <toolitem action='CustomerViewContacts'/>
        <separator/>
        <toolitem action='ServerRemove'/>
        <toolitem action='ServerAdd'/>
        <toolitem action='ServerDuplicate'/>
        <separator/>
        <toolitem action='DatabaseStatistics'/>
        <separator/>
        <toolitem action='TerminalIPMI'/>
        <toolitem action='TerminalX11'/>
        <toolitem action='TerminalTunnel'/>
        <toolitem action='TerminalConnect'/>
      </toolbar>
    
      <toolbar name='TerminalToolBar'>
        <toolitem action='TerminalNew'/>
        <toolitem action='TerminalClose'/>
        <toolitem action='TerminalSendFiles'/>
        <separator/>
        <toolitem action='TerminalStopFeedChild'/>
        <toolitem action='TerminalDuplicate'/>
        <toolitem action='TerminalDetach'/>
        <toolitem action='TerminalRename'/>
        <toolitem action='TerminalBroadcast'/>
        <separator/>
        <toolitem action='TerminalBecomeRoot'/>
        %(terminal_commands_tool)s
      </toolbar>
    
      <popup name='TerminalPopup'>
        <menuitem action='TerminalNew'/>
        <menuitem action='TerminalClose'/>
        <menuitem action='TerminalSendFiles'/>
        <separator/>
        <menuitem action='TerminalStopFeedChild'/>
        <menuitem action='TerminalDuplicate'/>
        <menuitem action='TerminalDetach'/>
        <menuitem action='TerminalRename'/>
        <menuitem action='TerminalBroadcast'/>
        <separator/>
        <menuitem action='TerminalBecomeRoot'/>
        %(terminal_commands_menu)s
        <separator/>
        <menuitem action='ChangeCurrentTabLeft'/>
        <menuitem action='ChangeCurrentTabRight'/>
        <separator/>
        <menuitem action='FullScreen'/>
        <menuitem action='ShowMenubar'/>
        <menuitem action='ShowToolbar'/>
        <menuitem action='TerminalShowScrollbar'/>
        <separator/>
        <menuitem action='GPreferences'/>
        <menuitem action='TPreferences'/>
        <separator/>
        <menuitem action='Quit'/>
      </popup>
    
    </ui>""" % terminal_menus

def fix_actions(actions, instance):
    """
    Helper function to map methods to an instance
    """
    retval = []
    
    for i in range(len(actions)):
        curr = actions[i]
        if len(curr) > 5:
            curr = list(curr)
            curr[5] = getattr(instance, curr[5])
            curr = tuple(curr)
            
        retval.append(curr)
    return retval

def load_config():
    """
    Load config all opened windows
    """
    global _all_windows

    for window in _all_windows:
        window.load_config()

    eemyle.load_config()
    terminal.load_config()

class GEemyleUI(gtk.Window):

    def __init__(self, is_main_window=False):
        
        gtk.Window.__init__(self)

        self.is_main_window = is_main_window

        self.set_title(config.PACKAGE)
        self.realize()
        self.set_icon_from_file(config.APP_ICON_FILEPATH)

        # Storing globals objects
        self.nb_geemyle = gtk.Notebook()
        self.ev_geemyle = gtk.EventBox()
        self.sb_geemyle = gtk.Statusbar()
        self.pb_geemyle = gtk.ProgressBar()
        self.tt_geemyle = gtk.Tooltips()
        self.tt_geemyle.enable()
        
        self.create_ui()
        self.construct_frame()
        self.show_all()
        self.load_config()
        
        self.connect('destroy', self.on_quit_activate)

        if self.is_main_window:
            gobject.timeout_add(100, self.on_open_eemyle_view_activate)

    def create_ui(self):
        ag = gtk.ActionGroup('WindowActions')

        # Map actions
        actions = fix_actions(window_actions, self)
        ag.add_actions(actions)

        actions = fix_actions(toggle_actions, self)
        ag.add_toggle_actions(actions)

        methods = radios_actions.keys()
        for method in methods:
            attr = getattr(self, method)
            ag.add_radio_actions(radios_actions[method], value=1, on_change=attr)

        self.ui = gtk.UIManager()
        self.ui.insert_action_group(ag, 0)
        self.ui.add_ui_from_string(ui_string)
        self.add_accel_group(self.ui.get_accel_group())

        ## Deactivate F10 :
        defaults = gtk.settings_get_default()
        defaults.set_string_property('gtk-menu-bar-accel', 'VoidSymbol', 'VoidSymbol')

    def construct_frame(self):
        global _all_windows

        # Adding the window to all opened windows
        _all_windows.append(self)

        self.sb_geemyle.set_has_resize_grip(False)
        vbox = gtk.VBox()
        self.add(vbox)

        # Menubar
        vbox.pack_start(self.ui.get_widget('/Menubar'), expand=False)

        # Toolbars
        for toolbar_str in [ '/EemyleToolBar', '/TerminalToolBar' ]:
            toolbar = self.ui.get_widget(toolbar_str)
            toolbar.set_style(gtk.TOOLBAR_ICONS)
            vbox.pack_start(toolbar, expand=False)

        # Some buttons which are slaves
        if not self.is_main_window:
            self.ui.get_widget('/TerminalToolBar/TerminalBroadcast').set_sensitive(False)
            self.ui.get_widget('/Menubar/TerminalMenu/TerminalBroadcast').set_sensitive(False)
            self.ui.get_widget('/TerminalPopup/TerminalBroadcast').set_sensitive(False)

        # Notebook
        self.nb_geemyle.set_scrollable(True)
        self.nb_geemyle.connect('switch-page', self.on_nb_geemyle_switch_page)
        vbox.pack_start(self.nb_geemyle)

        # Statusbar and progressbar
        hbox = gtk.HBox()
        self.ev_geemyle.add(self.sb_geemyle)
        hbox.pack_start(self.ev_geemyle)
        hbox.pack_start(self.pb_geemyle, expand=False)
        vbox.pack_start(hbox, expand=False)

    def on_open_eemyle_view_activate(self, action=None):

        child = eemyle.EemyleView(self)
        self.nb_geemyle.append_page(child, gtk.Label(_('Eemyle')))

        return False

    def on_nb_geemyle_switch_page(self, notebook, page, page_num):
        """
        Try to give the focus to a terminal
        """

        # Get the selected box
        child = notebook.get_nth_page(page_num)
        label = notebook.get_tab_label(child)

        # Hide the toolbars
        for toolbar_str in [ '/EemyleToolBar', '/TerminalToolBar' ]:
            toolbar = self.ui.get_widget(toolbar_str)
            toolbar.hide()

        if isinstance(child, terminal.EemyleTerminalBox):
            # Title
            child.on_terminal_window_title_changed()

            # Menus
            self.ui.get_widget('/Menubar/TerminalMenu').set_sensitive(True)
            
            # Toolbar
            if config.get('geemyle', 'show_toolbar'):
                self.ui.get_widget('/TerminalToolBar').show()

            child.terminal.set_property('has-focus', True)

        else:
            # Title
            self.set_title('%s - %s' % (label.get_text(), config.PACKAGE))

            # Menus
            self.ui.get_widget('/Menubar/TerminalMenu').set_sensitive(False)

            # Toolbar
            if config.get('geemyle', 'show_toolbar'):
                self.ui.get_widget('/EemyleToolBar').show()

        # Menus and buttons
        n_pages = notebook.get_n_pages()
        self.ui.get_widget('/Menubar/ViewMenu/MoveTabLeft').set_sensitive(False)
        self.ui.get_widget('/Menubar/ViewMenu/MoveTabRight').set_sensitive(False)
        self.ui.get_widget('/TerminalPopup/ChangeCurrentTabLeft').set_sensitive(False)
        self.ui.get_widget('/TerminalPopup/ChangeCurrentTabRight').set_sensitive(False)

        if page_num > 0:
            self.ui.get_widget('/TerminalPopup/ChangeCurrentTabLeft').set_sensitive(True)

        if page_num == n_pages - 1:
            self.ui.get_widget('/TerminalPopup/ChangeCurrentTabRight').set_sensitive(True)

        if page_num > 1:
            self.ui.get_widget('/Menubar/ViewMenu/MoveTabLeft').set_sensitive(True)

        if page_num > 0 and page_num < n_pages - 1:
            self.ui.get_widget('/Menubar/ViewMenu/MoveTabRight').set_sensitive(True)
            
        return True
        
    def on_fullscreen_toggled(self, action):
        """
        Toggle full screen mmode
        """
        
        if action.get_active():
            config.set('geemyle', 'full_screen', True)
            self.fullscreen()
        else:
            config.set('geemyle', 'full_screen', False)
            self.unfullscreen()

        return True

    def on_show_menubar_toggled(self, action):
        """
        Show or hide menubar
        """
        
        menubar = self.ui.get_widget('/Menubar')
        if action.get_active():
            config.set('geemyle', 'show_menubar', True)
            menubar.show()
        else:
            config.set('geemyle', 'show_menubar', False)
            menubar.hide()

        return True

    def on_show_toolbar_toggled(self, action):
        """
        Show or hide the toolbar
        """

        # Hide the toolbars
        for toolbar_str in [ '/EemyleToolBar', '/TerminalToolBar' ]:
            toolbar = self.ui.get_widget(toolbar_str)
            toolbar.hide()

        config.set('geemyle', 'show_toolbar', action.get_active())

        if action.get_active():
            page_num = self.nb_geemyle.get_current_page()
            child    = self.nb_geemyle.get_nth_page(page_num)

            if isinstance(child, terminal.EemyleTerminalBox):
                self.ui.get_widget('/TerminalToolBar').show()
            else:
                self.ui.get_widget('/EemyleToolBar').show()

    def on_move_tab_left_activate(self, action, child=None):
        """
        Move a tab to the left
        """

        if child:
            # Action was activated by the popup menu
            current_page = self.nb_geemyle.page_num(child)
        else:
            # Menu or toolbar
            current_page, label, child = self.get_current_child()

        if not current_page > 1:
            return True

        self.nb_geemyle.reorder_child(child, current_page - 1)

    def on_move_tab_right_activate(self, action, child=None):
        """
        Move a tab to the right
        """
        
        if child:
            # Action was activated by the popup menu
            current_page = self.nb_geemyle.page_num(child)
        else:
            # Menu or toolbar
            current_page, label, child = self.get_current_child()

        self.nb_geemyle.reorder_child(child, current_page + 1)

    def on_change_current_tab_left_activate(self, action):
        """
        Change the current tab to the left
        """
        
        self.nb_geemyle.prev_page()

    def on_change_current_tab_right_activate(self, action):
        """
        Change the current tab to the right
        """
        
        self.nb_geemyle.next_page()

    def on_terminal_new_activate(self, action):
        
        # Open a new local terminal
        self.terminal_open_connection()

    def load_config(self):

        if config.get('geemyle', 'full_screen'):
            self.fullscreen()

        if not config.get('geemyle', 'show_menubar'):
            menubar = self.ui.get_widget('/Menubar')
            menubar.hide()

        if not config.get('geemyle', 'show_toolbar'):
            for toolbar_str in [ '/EemyleToolBar', '/TerminalToolBar' ]:
                toolbar = self.ui.get_widget(toolbar_str)
                toolbar.hide()

        tab_pos = config.get('geemyle', 'tab_position')
        if tab_pos == 'left':
            tab_pos = gtk.POS_LEFT
        elif tab_pos == 'right':
            tab_pos = gtk.POS_RIGHT
        elif tab_pos == 'bottom':
            tab_pos = gtk.POS_BOTTOM
        else:
            tab_pos = gtk.POS_TOP

        self.nb_geemyle.set_tab_pos(tab_pos)

    def on_reset_percent(self):

        # Resetting the progressbar
        self.pb_geemyle.set_fraction(0.0)
        self.pb_geemyle.set_text('')

        # Resetting the status bar
        self.sb_geemyle.pop(0)

        # We don't want to go back here
        return False

    def on_set_message(self, message):
        
        self.sb_geemyle.pop(0)
        if message:
            self.sb_geemyle.push(0, message)
        
    def on_set_percent(self, percent, message):
        
        fraction = float(percent) / 100
        self.pb_geemyle.set_fraction(fraction)
        self.pb_geemyle.set_text('%s - %.d%%' % (message, percent))

        if percent == 100:
            # Wait 1 sec before resetting the progressbar
            gobject.timeout_add(1000, self.on_reset_percent)

        return True

    def on_terminal_reset_activate(self, action):
        """
        Tell the current terminal to reset
        """

        current_page, label, child = self.get_current_child()
        if not isinstance(child, terminal.EemyleTerminalBox):
            return False

        child.terminal.reset(True, True)

    
    def on_terminal_set_encoding_changed(self, old_action, new_action):
        """
        Change the encoding of the current terminal
        """

        current_page, label, child = self.get_current_child()
        if not isinstance(child, terminal.EemyleTerminalBox):
            return False

        try:
            name     = new_action.get_name()
            n_enc    = int(name.split('_')[1])
            encoding = terminal.ENCODINGS[n_enc][1]
        except:
            return False

        child.terminal.set_encoding(encoding)

    def on_terminal_set_width_changed(self, old_action, new_action):
        """
        Change the width of the current terminal
        """

        current_page, label, child = self.get_current_child()
        if not isinstance(child, terminal.EemyleTerminalBox):
            return False

        name    = new_action.get_name()
        n_width = int(name.split('_')[1])
        width   = terminal.WIDTHS[n_width][1]

        child.terminal.set_size(width)

    def on_terminal_close_activate(self, action=None, child=None):
        """
        Called by:
        - close button from the toolbar  (action)
        - close entry from menu          (action)
        - child-exited from the terminal (child)
        """

        #print 'main_window:on_terminal_close_activate was called, child is', child

        if child == None:
            page_num = self.nb_geemyle.get_current_page()
            child    = self.nb_geemyle.get_nth_page(page_num)
            if isinstance(child, terminal.EemyleTerminalBox):
                # Closing the connection will cause coming back here with child = terminal.EemyleTerminalBox
                child.terminal.on_close_connection_activate()
                return True
        else:
            page_num = self.nb_geemyle.page_num(child)

        self.sb_geemyle.pop(0)

        if isinstance(child, terminal.EemyleTerminalBox):
            buffer = ''
            for line in child.terminal.get_buffer().split('\n'):
                if line:
                    buffer += '%s\n' % line
                
            buffer = buffer[:-1]
    
            # Try to see an error
            r = re.compile('.*WARNING: REMOTE HOST IDENTIFICATION HAS CHANGED!.*')
            f1 = r.findall(buffer)

            r = re.compile('ssh: connect to host (.*) port 22: (.*)')
            f2 = r.findall(buffer)

            if f1:
                buffer = buffer.replace('@', '')
                buffer = buffer.replace('  ', '')
                self.sb_geemyle.push(0, _('WARNING: REMOTE HOST IDENTIFICATION HAS CHANGED!'))
                #FIXME: Freezing the ui mygtk.on_popup_activate(None, buffer, message_type=gtk.MESSAGE_WARNING)

            elif f2:
                message = _('connect to host %s: %s') % f2[0]
                self.sb_geemyle.push(0, message)
                #FIXME: Freezing the ui mygtk.on_popup_activate(None, '\n\n%s.' % message, message_type=gtk.MESSAGE_WARNING)
    
            else:
                self.sb_geemyle.push(0, _('Connection closed.'))

        # Removing the page from the notebook
        self.nb_geemyle.remove_page(page_num)

        # Closing the window if there is no more page on the notebook
        if self.nb_geemyle.get_n_pages() == 0:
            self.on_quit_activate()


    def on_terminal_send_files_activate(self, action):
        
        current_page, label, child = self.get_current_child()
        if not isinstance(child, terminal.EemyleTerminalBox):
            return False

        dialog = gtk.FileChooserDialog(title=_('Choose file(s) to upload'), parent=self, action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                       buttons=(gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))

        dialog.realize()
        dialog.set_transient_for(self)
        dialog.set_select_multiple(True)

        response = dialog.run()
        if response == gtk.RESPONSE_OK:

            # Get selected files
            files = dialog.get_filenames()
            dialog.destroy()
            
            # Send files to the current terminal
            if not child.terminal.send_files(files):
                dialog = gtk.MessageDialog(parent=self, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_CLOSE,
                                           message_format=_("""Failed to send file(s) to the terminal.

Verify that:
 - there is no transfer on this terminal
 - broadcast mode is not activate
 - the connection is ready
 - the terminal is not waiting for sending the root password"""))

                dialog.realize()
                dialog.set_transient_for(self)
                dialog.run()
                dialog.destroy()

        else:
            dialog.destroy()

    def on_geemyle_preferences_activate(self, action):
        
        dialog = eemyle.EemylePreferences(load_config)

        dialog.show_all()
        dialog.set_transient_for(self)

        dialog.run()
        dialog.destroy()

    def on_about_activate(self, action):

        dialog = gtk.AboutDialog()

        dialog.set_name(config.PACKAGE)
        dialog.set_version(config.VERSION)
        dialog.set_copyright('Copyright 2005-2006 Easter-eggs')
        dialog.set_comments('comments')
        dialog.set_license(license)
        dialog.set_website('http://www.easter-eggs.com')
        dialog.set_website_label('Easter-eggs')
        dialog.set_authors(('Cyril Lacoux <clacoux@easter-eggs.com>', ))

        dialog.show_all()

        dialog.set_transient_for(self)
        dialog.set_modal(True)

        dialog.run()
        dialog.destroy()

    def get_current_child(self):
        """
        Get the current widget from NoteBook
        """

        current_page = self.nb_geemyle.get_current_page()
        child        = self.nb_geemyle.get_nth_page(current_page)
        label        = self.nb_geemyle.get_tab_label(child)

        return (current_page, label, child)

    ##########################################################################
    # Terminal functions
    def on_terminal_preferences_activate(self, action):
                                   
        dialog = terminal.TerminalPreferences(load_config)

        dialog.show_all()
        dialog.set_transient_for(self)

        dialog.run()
        dialog.destroy()


    def terminal_connect(self, traceroute):
        """
        Connect to a server
        """

        if config.get('eemyle', 'use_first_hop'):
            server = {}
            server['ip']           = config.get('eemyle', 'fh_ip_address')
            server['server_name']  = config.get('eemyle', 'fh_hostname')
            server['admin_name']   = config.get('eemyle', 'fh_username')
            server['admin_pass']   = config.get('eemyle', 'fh_password')
            server['root_pass']    = ''
            server['session_name'] = 'first hop'
            server['ssh_port']     = 22 # FIXME: -> conf

            traceroute.insert(0, server)

        server_id      = int(traceroute[-1]['id_server'])
        name           = traceroute[-1]['session_name']
        session_infos  = traceroute
        session_datas  = eemyle.get_connect_data(server_id)

        if 'run_on_previous_hop' in session_datas:
            if session_datas['run_on_previous_hop']:
                session_infos.pop()

        if 'session_name_prefix' in session_datas:
            name = "%s - %s" % (session_datas['session_name_prefix'],name)

        self.terminal_open_connection(name=name, session_infos=session_infos, session_datas=session_datas)

        del(server_id)
        del(traceroute)
        del(session_infos)
        del(session_datas)

    def on_customer_refresh_activate(self, action):

        current_page, label, child = self.get_current_child()
        if not isinstance(child, eemyle.EemyleView):
            return False

        child.on_bt_display_list_refresh_clicked()
    
    def on_customer_view_contacts_activate(self, action):

        current_page, label, child = self.get_current_child()
        if not isinstance(child, eemyle.EemyleView):
            return False

        child.on_bt_view_contacts_clicked()

    def on_server_remove_activate(self, action):
        
        current_page, label, child = self.get_current_child()
        if not isinstance(child, eemyle.EemyleView):
            return False

        child.on_bt_remove_server_clicked()

    def on_server_add_activate(self, action):
        
        current_page, label, child = self.get_current_child()
        if not isinstance(child, eemyle.EemyleView):
            return False

        child.on_bt_add_server_clicked()

    def on_server_duplicate_activate(self, action):
        
        current_page, label, child = self.get_current_child()
        if not isinstance(child, eemyle.EemyleView):
            return False

        child.on_bt_duplicate_server_clicked()

    def on_database_statistics_activate(self, action):
        
        current_page, label, child = self.get_current_child()
        if not isinstance(child, eemyle.EemyleView):
            return False

        child.on_database_statistics_clicked()

    def on_terminal_connect_activate(self, action):
        
        current_page, label, child = self.get_current_child()
        if not isinstance(child, eemyle.EemyleView):
            return False

        child.on_bt_connect_clicked()

    def on_terminal_ipmi_activate(self, action):

        current_page, label, child = self.get_current_child()
        if not isinstance(child, eemyle.EemyleView):
            return False

        child.on_bt_ipmi_clicked()

    def on_kill_ipmi_activate(self, action):

        current_page, label, child = self.get_current_child()
        if not isinstance(child, eemyle.EemyleView):
            return False

        child.on_bt_kill_ipmi_clicked()

    def on_reboot_by_ipmi_activate(self, action):

        current_page, label, child = self.get_current_child()
        if not isinstance(child, eemyle.EemyleView):
            return False

        child.on_reboot_by_ipmi_clicked()


    def on_terminal_tunnel_activate(self, action):

        current_page, label, child = self.get_current_child()
        if not isinstance(child, eemyle.EemyleView):
            return False

        child.on_bt_tunnel_clicked()

    def on_terminal_x11_activate(self, action):

        current_page, label, child = self.get_current_child()
        if not isinstance(child, eemyle.EemyleView):
            return False

        child.on_bt_x11_clicked()

    def on_terminal_duplicate_activate(self, action, child=None):
        """
        Duplicate a terminal to a new tab
        """

        if child:
            # Action was activated by the popup menu
            label = child.label
        else:
            # Menu or toolbar
            current_page, label, child = self.get_current_child()
            if not isinstance(child, terminal.EemyleTerminalBox):
                return False

        self.terminal_open_connection(name=child.orig_name, session_infos=child.terminal.session_infos)

    def on_terminal_broadcast_toggled(self, action):
        """
        Send input to all terminals
        """
        if action.get_active():
            # Setting the Status bar color to RED
            self.ev_geemyle.set_name('red')
            self.on_set_message('Entering broadcast mode.')
            if not terminal.get_terminal_broadcast():
                terminal.set_terminal_broadcast(True)

        else:
            # Resetting the Status bar color
            self.ev_geemyle.set_name('GtkEventBox')
            self.on_set_message('Leaving broadcast mode.')
            if terminal.get_terminal_broadcast():
                terminal.set_terminal_broadcast(False)
        
    def on_terminal_detach_activate(self, action, child=None):
        """
        Detach a terminal to a new window
        """
        
        if not child:
            # Menu or toolbar
            current_page, label, child = self.get_current_child()
            if not isinstance(child, terminal.EemyleTerminalBox):
                return False
        
        # Removing the child from the notebook
        self.nb_geemyle.remove(child)
        
        ui = GEemyleUI()
        ui.nb_geemyle.append_page(child, child.label)
        child.geemyle_ui          = ui
        child.terminal.geemyle_ui = ui
        child.on_terminal_window_title_changed()
        
        # Closing the window if there is no more page on the notebook
        if self.nb_geemyle.get_n_pages() == 0:
            self.on_quit_activate()

        return True

    def on_terminal_rename_activate(self, action, child=None):
        """
        Detach the current terminal to a new window
        """
        
        if child:
            # Action was activated by the popup menu
            label = child.label
        else:
            # Menu or toolbar
            current_page, label, child = self.get_current_child()
            if not isinstance(child, terminal.EemyleTerminalBox):
                return False
        
        dialog = gtk.Dialog(flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT, buttons=(gtk.STOCK_OK,     gtk.RESPONSE_OK, 
                                                                                            gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))

        dialog.set_title('Rename terminal')
        dialog.set_icon_from_file(config.APP_ICON_FILEPATH)
        dialog.set_default_response(gtk.RESPONSE_OK)

        dialog.realize()
        dialog.set_transient_for(self)
        dialog.set_resizable(False)
        
        vbox = gtk.VBox()
        vbox.set_spacing(5)
        vbox.set_border_width(10)

        label2 = gtk.Label('Name of the terminal:')
        label2.set_alignment(0, 0.5)
        vbox.pack_start(label2)

        entry  = gtk.Entry()
        entry.set_size_request(300, -1)
        entry.set_text(label.get_text())
        vbox.pack_start(entry)

        dialog.vbox.pack_start(vbox)

        dialog.show_all()

        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            text = entry.get_text()
            if text:
                label.set_text(text)
            
        dialog.destroy()
        return True
        
    def on_terminal_show_scrollbar_toggled(self, action):
        """
        Show or hide current terminals' scrollbar
        """

        config.set('terminal', 'show_scrollbar', action.get_active())

        # Applying this to all opened terminals
        terminal.load_config()

    def on_terminal_stop_feed_child_activate(self, action):
        """
        Stop Feeding terminal's child
        """

        current_page, label, child = self.get_current_child()
        if not isinstance(child, terminal.EemyleTerminalBox):
            return False

        child.terminal.on_stop_feed_child_activate()

    def on_terminal_become_root_activate(self, action):
        """
        Become root
        """

        current_page, label, child = self.get_current_child()
        if not isinstance(child, terminal.EemyleTerminalBox):
            return False

        child.terminal.on_become_root_activate()

    def on_terminal_send_command_activate(self, action):
        """
        Send a define command to the current terminal
        """

        current_page, label, child = self.get_current_child()
        if not isinstance(child, terminal.EemyleTerminalBox):
            return False

        try:
            name  = action.get_name()
            n_cmd = int(name.split('_')[1])
            cmd   = config.get('terminal', 'commands')[n_cmd][4]
        except:
            return False

        child.terminal.on_send_command_activate(cmd)

    def terminal_open_connection(self, name=_('Terminal'), session_infos=[], session_commands=[], session_datas={}):

        child = terminal.EemyleTerminalBox(self, name, session_infos, session_commands, session_datas)

        # Storing the box into a new tab
        page_num = self.nb_geemyle.append_page(child, child.get_label())

        # Set the current tab
        self.nb_geemyle.set_current_page(page_num)
        
        # Give the focus to the terminal
        child.terminal.set_property('has-focus', True)

    def on_terminal_sendto_activate(self, menu_item, child, ui):
        """
        Sends a Terminal to anotehr window
        """

        # Removing the child from the notebook
        self.nb_geemyle.remove(child)
        
        ui.nb_geemyle.append_page(child, child.label)
        child.geemyle_ui          = ui
        child.terminal.geemyle_ui = ui
        child.on_terminal_window_title_changed()
        
        # Closing the window if there is no more page on the notebook
        if self.nb_geemyle.get_n_pages() == 0:
            self.on_quit_activate()

        return True

    def get_all_windows(self):
        """
        Get the list of available windows
        """
        global _all_windows

        return _all_windows

    def on_quit_activate(self, *kargs):
        """
        Close the window
        """
        global _all_windows

        # Removing the window from all windows
        if self in _all_windows:
            _all_windows.remove(self)

        # Closing all existing Terminals
        page_num = self.nb_geemyle.get_n_pages()

        for i in range(0, page_num):
            child = self.nb_geemyle.get_nth_page(i)
            if isinstance(child, terminal.EemyleTerminalBox):
                child.terminal.on_close_connection_activate()

            elif isinstance(child, eemyle.EemyleView):
                child.on_close_activate()

        self.destroy()

        if self.is_main_window:
            gtk.main_quit()       
