#!/usr/bin/env python
#
# GEemyle: Custom GTK widgets
# Copyright (C) 2003-2006 Easter-eggs <info@easter-eggs.com>
# Authors:
#      Cyril Lacoux (Yack) <clacoux@easter-eggs.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

__author__  = "Cyril Lacoux [Yack] <clacoux@easter-eggs.com>"
__cvsid__   = "$Id$"
__version__ = "$Revision$"[11:-2]
__all__     = ["GEemyleUI"]

import pygtk
pygtk.require('2.0')
import gtk
import gobject

rc_string = """
        style "blue" {
            bg[NORMAL]             = "#3063b0"
            bg[PRELIGHT]           = "#3063b0"
            bg[ACTIVE]             = "#3063b0"
            bg[SELECTED]           = "#3063b0"
            fg[NORMAL]             = "#ffffff"
            fg[PRELIGHT]           = "#ffffff"
            fg[ACTIVE]             = "#ffffff"
            fg[SELECTED]           = "#ffffff"
            GtkRange::slider_width = 15
            GtkScrollbar::min_slider_length = 15
        }
        widget "*.blue" style "blue"

        style "red" {
            bg[NORMAL]             = "#e71717"
            bg[PRELIGHT]           = "#e71717"
            bg[ACTIVE]             = "#e71717"
            bg[SELECTED]           = "#e71717"
            fg[NORMAL]             = "#ffffff"
            fg[PRELIGHT]           = "#ffffff"
            fg[ACTIVE]             = "#ffffff"
            fg[SELECTED]           = "#ffffff"
            GtkRange::slider_width = 15
            GtkScrollbar::min_slider_length = 15
        }
        widget "*.red" style "red"
"""

gtk.rc_parse_string(rc_string)

def on_popup_activate(widget, message, message_type=gtk.MESSAGE_INFO):

    dialog = gtk.MessageDialog(None, gtk.DIALOG_DESTROY_WITH_PARENT,
                               message_type, gtk.BUTTONS_OK)
    dialog.set_markup(message)
    dialog.show_all()

    dialog.window.set_transient_for(main_window)
    dialog.set_modal(True)

#    dialog.set_icon_from_file(config.APP_ICON_FILEPATH)
    dialog.run()
    dialog.destroy()

    return True

class ComboBoxEntryCompletion(gtk.ComboBoxEntry):

    def __init__(self, model=None, column=-1):

        gtk.ComboBoxEntry.__init__(self, model, column)

        self.EntryCompletion = gtk.EntryCompletion()

        if not model == None:
            self.EntryCompletion.set_model(model)
            
        if column >= 0:
            self.EntryCompletion.set_text_column(column)

        self.child.set_completion(self.EntryCompletion)

        self.EntryCompletion.connect('match-selected', self.match_selected)

    def match_selected(self, completion, model, iter):
        
        iter = model.convert_iter_to_child_iter(iter)
        self.set_active_iter(iter)

    def set_model(self, model):
        
        self.EntryCompletion.set_model(model)
        gtk.ComboBoxEntry.set_model(self, model)

    def set_text_column(self, column):
        
        self.EntryCompletion.set_text_column(column)
        gtk.ComboBoxEntry.set_text_column(self, column)


class Button(gtk.Button):

    def __init__(self, stock=None, label=None):
        
        gtk.Button.__init__(self)
        
        self.image   = gtk.Image()
        self.label   = gtk.Label()

        self.construct_frame()
        if stock:
            self.set_stock(stock)
        if label:
            self.set_label(label)

    def construct_frame(self):
        
        hbox = gtk.HBox()
        hbox.set_spacing(5)

        # Image
        hbox.pack_start(self.image, expand=False)
        
        # Label
        self.label.set_alignment(0.0, 0.5)
        self.label.set_use_underline(True)
        self.label.set_mnemonic_widget(self)
        hbox.pack_start(self.label)

        alg = gtk.Alignment(0.5, 0.5)
        alg.add(hbox)

        self.add(alg)

    def set_stock(self, stock):

        self.image.set_from_stock(stock, gtk.ICON_SIZE_BUTTON)

    def set_label(self, label):
        
        self.label.set_markup_with_mnemonic(label)


def add_columns_to_treeview(treeview, columns):
    
    for name, renderer, renderer_properties, column_id, visible, sort_id in columns:

        renderer_inst = renderer()
        renderer_inst.set_data('column', column_id)
        for property, value in renderer_properties:
            renderer_inst.set_property(property, value)

        column = gtk.TreeViewColumn(name)
        column.pack_start(renderer_inst)

        if renderer == gtk.CellRendererText:
            column.add_attribute(renderer_inst, 'text',   column_id)

        elif renderer == gtk.CellRendererPixbuf:
            column.add_attribute(renderer_inst, 'pixbuf', column_id)

        elif renderer == gtk.CellRendererToggle:
            column.add_attribute(renderer_inst, 'active', column_id)
        
        column.set_visible(visible)
        column.set_sort_column_id(sort_id)
        column.set_reorderable(True)

        treeview.append_column(column)


class SimpleComboBox(gtk.ComboBox):

    def __init__(self, list=None):
        
        gtk.ComboBox.__init__(self)

        model = gtk.ListStore(gobject.TYPE_INT,
                              gobject.TYPE_STRING)

        self.set_model(model)
        
        cell = gtk.CellRendererText()
        self.pack_start(cell, True)
        self.add_attribute(cell, 'text', 1)

        if list:
            self.set_list(list)

    def set_list(self, list):

        model = self.get_model()
        model.clear()
        for id, name in list:
            iter = model.append((int(id), name))

    def set_active(self, id):
        
        try:
            id = int(id)
        except:
            gtk.ComboBox.set_active(self, -1)
            return True

        model = self.get_model()

        iter = model.get_iter_first()
        while iter:
            if model.get_value(iter, 0) == id:
                self.set_active_iter(iter)
                iter = None
            else:
                iter = model.iter_next(iter)

    def get_active(self):

        model = self.get_model()
        iter  = self.get_active_iter()
        if not iter:
            return -1

        return model.get_value(iter, 0)
