#!/usr/bin/env python
#
# GEemyle: Terminal preferences
# Copyright (C) 2003-2006 Easter-eggs <info@easter-eggs.com>
# Authors:
#      Cyril Lacoux (Yack) <clacoux@easter-eggs.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

__author__  = "Cyril Lacoux [Yack] <clacoux@easter-eggs.com>"
__cvsid__   = "$Id$"
__version__ = "$Revision$"[11:-2]
__all__     = ["TerminalPreferences"]


import pygtk
pygtk.require('2.0')
import gtk
import gobject
import re

import geemyle.config          as config
import geemyle.gettext         as gettext
import geemyle.gui.mygtk       as mygtk
from   geemyle.gui.preferences import Preferences

import terminal

_ = gettext._

class GeneralPreferences(Preferences):

    def __init__(self):

        Preferences.__init__(self, _('General'))

        self.use_system_font = gtk.CheckButton(_('Use system font'))
        self.fontsel         = gtk.FontButton()
        self.audible_bell    = gtk.CheckButton(_('Audible bell'))
        self.visible_bell    = gtk.CheckButton(_('Visible bell'))
        self.allow_bold      = gtk.CheckButton(_('Allow bold'))
        self.cursor_blinks   = gtk.CheckButton(_('Cursor blinks'))
        self.mouse_autohide  = gtk.CheckButton(_('Mouse autohide'))
        self.word_chars      = gtk.Entry()
        self.prefered_shell  = gtk.Entry()

        self.construct_frame()
        self.get_config()

    def construct_frame(self):

        self.use_system_font.connect('toggled', self.use_system_font_toggled)
        self.fontsel.connect('font-set',        self.emit_changed)
        self.audible_bell.connect('toggled',    self.emit_changed)
        self.visible_bell.connect('toggled',    self.emit_changed)
        self.allow_bold.connect('toggled',      self.emit_changed)
        self.cursor_blinks.connect('toggled',   self.emit_changed)
        self.mouse_autohide.connect('toggled',  self.emit_changed)
        self.word_chars.connect('changed',      self.emit_changed)
        self.prefered_shell.connect('changed',  self.emit_changed)

        i = 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('General:'))
        self.attach(label,                1, 3, i, i+1)

        i += 1
        self.attach(self.use_system_font, 1, 2, i, i+1)
        self.attach(self.fontsel,         2, 3, i, i+1)

        i += 1
        self.attach(self.audible_bell,    1, 2, i, i+1)
        self.attach(self.visible_bell,    2, 3, i, i+1)
        
        i += 1
        self.attach(self.allow_bold,      1, 2, i, i+1)

        i += 1
        self.attach(self.cursor_blinks,   1, 2, i, i+1)
        self.attach(self.mouse_autohide,  2, 3, i, i+1)

        i += 1
        label = gtk.Label(_('Select-by-word characters:'))
        label.set_alignment(0, 0.5)
        self.attach(label,                1, 2, i, i+1)
        self.attach(self.word_chars,      2, 3, i, i+1)

        i += 1
        label = gtk.Label(_('Prefered shell:'))
        label.set_alignment(0, 0.5)
        self.attach(label,                1, 2, i, i+1)
        self.attach(self.prefered_shell,  2, 3, i, i+1)

        self.show_all()

    def use_system_font_toggled(self, button):
        
        self.fontsel.set_sensitive(not button.get_active())
        self.emit_changed()
            
    def get_config(self):

        self.use_system_font.set_active(config.get('terminal', 'use_system_font'))
        self.fontsel.set_font_name(config.get('terminal', 'font_name'))
        self.audible_bell.set_active(config.get('terminal', 'audible_bell'))
        self.visible_bell.set_active(config.get('terminal', 'visible_bell'))
        self.allow_bold.set_active(config.get('terminal', 'allow_bold'))
        self.cursor_blinks.set_active(config.get('terminal', 'cursor_blinks'))
        self.mouse_autohide.set_active(config.get('terminal', 'mouse_autohide'))
        self.word_chars.set_text(config.get('terminal', 'word_chars'))
        self.prefered_shell.set_text(config.get('terminal', 'prefered_shell'))

        self.use_system_font_toggled(self.use_system_font)

    def set_config(self):
        
        config.set('terminal', 'use_system_font', self.use_system_font.get_active())
        config.set('terminal', 'font_name',       self.fontsel.get_font_name())
        config.set('terminal', 'audible_bell',    self.audible_bell.get_active())
        config.set('terminal', 'visible_bell',    self.visible_bell.get_active())
        config.set('terminal', 'allow_bold',      self.allow_bold.get_active())
        config.set('terminal', 'cursor_blinks',   self.cursor_blinks.get_active())
        config.set('terminal', 'mouse_autohide',  self.mouse_autohide.get_active())
        config.set('terminal', 'word_chars',      self.word_chars.get_text())
        config.set('terminal', 'prefered_shell',  self.prefered_shell.get_text())


class ColorPreferences(Preferences):

    def __init__(self):

        Preferences.__init__(self, _('Colors'))

        self.use_theme_colors = gtk.CheckButton(_('Use colors from system theme'))
        self.color_scheme     = gtk.ComboBox(gtk.ListStore(gobject.TYPE_STRING))
        self.foreground_color = gtk.ColorButton()
        self.background_color = gtk.ColorButton()
        self.palette_scheme   = gtk.ComboBox(gtk.ListStore(gobject.TYPE_STRING))

        self.palette = []
        self.palette.append(gtk.ColorButton())
        self.palette.append(gtk.ColorButton())
        self.palette.append(gtk.ColorButton())
        self.palette.append(gtk.ColorButton())
        self.palette.append(gtk.ColorButton())
        self.palette.append(gtk.ColorButton())
        self.palette.append(gtk.ColorButton())
        self.palette.append(gtk.ColorButton())
        self.palette.append(gtk.ColorButton())
        self.palette.append(gtk.ColorButton())
        self.palette.append(gtk.ColorButton())
        self.palette.append(gtk.ColorButton())
        self.palette.append(gtk.ColorButton())
        self.palette.append(gtk.ColorButton())
        self.palette.append(gtk.ColorButton())
        self.palette.append(gtk.ColorButton())

        self.construct_frame()
        self.get_config()

    def construct_frame(self):

        self.use_theme_colors.connect('toggled', self.use_theme_colors_toggled)
        self.color_scheme.connect('changed', self.color_scheme_changed)
        self.foreground_color.connect('color-set', self.color_changed)
        self.background_color.connect('color-set', self.color_changed)
        self.palette_scheme.connect('changed', self.palette_scheme_changed)
        for palette_item in self.palette:
            palette_item.connect('color-set', self.palette_changed)

        cell = gtk.CellRendererText()
        self.color_scheme.pack_start(cell, True)
        self.color_scheme.add_attribute(cell, 'text', 0)

        cell = gtk.CellRendererText()
        self.palette_scheme.pack_start(cell, True)
        self.palette_scheme.add_attribute(cell, 'text', 0)

        i = 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Foreground and background:'))
        self.attach(label,                  1, 5, i, i+1)

        i += 1
        self.attach(self.use_theme_colors,  1, 5, i, i+1)

        i += 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup(_('Built-in schemes:'))
        self.attach(label,                  1, 2, i, i+1)
        self.attach(self.color_scheme,      2, 5, i, i+1)

        i += 1
        label = gtk.Label(_('Text color:'))
        label.set_alignment(0, 0.5)
        self.attach(label,                  1, 2, i, i+1)
        self.attach(self.foreground_color,  2, 3, i, i+1)
        label = gtk.Label(_('Background color:'))
        label.set_alignment(0, 0.5)
        self.attach(label,                  3, 4, i, i+1)
        self.attach(self.background_color,  4, 5, i, i+1)

        i += 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Palette:'))
        self.attach(label,                  1, 5, i, i+1)

        i += 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup(_('Built-in schemes:'))
        self.attach(label,                  1, 2, i, i+1)
        self.attach(self.palette_scheme,    2, 5, i, i+1)

        i += 1
        table = gtk.Table()
        ii = 1
        jj = 1
        for color in self.palette:
            table.attach(color, ii, ii+1, jj, jj+1)
            ii += 1
            if ii > 8:
                ii = 1
                jj += 1

        self.attach(table,                   1, 5, i, i+1)


    def use_theme_colors_toggled(self, button):
        
        self.color_scheme.set_sensitive(not button.get_active())
        self.foreground_color.set_sensitive(not button.get_active())
        self.background_color.set_sensitive(not button.get_active())
            
        self.emit_changed()

    def color_scheme_changed(self, combo):
        """
        Called when the color's scheme is changed
        """

        model = combo.get_model()
        iter  = combo.get_active_iter()

        scheme = model.get_value(iter, 0)
        if config.COLOR_SCHEMES.has_key(scheme):
            foreground_color = terminal.get_gdkcolor_from_tuple(config.COLOR_SCHEMES[scheme][0])
            background_color = terminal.get_gdkcolor_from_tuple(config.COLOR_SCHEMES[scheme][1])

            self.foreground_color.set_color(foreground_color)
            self.background_color.set_color(background_color)
        
        self.emit_changed()
        
    def color_changed(self, button):
        """
        Called when foreground or background color is changed manually
        """

        # Getting selected colors
        foreground_color = terminal.get_tuplecolor_from_gdk(self.foreground_color.get_color())
        background_color = terminal.get_tuplecolor_from_gdk(self.background_color.get_color())
        
        # Retrieving colors in schemes
        model = self.color_scheme.get_model()
        iter  = model.get_iter_first()

        while iter:
            scheme = model.get_value(iter, 0)
            if config.COLOR_SCHEMES.has_key(scheme):
                if config.COLOR_SCHEMES[scheme] == (foreground_color, background_color):
                    break
            iter = model.iter_next(iter)
                    
        # Custom?
        if not iter:
            iter  = model.get_iter_first()
            scheme = _('Custom')
            config.COLOR_SCHEMES[scheme] = (foreground_color, background_color)
            config.set('terminal', 'custom_color_fg', foreground_color)
            config.set('terminal', 'custom_color_bg', background_color)

        # Storing the scheme
        config.set('terminal', 'color_scheme', scheme)

        self.color_scheme.set_active_iter(iter)

        self.emit_changed()
        
    def palette_scheme_changed(self, combo):
        """
        Called when the palette's scheme is changed
        """

        model = combo.get_model()
        iter  = combo.get_active_iter()

        scheme = model.get_value(iter, 0)
        if config.PALETTE_SCHEMES.has_key(scheme):
            
            i = 0
            for color in config.PALETTE_SCHEMES[scheme]:
                color = terminal.get_gdkcolor_from_tuple(color)
                self.palette[i].set_color(color)
                i += 1
        
        self.emit_changed()

    def palette_changed(self, button):
        """
        Called when one of the palette's color is changed manually
        """

        # Getting selected colors
        palette = []
        for palette_item in self.palette:
            palette.append(terminal.get_tuplecolor_from_gdk(palette_item.get_color()))

        # Retrieving colors in schemes
        model = self.palette_scheme.get_model()
        iter  = model.get_iter_first()

        while iter:
            scheme = model.get_value(iter, 0)
            if config.PALETTE_SCHEMES.has_key(scheme):
                if config.PALETTE_SCHEMES[scheme] == palette:
                    break
            iter = model.iter_next(iter)
                    
        # Custom?
        if not iter:
            iter   = model.get_iter_first()
            scheme = _('Custom')
            config.PALETTE_SCHEMES[scheme] = palette

            # Storing the colors
            i = 0
            for palette_item in palette:
                config.set('terminal', 'custom_palette_%d' % i, palette_item)
                i += 1

        # Storing the scheme
        config.set('terminal', 'palette_scheme', scheme)

        self.palette_scheme.set_active_iter(iter)
        
        self.emit_changed()
 
    def get_config(self):

        # Theme
        self.use_theme_colors.set_active(config.get('terminal', 'use_theme_colors'))

        # color scheme
        model = self.color_scheme.get_model()
        model.clear()
        for scheme in [_('Custom')] + config.COLOR_SCHEMES.keys():
            iter = model.append((_(scheme), ))
            if config.get('terminal', 'color_scheme') == scheme:
                self.color_scheme.set_active_iter(iter)

        # palette scheme
        model = self.palette_scheme.get_model()
        model.clear()
        for scheme in [_('Custom')] + config.PALETTE_SCHEMES.keys():
            iter = model.append((_(scheme), ))
            if config.get('terminal', 'palette_scheme') == scheme:
                self.palette_scheme.set_active_iter(iter)


    def set_config(self):
        
        config.set('terminal', 'use_theme_colors', self.use_theme_colors.get_active())
        config.set('terminal', 'color_scheme',     self.color_scheme.get_active_text())
        config.set('terminal', 'palette_scheme',   self.palette_scheme.get_active_text())


class EffectPreferences(Preferences):

    def __init__(self):
        
        Preferences.__init__(self, _('Effects'))

        # Radio button
        self.solid_background       = gtk.RadioButton(label=_('_None (use solid color)'))
        self.image_background       = gtk.RadioButton(group=self.solid_background, label=_('_Background image'))
        self.transparent_background = gtk.RadioButton(group=self.solid_background, label=_('_Transparent background'))

        filter = gtk.FileFilter()
        filter.set_name("Images")
        filter.add_mime_type("image/png")
        filter.add_mime_type("image/jpeg")
        filter.add_mime_type("image/gif")
        filter.add_pattern("*.png")
        filter.add_pattern("*.jpg")
        filter.add_pattern("*.gif")
        filter.add_pattern("*.tif")
        filter.add_pattern("*.xpm")
        

        filechooser = gtk.FileChooserDialog(buttons=(gtk.STOCK_OPEN,   gtk.RESPONSE_OK,
                                                     gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
        filechooser.set_filter(filter)

        self.background_image_file  = gtk.FileChooserButton(filechooser)
        self.background_saturation  = gtk.HScale()

        self.construct_frame()
        self.get_config()

    def construct_frame(self):

        self.solid_background.connect('toggled',       self.background_changed)
        self.image_background.connect('toggled',       self.background_changed)
        self.transparent_background.connect('toggled', self.background_changed)

        self.background_saturation.connect('value-changed',     self.emit_changed)

        adjustment = gtk.Adjustment(0.5, 0.0, 1.0, 0.1, 0, 0)
        self.background_saturation.set_draw_value(False)
        self.background_saturation.set_adjustment(adjustment)

        i = 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Background:'))
        self.attach(label,                       1, 3, i, i+1)

        i += 1
        self.attach(self.solid_background,       1, 3, i, i+1)

        i += 1
        self.attach(self.image_background,       1, 3, i, i+1)
        
        i += 1
        self.attach(self.background_image_file,  1, 4, i, i+1)

        i += 1
        self.attach(self.transparent_background, 1, 3, i, i+1)

        i += 1
        label = gtk.Label()
        label.set_markup('<b>%s</b>' % _('Shade transparent or image background:'))
        label.set_alignment(0, 0.5)
        self.attach(label,                       1, 3, i, i+1)

        i += 1
        hbox = gtk.HBox()
        label1 = gtk.Label()
        label1.set_markup('<i>%s</i>' % _('None'))
        label1.set_alignment(1.0, 0.5)
        label2 = gtk.Label()
        label2.set_markup('<i>%s</i>' % _('Maximum'))
        label2.set_alignment(0, 0.5)

        hbox.pack_start(label1, expand=False)
        hbox.pack_start(self.background_saturation)
        hbox.pack_start(label2, expand=False)
        self.attach(hbox,                        1, 4, i, i+1)

    def background_changed(self, button):
        
        if self.solid_background.get_active():
            self.background_image_file.set_sensitive(False)
            self.background_saturation.set_sensitive(False)

        elif self.image_background.get_active():
            self.background_image_file.set_sensitive(True)
            self.background_saturation.set_sensitive(True)

        elif self.transparent_background.get_active():
            self.background_image_file.set_sensitive(False)
            self.background_saturation.set_sensitive(True)

        self.emit_changed()

    def browse_imagefile(self, button):

        filechooser.set_filter(filter)
        filechooser.show_all()
        filechooser.window.set_transient_for(mygtk.main_window)
        filechooser.set_modal(True)

        response = filechooser.run()
        if response == gtk.RESPONSE_OK:
            filename = filechooser.get_filename()
            self.background_image_file.set_text(filename)

            self.emit_changed()

        filechooser.destroy()

    def get_config(self):
        
        if config.get('terminal', 'background_image_file'):
            self.background_image_file.set_filename(config.get('terminal', 'background_image_file'))
            self.image_background.set_active(True)

        elif config.get('terminal', 'background_transparent'):
            self.transparent_background.set_active(True)

        else:
            self.solid_background.set_active(True)
 

        self.background_saturation.set_value(config.get('terminal', 'background_saturation'))

    def set_config(self):

        if self.solid_background.get_active():
            config.set('terminal', 'background_image_file',  None)
            config.set('terminal', 'background_transparent', False)

        elif self.image_background.get_active():
            config.set('terminal', 'background_image_file',  self.background_image_file.get_filename())
            config.set('terminal', 'background_transparent', False)

        elif self.transparent_background.get_active():
            config.set('terminal', 'background_image_file',  None)
            config.set('terminal', 'background_transparent', True)

        config.set('terminal', 'background_saturation', self.background_saturation.get_value())

SCROLL_RATIO    = float(318)/float(500)
MAX_LINES       = 100000
MAX_SCROLL_STEP = 200

class ScrollPreferences(Preferences):

    def __init__(self):
        
        Preferences.__init__(self, _('Scrolling'))

        self.show_scrollbar      = gtk.CheckButton(_('Show scrollbar'))
        self.scrollbar_position  = gtk.ComboBox(gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING))
        self.scrollback_lines    = gtk.SpinButton()
        self.scrollback_bytes    = gtk.SpinButton()
        self.scroll_step         = gtk.SpinButton()
        self.scroll_on_output    = gtk.CheckButton(_('Scroll on output'))
        self.scroll_on_keystroke = gtk.CheckButton(_('Scroll on keystroke'))

        self.lines_adjustment = gtk.Adjustment(0, 0, MAX_LINES,              1, 1, 0)
        self.lines_handlerId  = self.lines_adjustment.connect_after('value-changed', self.scroll_changed)

        self.bytes_adjustment = gtk.Adjustment(0, 0, MAX_LINES*SCROLL_RATIO, 1, 1, 0)
        self.bytes_handlerId  = self.bytes_adjustment.connect_after('value-changed', self.scroll_changed)

        self.construct_frame()
        self.get_config()

    def construct_frame(self):

        self.show_scrollbar.connect('toggled',      self.on_show_scrollbar_toggled)
        self.scrollbar_position.connect('changed',  self.emit_changed)
        self.scroll_on_output.connect('toggled',    self.emit_changed)
        self.scroll_on_keystroke.connect('toggled', self.emit_changed)

        cell = gtk.CellRendererText()
        self.scrollbar_position.pack_start(cell, True)
        self.scrollbar_position.add_attribute(cell, 'text', 0)

        self.scrollback_lines.set_adjustment(self.lines_adjustment)
        self.scrollback_lines.set_width_chars(len(str(MAX_LINES)))
        self.scrollback_lines.set_max_length(len(str(MAX_LINES)))

        self.scrollback_bytes.set_adjustment(self.bytes_adjustment)
        self.scrollback_bytes.set_width_chars(len(str(MAX_LINES*SCROLL_RATIO)))
        self.scrollback_bytes.set_max_length(len(str(MAX_LINES*SCROLL_RATIO)))

        adj = gtk.Adjustment(0, 0, MAX_SCROLL_STEP, 1, 1, 0)
        adj.connect('value-changed', self.emit_changed)

        self.scroll_step.set_adjustment(adj)
        self.scroll_step.set_width_chars(len(str(MAX_SCROLL_STEP)))
        self.scroll_step.set_max_length(len(str(MAX_SCROLL_STEP)))

        i = 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Scrolling'))
        self.attach(label,                    1, 3, i, i+1)

        i += 1
        self.attach(self.show_scrollbar,      1, 3, i, i+1)

        i += 1
        label = gtk.Label(_('Scrollbar position'))
        label.set_alignment(0, 0.5)
        self.attach(label,                    1, 2, i, i+1)
        self.attach(self.scrollbar_position,  2, 5, i, i+1)

        i += 1
        label = gtk.Label(_('Scrollback'))
        label.set_alignment(0, 0.5)
        self.attach(label,                    1, 2, i, i+1)
        self.attach(self.scrollback_lines,    2, 3, i, i+1)
        label = gtk.Label()
        label.set_markup('<i>%s</i>' % _('lines'))
        label.set_alignment(0, 0.5)
        self.attach(label,                    3, 4, i, i+1)
        self.attach(self.scrollback_bytes,    4, 5, i, i+1)
        label = gtk.Label()
        label.set_markup('<i>%s</i>' % _('kilobytes'))
        label.set_alignment(0, 0.5)
        self.attach(label,                    5, 6, i, i+1)

        i += 1
        label = gtk.Label(_('Scroll step'))
        label.set_alignment(0, 0.5)
        self.attach(label,                    1, 2, i, i+1)
        self.attach(self.scroll_step,         2, 3, i, i+1)
        label = gtk.Label()
        label.set_markup('<i>%s</i>' % _('lines'))
        label.set_alignment(0, 0.5)
        self.attach(label,                    3, 4, i, i+1)
        
        i += 1
        self.attach(self.scroll_on_output,    1, 3, i, i+1)

        i += 1
        self.attach(self.scroll_on_keystroke, 1, 3, i, i+1)

    def on_show_scrollbar_toggled(self, checkbutton):
        """
        Called when the show_scrollbar button is clicked
        """

        if checkbutton.get_active():
            self.scrollbar_position.set_sensitive(True)
        else:
            self.scrollbar_position.set_sensitive(False)

        self.emit_changed()

    def scroll_changed(self, adjustment):
        """
        Called when either bytes or lines are changed
        """

        if adjustment == self.lines_adjustment:

            # Calc bytes from lines
            lines = self.scrollback_lines.get_value_as_int()
            bytes = int(lines * SCROLL_RATIO)

            # We don't want to loop ;-)
            self.bytes_adjustment.handler_block(self.bytes_handlerId)
            self.scrollback_bytes.set_value(bytes)
            self.bytes_adjustment.handler_unblock(self.bytes_handlerId)

        elif adjustment == self.bytes_adjustment:

            # Calc lines from bytes
            bytes = self.scrollback_bytes.get_value_as_int()
            lines = int(bytes / SCROLL_RATIO)

            # We don't want to loop ;-)
            self.lines_adjustment.handler_block(self.lines_handlerId)
            self.scrollback_lines.set_value(lines)
            self.lines_adjustment.handler_unblock(self.lines_handlerId)

        self.emit_changed()

    def get_config(self):
        
        if config.get('terminal', 'show_scrollbar'):
            self.show_scrollbar.set_active(True)
        else:
            self.scrollbar_position.set_sensitive(False)

        model = self.scrollbar_position.get_model()
        model.clear()
        for display, position in ((_('Right side'), 'right'), (_('Left side'), 'left')):
            iter = model.append((display, position))
            if config.get('terminal', 'scrollbar_position') == position:
                self.scrollbar_position.set_active_iter(iter)

        self.scrollback_lines.set_value(config.get('terminal', 'scrollback_lines'))
        self.scroll_step.set_value(config.get('terminal', 'scroll_step'))
        self.scroll_on_output.set_active(config.get('terminal', 'scroll_on_output'))
        self.scroll_on_keystroke.set_active(config.get('terminal', 'scroll_on_keystroke'))

    def set_config(self):
        
        model = self.scrollbar_position.get_model()
        iter  = self.scrollbar_position.get_active_iter()
        
        config.set('terminal', 'show_scrollbar',      self.show_scrollbar.get_active())
        config.set('terminal', 'scrollbar_position',  model.get_value(iter, 1))
        config.set('terminal', 'scrollback_lines',    self.scrollback_lines.get_value_as_int())
        config.set('terminal', 'scroll_step',         self.scroll_step.get_value_as_int())
        config.set('terminal', 'scroll_on_output',    self.scroll_on_output.get_active())
        config.set('terminal', 'scroll_on_keystroke', self.scroll_on_keystroke.get_active())


class CompatPreferences(Preferences):

    def __init__(self):
        
        Preferences.__init__(self, _('Compatibility'))

        self.backspace_binding           = gtk.ComboBox(gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_INT))
        self.delete_binding              = gtk.ComboBox(gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_INT))
        self.use_prompt_for_window_title = gtk.CheckButton(_('Use prompt command for window title'))
        self.use_prompt_for_tab_label    = gtk.CheckButton(_('Use prompt command for tab label'))
        self.prompt_command              = gtk.Entry()

        self.construct_frame()
        self.get_config()

    def construct_frame(self):

        self.backspace_binding.connect('changed',           self.emit_changed)
        self.delete_binding.connect('changed',              self.emit_changed)
        self.use_prompt_for_window_title.connect('toggled', self.emit_changed)
        self.use_prompt_for_tab_label.connect('toggled',    self.emit_changed)
        self.prompt_command.connect('changed',              self.emit_changed)
        
        cell = gtk.CellRendererText()
        self.backspace_binding.pack_start(cell, True)
        self.backspace_binding.add_attribute(cell, 'text', 0)

        cell = gtk.CellRendererText()
        self.delete_binding.pack_start(cell, True)
        self.delete_binding.add_attribute(cell, 'text', 0)

        i = 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Compatibility:'))
        self.attach(label,                            1, 3, i, i+1)

        i += 1
        label = gtk.Label(_('Backspace key generates:'))
        label.set_alignment(0, 0.5)
        self.attach(label,                            1, 2, i, i+1)
        self.attach(self.backspace_binding,           2, 3, i, i+1)

        i += 1
        label = gtk.Label(_('Delete key generates:'))
        label.set_alignment(0, 0.5)
        self.attach(label,                            1, 2, i, i+1)
        self.attach(self.delete_binding,              2, 3, i, i+1)

        i += 1
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Prompt command:'))
        self.attach(label,                            1, 3, i, i+1)

        i += 1
        self.attach(self.use_prompt_for_window_title, 1, 3, i, i+1)

        i += 1
        self.attach(self.use_prompt_for_tab_label,    1, 3, i, i+1)

        i += 1
        label = gtk.Label(_('Prompt command:'))
        label.set_alignment(0, 0.5)
        self.attach(label,                            1, 2, i, i+1)
        self.attach(self.prompt_command,              2, 3, i, i+1)

    def get_config(self):

        erase_sequences = (_('Auto'),
                           _('ASCII Backspace'),
                           _('ADCII Delete'),
                           _('Delete sequence'))

        model = self.backspace_binding.get_model()
        model.clear()
        i = 0
        for sequence in erase_sequences:
            iter = model.append((sequence, i))
            if config.get('terminal', 'backspace_binding') == i:
                self.backspace_binding.set_active_iter(iter)
            i += 1

        model = self.delete_binding.get_model()
        model.clear()
        i = 0
        for sequence in erase_sequences:
            iter = model.append((sequence, i))
            if config.get('terminal', 'delete_binding') == i:
                self.delete_binding.set_active_iter(iter)
            i += 1

        self.use_prompt_for_window_title.set_active(config.get('terminal', 'use_prompt_for_window_title'))
        self.use_prompt_for_tab_label.set_active(config.get('terminal', 'use_prompt_for_tab_label'))
        self.prompt_command.set_text(config.get('terminal', 'prompt_command'))

    def set_config(self):
        
        model = self.backspace_binding.get_model()
        iter  = self.backspace_binding.get_active_iter()
        config.set('terminal', 'backspace_binding', model.get_value(iter, 1))

        model = self.delete_binding.get_model()
        iter  = self.delete_binding.get_active_iter()
        config.set('terminal', 'delete_binding', model.get_value(iter, 1))

        config.set('terminal', 'use_prompt_for_window_title', self.use_prompt_for_window_title.get_active())
        config.set('terminal', 'use_prompt_for_tab_label',    self.use_prompt_for_tab_label.get_active())
        config.set('terminal', 'prompt_command',              self.prompt_command.get_text())


def get_stock_items():
    """
    Helper to retreive all Stock Itens from GTK
    """

    stock_ids = gtk.stock_list_ids()
    stock_ids.sort()

    stock_items = []
    for stock_id in stock_ids:
        stock_item = gtk.stock_lookup(stock_id)
        if stock_item:
            stock_items.append(stock_item)

    return stock_items

class ShortcutDialog(gtk.Dialog):

    def __init__(self):
        
        gtk.Dialog.__init__(self, title=_('Shortcut configuration'), flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                                  buttons=(gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))

        self.set_icon_from_file(config.APP_ICON_FILEPATH)

        self.en_shortcut = gtk.Entry()
        self.modifiers   = []
        self.key         = ''

        self.connect('event', self.on_event)

        self.construct_frame()

    def construct_frame(self):

        self.en_shortcut.set_sensitive(False)

        hbox = gtk.HBox()
        hbox.set_spacing(5)
        hbox.set_border_width(5)

        label = gtk.Label(_('Type a shortcut'))
        hbox.pack_start(label, expand=False)
        hbox.pack_start(self.en_shortcut)

        self.vbox.pack_start(hbox)

        self.show_all()

    def on_event(self, me, event):

        if self.key:
            return

        if event.type == gtk.gdk.KEY_PRESS:

            keyval_name = gtk.gdk.keyval_name(event.keyval)
            keyval_name = keyval_name.split('_')[0]
            keyval_name = keyval_name.capitalize()

            if keyval_name == 'Return':
                return

            if len(keyval_name) == 1:
                self.key = keyval_name
            else:
                self.modifiers.append(keyval_name)

        elif event.type == gtk.gdk.KEY_RELEASE:
            keyval_name = gtk.gdk.keyval_name(event.keyval)
            keyval_name = keyval_name.split('_')[0]

            if not len(keyval_name) == 1 and not self.key:
                self.modifiers.remove(keyval_name)
                
        shortcut = self.modifiers[:]
        if self.key:
            shortcut.append(self.key)
            
        self.en_shortcut.set_text('+'.join(shortcut))



                         # Name                 # Renderer              # Renderer attributes  # col_id  # visibility # sort_id
COMMANDS_TV_COLUMNS = ( ('stock_id',            gtk.CellRendererText,   (),                    0,        False,       0),
                        (_('Icon'),             gtk.CellRendererPixbuf, (),                    1,        True,        1),
                        (_('Label'),            gtk.CellRendererText,   (),                    2,        True,        2),
                        (_('Command'),          gtk.CellRendererText,   (),                    3,        True,        3) )


class CommandPreferences(Preferences):

    def __init__(self):
        
        Preferences.__init__(self, _('Commands'))

        model = gtk.ListStore(gobject.TYPE_STRING,  # stock_id
                              gtk.gdk.Pixbuf,       # Icon
                              gobject.TYPE_STRING,  # Label
                              gobject.TYPE_STRING,  # Command
                              gobject.TYPE_STRING,  # Tooltip
                              gobject.TYPE_STRING)  # Accelerator

        self.tv_commands = gtk.TreeView(model)

        model = gtk.ListStore(gobject.TYPE_STRING,  # stock_id
                              gtk.gdk.Pixbuf,       # Icon
                              gobject.TYPE_STRING)  # Label

        self.stock_ids   = []
        self.cb_icon     = gtk.ComboBox(model)
        self.en_label    = gtk.Entry()
        self.bt_shortcut = gtk.Button(label='')
        self.en_tooltip  = gtk.Entry()
        self.en_command  = gtk.Entry()

        # Buttons
        self.bt_remove_command = gtk.Button(stock=gtk.STOCK_REMOVE)
        self.bt_add_command    = gtk.Button(stock=gtk.STOCK_ADD)

        self.feed_objects()
        self.construct_frame()
        self.get_config()

    def construct_frame(self):

        # Commands
        self.tv_commands.set_rules_hint(True)
        self.tv_commands.set_headers_visible(False)
        self.tv_commands.set_headers_clickable(True)
        self.tv_commands.set_size_request(-1, 100)
        self.tv_commands.connect('cursor-changed', self.on_tv_commands_cursor_changed)

        mygtk.add_columns_to_treeview(self.tv_commands, COMMANDS_TV_COLUMNS)

        sw = gtk.ScrolledWindow()
        sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        sw.add(self.tv_commands)

        # Icons
        renderer = gtk.CellRendererPixbuf()
        self.cb_icon.pack_start(renderer, expand=False)
        self.cb_icon.add_attribute(renderer, 'pixbuf', 1)

        renderer = gtk.CellRendererText()
        self.cb_icon.pack_start(renderer)
        self.cb_icon.add_attribute(renderer, 'text',   2)

        # Buttons
        hb_control = gtk.HButtonBox()
        hb_control.set_layout(gtk.BUTTONBOX_END)
        hb_control.set_spacing(10)

        self.bt_remove_command.connect("clicked", self.on_bt_remove_command_clicked)
        hb_control.pack_start(self.bt_remove_command)

        self.bt_add_command.connect("clicked",    self.on_bt_add_command_clicked)
        hb_control.pack_start(self.bt_add_command)

        # Disabling widgets
        self.bt_remove_command.set_sensitive(False)
        self.cb_icon.set_sensitive(False)
        self.en_label.set_sensitive(False)
        self.bt_shortcut.set_sensitive(False)
        self.en_tooltip.set_sensitive(False)
        self.en_command.set_sensitive(False)

        # Signal connexions
        self.cb_icon.connect('changed',     self.on_something_changed)
        self.en_label.connect('changed',    self.on_something_changed)
        self.bt_shortcut.connect('clicked', self.on_bt_shortcut_clicked)
        self.en_tooltip.connect('changed',  self.on_something_changed)
        self.en_command.connect('changed',  self.on_something_changed)

        i = 0
        label = gtk.Label()
        label.set_alignment(0, 0.5)
        label.set_markup('<b>%s</b>' % _('Commands:'))
        self.attach(label,            0, 4, i, i+1)

        i += 1
        self.attach(sw,               0, 4, i, i+1)

        i += 1
        label = gtk.Label(_('Icon'))
        label.set_alignment(0, 0.5)
        self.attach(label,            0, 1, i, i+1)
        self.attach(self.cb_icon,     1, 2, i, i+1)

        label = gtk.Label(_('Label'))
        label.set_alignment(0, 0.5)
        self.attach(label,            2, 3, i, i+1)
        self.attach(self.en_label,    3, 4, i, i+1)

        i += 1
        label = gtk.Label(_('Shortcut'))
        label.set_alignment(0, 0.5)
        self.attach(label,            0, 1, i, i+1)
        self.attach(self.bt_shortcut, 1, 2, i, i+1)

        label = gtk.Label(_('Tooltip'))
        label.set_alignment(0, 0.5)
        self.attach(label,            2, 3, i, i+1)
        self.attach(self.en_tooltip,  3, 4, i, i+1)

        i += 1
        label = gtk.Label(_('Command'))
        label.set_alignment(0, 0.5)
        self.attach(label,            0, 1, i, i+1)
        self.attach(self.en_command,  1, 4, i, i+1)

        i += 1
        label = gtk.Label()
        label.set_markup('<b><i>%s</i></b>' % _('Note that you must restart %s in order to apply any changes.') % config.PACKAGE)
        label.set_alignment(1, 0.5)
        self.attach(label,            0, 4, i, i+1)

        i += 1
        self.attach(hb_control,       0, 4, i, i+1)

        
    def feed_objects(self):

        # Get all the stock items
        stock_items = get_stock_items()
        
        model = self.cb_icon.get_model()
        model.clear()

        for stock_item in stock_items:
            stock_id = stock_item[0]
            label    = stock_item[1].replace('_', '')

            pixbuf = self.cb_icon.render_icon(stock_id=stock_id,
                                              size=gtk.ICON_SIZE_MENU,
                                              detail=None)

            self.stock_ids.append(stock_id)
            model.append((stock_id,
                          pixbuf,
                          label))

    def get_current_command(self):
        
        path  = self.tv_commands.get_cursor()[0]
        model = self.tv_commands.get_model()
        iter  = model.get_iter(path)

        if iter:
            # Getting the values from the TreeModel
            stock_id = model.get_value(iter, 0)
            label    = model.get_value(iter, 2)
            command  = model.get_value(iter, 3)
            tooltip  = model.get_value(iter, 4)
            shortcut = model.get_value(iter, 5)

            return (stock_id, label, shortcut, tooltip, command)

        return None

    def on_bt_remove_command_clicked(self, button):
        """
        Remove the selected command from the List
        """
        
        selection   = self.tv_commands.get_selection()
        model, iter = selection.get_selected()

        if iter:
            path = model.get_path(iter)[0]
            model.remove(iter)

        # Disabling widgets
        self.bt_remove_command.set_sensitive(False)
        self.cb_icon.set_sensitive(False)
        self.en_label.set_sensitive(False)
        self.bt_shortcut.set_sensitive(False)
        self.en_tooltip.set_sensitive(False)
        self.en_command.set_sensitive(False)

        self.emit_changed()

    def on_bt_add_command_clicked(self, button):
        """
        Add a new command to the list
        """

        model  = self.tv_commands.get_model()

        stock_id = gtk.STOCK_NEW
        pixbuf   = self.tv_commands.render_icon(stock_id=stock_id,
                                                size=gtk.ICON_SIZE_MENU,
                                                detail=None)

        model.append((stock_id,
                      pixbuf,
                      _('Unconfigured command'),
                      'echo ' + _('Unconfigured command'),
                      _('Unconfigured command'),
                      str(None)))

        self.emit_changed()

    def on_tv_commands_cursor_changed(self, treeview):

        # Enabling widgets
        self.bt_remove_command.set_sensitive(True)
        self.cb_icon.set_sensitive(True)
        self.en_label.set_sensitive(True)
        self.bt_shortcut.set_sensitive(True)
        self.en_tooltip.set_sensitive(True)
        self.en_command.set_sensitive(True)

        # Display the command
        command = self.get_current_command()

        if command:
            self.cb_icon.set_active(self.stock_ids.index(command[0]))
            self.en_label.set_text(command[1])
            
            shortcut = command[2]

            if shortcut and not shortcut == 'None':
                r    = re.compile('<(\w+)>')
                key  = shortcut[-1]
                mods = r.findall(shortcut[:-1])
                shortcut = ''
                for mod in mods:
                    shortcut += '%s+' % mod.capitalize()
                shortcut += key

            self.bt_shortcut.set_label(shortcut)

            self.en_tooltip.set_text(command[3])
            self.en_command.set_text(command[4])

        else:
            self.cb_icon.set_active(-1)
            self.en_label.set_text()
            self.bt_shortcut.set_label()
            self.en_tooltip.set_text()
            self.en_command.set_text()

    def on_something_changed(self, *args):

        path  = self.tv_commands.get_cursor()[0]
        model = self.tv_commands.get_model()
        iter  = model.get_iter(path)

        if not iter:
            return False

        index = self.cb_icon.get_active()
        if not index == -1:
            stock_id = self.stock_ids[index]
            pixbuf = self.tv_commands.render_icon(stock_id=stock_id,
                                                  size=gtk.ICON_SIZE_MENU,
                                                  detail=None)
        else:
            stock_id = ''
            pixbuf   = None

        label    = self.en_label.get_text()
        command  = self.en_command.get_text()
        tooltip  = self.en_tooltip.get_text()
        shortcut = self.bt_shortcut.get_label()
        
        if not shortcut == 'None':
            shortcuts = shortcut.split('+')
            shortcut  = ''
            for key in shortcuts[:-1]:
                shortcut += '<%s>' % key.lower()

            shortcut += shortcuts[-1]

        model.set(iter, 0, stock_id,
                        1, pixbuf,
                        2, label,
                        3, command,
                        4, tooltip,
                        5, shortcut)

        self.emit_changed()

    def on_bt_shortcut_clicked(self, button):

        shortcut = button.get_label()
        dialog   = ShortcutDialog()
        response = dialog.run()
        dialog.destroy()

        if response == gtk.RESPONSE_OK:
            shortcut = dialog.en_shortcut.get_text()
            self.bt_shortcut.set_label(shortcut)
            self.on_something_changed()

    def get_config(self):
        """
        Insert all commands from config to list
        """

        model = self.tv_commands.get_model()
        model.clear()

        commands = config.get('terminal', 'commands')
        if not commands:
            return True

        for stock_id, label, shortcut, tooltip, command in commands:
            pixbuf = self.tv_commands.render_icon(stock_id=stock_id,
                                                  size=gtk.ICON_SIZE_MENU,
                                                  detail=None)

            model.append((stock_id,
                          pixbuf,
                          label,
                          command,
                          tooltip,
                          str(shortcut)))

    def set_config(self):
        """
        Get commands list from Treeview and store it to config
        """

        model = self.tv_commands.get_model()
        iter  = model.get_iter_first()

        commands = []
        while iter:
            # Getting the values from the TreeModel
            stock_id = model.get_value(iter, 0)
            label    = model.get_value(iter, 2)
            command  = model.get_value(iter, 3)
            tooltip  = model.get_value(iter, 4)
            shortcut = model.get_value(iter, 5)
            if shortcut == 'None':
                shortcut = None

            commands.append((stock_id, label, shortcut, tooltip, command))
            iter = model.iter_next(iter)

        config.set('terminal', 'commands', commands)


class TerminalPreferences(gtk.Dialog):

    def __init__(self, load_config_callback):

        gtk.Dialog.__init__(self, flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT, buttons=(gtk.STOCK_CLOSE, gtk.RESPONSE_CANCEL))

        self.set_title('%s - %s' % (config.PACKAGE, _('Terminal preferences')))
        self.set_resizable(False)

        self.load_config_callback = load_config_callback
        self.notebook = gtk.Notebook()

        self.construct_frame()
        self.connect('destroy', self.on_destroy)

    def construct_frame(self):

        table = gtk.Table(3,3)
        pref = GeneralPreferences()
        pref.connect('something-changed', self.something_changed)
        table.attach(pref, 1,2,1,2, gtk.SHRINK, gtk.SHRINK)
        self.notebook.append_page(table, pref.label)

        table = gtk.Table(3,3)
        pref = ColorPreferences()
        pref.connect('something-changed', self.something_changed)
        table.attach(pref, 1,2,1,2, gtk.SHRINK, gtk.SHRINK)
        self.notebook.append_page(table, pref.label)

        table = gtk.Table(3,3)
        pref = EffectPreferences()
        pref.connect('something-changed', self.something_changed)
        table.attach(pref, 1,2,1,2, gtk.SHRINK, gtk.SHRINK)
        self.notebook.append_page(table, pref.label)

        table = gtk.Table(3,3)
        pref = ScrollPreferences()
        pref.connect('something-changed', self.something_changed)
        table.attach(pref, 1,2,1,2, gtk.SHRINK, gtk.SHRINK)
        self.notebook.append_page(table, pref.label)

        table = gtk.Table(3,3)
        pref = CompatPreferences()
        pref.connect('something-changed', self.something_changed)
        table.attach(pref, 1,2,1,2, gtk.SHRINK, gtk.SHRINK)
        self.notebook.append_page(table, pref.label)

        table = gtk.Table(3,3)
        pref = CommandPreferences()
        pref.connect('something-changed', self.something_changed)
        table.attach(pref, 1,2,1,2, gtk.SHRINK, gtk.SHRINK)
        self.notebook.append_page(table, pref.label)

        self.vbox.pack_start(self.notebook)

    def something_changed(self, pref):
        
        # Retrieving the config from all tabs
        page_num = self.notebook.get_n_pages()
        for i in range(0, page_num):
            child_pref = self.notebook.get_nth_page(i).get_children()[0]
            if child_pref:
                child_pref.set_config()

        # Applying the change
        self.load_config_callback()

    def on_destroy(self, dialog):
        
        # Write the config
        if not config.write():
            # FIXME: onpopup
            pass
