#!/usr/bin/env python
#
# GEemyle: WebBrowser module
# Copyright (C) 2003-2006 Easter-eggs <info@easter-eggs.com>
# Authors:
#      Cyril Lacoux (Yack) <clacoux@easter-eggs.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

__author__  = "Cyril Lacoux [Yack] <clacoux@easter-eggs.com>"
__cvsid__   = "$Id$"
__version__ = "$Revision$"[11:-2]
__all__     = ["open_uri", "mail_to", "phone_call", "send_fax"]

import os
import thread
import config
import telnetlib

def asterisk_dialing(phone_number):
    """
    Request a dial to Asterisk server
    """

    # Connection
    manager = telnetlib.Telnet(config.get('geemyle', 'asterisk_host'),
                               config.get('geemyle', 'asterisk_port'))

    index, match, text = manager.expect(['Asterisk Call Manager/.*', ], 1)
    if not match:
        manager.close()
        return False

    # Authentication
    manager.write('Action: Login\r\n')
    manager.write('UserName: %s\r\n'    % config.get('geemyle', 'asterisk_user'))
    manager.write('Secret: %s\r\n'      % config.get('geemyle', 'asterisk_pass'))
    manager.write('\r\n')

    index, match, text = manager.expect(['Response: Success\r\nMessage: Authentication accepted\r\n\r\n', ], 1)
    if not match:
        manager.close()
        return False

    # Dial
    manager.write('Action: Originate\r\n')
    manager.write('Channel: %s\r\n'     % config.get('geemyle', 'asterisk_ext'))
    manager.write('Context: %s\r\n'     % config.get('geemyle', 'asterisk_ctx'))
    manager.write('Exten: %s\r\n'       % phone_number.replace('.', ''))
    manager.write('Priority: 1\r\n')
    manager.write('Callerid: %s\r\n'    % phone_number.replace('.', ''))
    manager.write('\r\n')

    index, match, text = manager.expect(['Response: Success\r\nMessage: Originate successfully', ], 10)
    if not match:
        manager.close()
        return False

    manager.close()
    return True
    
def open_uri(uri):
    """
    Open an URI
    """
    if not uri:
        return False

    try:
        cmd = '%s >/dev/null 2>&1' % config.get('geemyle', 'browser_command') % uri
    except:
        return False

    thread.start_new_thread(os.system, (cmd, ))
    return True

def mail_to(mail_address):
    """
    Open a mailer
    """
    if not mail_address:
        return False

    try:
        cmd = '%s >/dev/null 2>&1' % config.get('geemyle', 'mail_command') % mail_address
    except:
        return False

    thread.start_new_thread(os.system, (cmd, ))
    return True

def phone_call(phone_number):
    """
    Start a phone call
    """
    if not phone_number:
        return False

    thread.start_new_thread(asterisk_dialing, (phone_number, ))
    return True

def send_fax(fax_number):
    """
    Send a fax
    """
    if not fax_number:
        return False

    try:
        cmd = '%s >/dev/null 2>&1' % config.get('geemyle', 'fax_command') % fax_number
    except:
        return False

    thread.start_new_thread(os.system, (cmd, ))
    return True

